/** @file sp_dplpexecutor.h Executor with IO device  */

/* 
   FAU Discrete Event Systems Library (libfaudes)

   Copyright (C) 2008  Thomas Moor

*/

// load configuration
#include "corefaudes.h"

#ifndef FAUDES_SP_DPLPEXECUTOR_H
#define FAUDES_SP_DPLPEXECUTOR_H


#include "tp_include.h"
#include "sp_plpexecutor.h" 
#include "sp_simeventset.h"
#include <ctime>


// use iodevice or dummy if not available
#ifdef FAUDES_PLUGIN_IODEVICE
#include "iop_include.h"
#else
namespace faudes {
/** Dummy typedef in the absence of the IO Device plugin */
typedef void vDevice; 
}
#endif

namespace faudes {
	
/**
 * Executer with IO device to handle external/physical events
 *
 * \section SecSimulatorDPLPEX1 External/Physical Events and Time
 *
 * This executor class is derived from the ProposingExecutor and uses a vDevice
 * from the IO Device Plugin to handle sensor and actuator events and physical time. 
 * Technically, the class provides the routine SyncStep() that has to be called periodically to
 * synchronize executor time with physical time and to perform sensor readings and
 * actuator writings. There is also a convenience routine SyncRun() which runs a loop
 * with SyncStep().
 *
 * The SyncStep() procedure implements the below stages. It returns true, if indeed
 * a transition was executed. It returns false on if synchronistion fails or the specified
 * duration expired.
 *    
 *  -  get a proposal from ProposingEcexutor
 *  -  if physical time is ahead of the genartors current time, sync time by
 *      ExecuteTime; in the case that this is not consistent with the proposal, 
 *      an error is reported;
 *  -  if the generators current time is ahead of physical time, an error is reported
 *  -  if sensor events have been reported, execute them now
 *  -  if the proposal schedules an event for the generators current time, execute it
 *  -  if the proposals time is not yet executed, wait for that amount of time to pass
 *       or a sensor event to be delivered.
 *
 *
 * Implementation note: if multiple sensor events are available, they will be all executed.
 * This may change in a future version.
 *
 *
 * Naturally, the DeviceExecutor requires the IO Device plugin in order to be functional.
 * In the absence of the plugin, the DeviceExecutor will behave like a ProposingExecutor.
 *
 * \section SecSimulatorLPEX3 File IO
 *
 * The DeviceExecutor inherits file IO from the ProposingExecutor. The device is initialized
 * (eg configured from File) and then passed to the DeviceExecutor by the method Devicep().
 *
 * @ingroup SimulatorPlugin 
 */

class DeviceExecutor : public ProposingExecutor {  

 public:
		
  /*****************************************
   *****************************************
   *****************************************
   *****************************************/

  /** @name Constructors & Destructor */
  /** @{ doxygen group */

  /**
   * Creates an emtpy DeviceExecutor
   */
  DeviceExecutor();

  /**
   * Explicit destructor
   */
  ~DeviceExecutor();


  /** @} doxygen group */

  /*****************************************
   *****************************************
   *****************************************
   *****************************************/

  /** @name Re-implemenented from ParallelExecutor */
  /** @{ doxygen group */

  /**
   * Reset the DeviceExecutor.
   *
   * Reset the executor to its initial state and reset the device, ie clear queued
   * sensor event set actuators to a passive state
   *
   * @param seed
   *     Seed for PropossingExecutor random generator, 0<>system time
   */
  virtual void Reset(long int seed=0);

  /**
   * Clear all data (generators, simulation attributes etc)
   *
   * This includes the "HardwareReset" event.
   *
   */
  virtual void Clear(void);


  /**
   * Execute event.
   *
   * Programmatically override any internal schedules and execute the specified event.
   * This routine will does neither synchronize generator time nor events.
   *
   * @param event
   *   Event by index
   * @return 
   *  	 True on success
   */
  bool ExecuteEvent(Idx event);
		

  /** @} doxygen group */

  /** @name Application Interface */
  /** @{ doxygen group */

  /**
   * Execute generator time to sync with device time.
   *
   * If possible, execute the amount of time required for an
   * exact match. Otherwise, accept the specified tolerance. As a
   * last resort, issue a sync error. 
   *
   * @return
   *   True, on success ie synchron within tolerance.
   */
  bool SyncTime(void);

  /**
   * Execute scheduled or sensor events now
   *
   * If sensor events are ready, execute one of them.
   * Otherwise execute any event scheduled for now.
   * Otherwise execute nothing.
   *
   * @return
   *   Idx of event executed, 0 for no execution or error
   */
  Idx SyncEvents();


  /**
   * Wait for sensor events
   *
   * Wait the specified amount of time, for the proposed time to elaps,
   * or a sensor evemnt to occur - whatever comes first.
   * This function will *not* synchronise with generator time. You may 
   * call SyncTime afterwards.
   *
   * @param duration
   *   Max duration to wait for
   * @return
   *   True, if sensor events are available
   */
  bool SyncWait(tpTime::Type duration=tpTime::Max);

  /**
   * Execute one transition with synchronous physical signals.
   *
   * Calls SyncTime, SyncEvents and SyncWait to execute one transition.
   * It will, however, not pass more than the specified duration. 
   *
   * @param duration
   *   Max duration of execution wrt generator time, tpTime::Max for unlimited
   * @return
   *   Idx of executed event or 0 for time out or error
   */
  Idx SyncStep(tpTime::Type duration=tpTime::Max);

  /**
   * Run execution with synchronous physical signals.
   *
   * Loops SyncStep until the specified amount on time expired.
   *
   * @param duration
   *   Duration of execution wrt generator time, tpTime::Max for infinite
   * @return
   *  True, for no error
   */
  bool SyncRun(tpTime::Type duration=tpTime::Max);

  /**
   * Test Syncronisation
   *
   * @return True, if device time and events are in sync with
   *   generator time and events.
   *
   */
  bool IsSynchronous(void) const { return ! mSyncError; };

  /**
   * Set tolerance for time synchonisation.
   *
   * @param maxgap
   *   Max acceptable amount of time by which the generator may be behind physical time
   *
   */
  void Tolerance(tpTime::Type maxgap) {mMaxSyncGap=maxgap;};

  /**
   * Set device.
   *
   * The device must be configured. You must start
   * the device befor the first call to SyncRun. Ownership of 
   * the device stays with the caller.
   *
   * @param dev
   *   IO Device to use
   *
   */
  void Devicep(vDevice* dev);

  /**
   * Convenience: Start the device.
   *
   */
  void DeviceStart(void);

  /**
   * Convenience: Stop the device.
   *
   */
  void DeviceStop(void);

  /** @} doxygen group */

 protected:


  /** Sync error flag */
  bool mSyncError;

  /** Max gap between physical and generator time  */
  tpTime::Type mMaxSyncGap;
 
  /** Device reference */
  vDevice* pDevice;



}; // end class DeviceExecutor



} // namespace faudes


#endif // .h

