/** @file sp_simconditionset.h  Set of named simulation conditions */

/*
   Copyright (C) 2008  Thomas Moor
   Exclusive copyright is granted to Klaus Schmidt
*/


#include "tokenreader.h"
#include "tokenwriter.h"
#include "nameset.h"
#include "attributes.h"
#include "sp_densityfnct.h"


#ifndef FAUDES_SP_SIMCONDITIONSET_H
#define FAUDES_SP_SIMCONDITIONSET_H

namespace faudes {

/** 

@defgroup SimConditionAttributes Simulation Condition Attributes 

@ingroup SimulatorPlugin 

Attributes to define simulation conditions used for statistic analysis
of simulation runs.

@copydoc faudes::SimConditionAttribute

@{ 

*/

/**  
 * Defining data of event based simulation condition
 *
 * An event based condition has a set of start events and a set of
 * stop events. It becomes satisfied when a start event occurs and it 
 * becomes unsatisfied when a stop event occurs. Thus, when start or stop
 * events fail to alternate, the first event of the respective type is relevant.
 *
 */
class SimEventConditionAttribute {
 public:
  EventSet mStart;
  EventSet mStop;
};

/**  
 * Defining data of state based simulation condition
 *
 * A state based condition has a set of states per generator an is satisfied
 * while the components of the current discrete state vector are within the
 * respective set. A flag indicates whether each or some component 
 * must comply with the state set to satisfy the condition.
 * 
 * Note: Since state indices are local to generators, a SimStateConditionAttribute 
 * is only meaningful in the context of a particular ParallelExecutor. Hence, it is
 * advisable to use that ParallelExecutor for file io of conditions.
 *
 */

// state based condition
class  SimStateConditionAttribute {
public:
  bool mAllFlag;
  std::vector<StateSet> mStateSets;
};


/**
 * Attribute for a simulation condition
 *
 * In order to extract statistical data from a simulation by a LoggingExecutor
 * or some derived class, so called simulation conditions
 * are defined. At any instance of time, a condition is satisfied or
 * dissatisfied. Statistical data can then be requested regarding 
 * the period and duration of each condition. Currently, two
 * types of conditions are available:
 *
 * - SimStateConditionAttribute addresses conditions of the current state
 * - SimEventConditionAttribute addresses the external event sequence.
 *
 * A Condition may be flagged as a break condition to halt simulation
 * when satisfied. A condition may be enabled for tracking or not.
 *
 * The class SimConditionAttribute summarizes all data to represent
 * faudes simulation conditions. It also holds some state of the condition
 * wrt execution and provides an interface for sampling. The latter 
 * may be seperated to a different class in a future revision. The class
 * SimConditionAttribute does, however, not implement any test whether
 * or not a condition is satisfied. This is done by the LoggingExecutor.
 *
 * As a faudes attribute, conditions can be referenced by 
 * names the via std faudes container TaNameSet. Token io should be done
 * in the context of a particular ParallelExecutor using ParallelExecutor methods.
 */

class SimConditionAttribute : public AttributeVoid {

  public:

  /** Convenience typedef */
  typedef std::vector<StateSet>::iterator Iterator;
  typedef std::vector<StateSet>::const_iterator CIterator;

  /** Default constructor */
  SimConditionAttribute(void) : AttributeVoid() { 
    mEventCondition=false; mStateCondition=false; mEnabled=true; mBreakCondition=false; Reset();};

  /** Test for default value */
  bool IsDefault(void) const {
    return !mEventCondition && !mStateCondition && mEnabled && !mBreakCondition; };

  /** Does this attribute define a state condition? */
  bool IsStateCondition(void) const {return mStateCondition; };

  /** Does this attribute define a event condition? */
  bool IsEventCondition(void) const {return mEventCondition; };

  /** Does this attribute define a break condition? */
  bool IsBreakCondition(void) const {return mBreakCondition; };

  /** Is this condition active? */
  bool IsEnabled(void) const {return mEnabled; };

  /** Set state condition attribute */
  void StateCondition(const SimStateConditionAttribute&  rStateConditionAttribute) 
    {mStateConditionAttribute=rStateConditionAttribute; mStateCondition=true; mEventCondition=false;
    for(Idx i=0; i<mStateConditionAttribute.mStateSets.size(); i++) 
      mStateConditionAttribute.mStateSets.at(i).Name("StateSet");
  };

  /** Set event condition attribute */
  void EventCondition(const SimEventConditionAttribute&  rEventConditionAttribute) 
    {mEventConditionAttribute=rEventConditionAttribute; mEventCondition=true; mStateCondition=false;};

  /** Set break flag */
  void BreakCondition(bool on) {mBreakCondition=on; };

  /** Set enabled flag */
  void Enabled(bool on) {mEnabled=on; };

  /**  Inspect event condition */
  const SimEventConditionAttribute& EventCondition(void) const {return mEventConditionAttribute; };

  /**  Inspect state condition */
  const SimStateConditionAttribute& StateCondition(void) const {return mStateConditionAttribute; };

  /** Reset conditions execution  state */
  void Reset(void) 
  { mSatisfied=false; mActivationTime=-1; mSamplesPeriod.Clear(); mSamplesDuration.Clear(); };

  /** Test whether the condition is recorded to be satisfied */
  // condition state: get satisfied 
  bool Satisfied(void) const { return mSatisfied; };

  /** Indicate that the condition became satisfied/dissatisfied at the specifie time */
  void Satisfied(bool on, tpTime::Type now);
   
  /** Condition state: sampled condition periods */
  SampledDensityFunction mSamplesPeriod;

  /** Condition state: sampled condition durations */
  SampledDensityFunction mSamplesDuration;

protected:
 
  /** Indicate precense of a event condition */
  bool mEventCondition;

  /** Indicate precense of a state condition */
  bool mStateCondition;

  /** Indicate that we halt simulation when this condition is satisfied */
  bool mBreakCondition;

  /** Indicate that this condition should be considere at all */
  bool mEnabled;

  /** Event based condotion data */
  SimEventConditionAttribute mEventConditionAttribute;

  /** State based condotion data */
  SimStateConditionAttribute mStateConditionAttribute;

  /** Condotion state: recorded as satisfied */
  bool mSatisfied;

  /** Condition state: when last satisfied became true */
  tpTime::Type mActivationTime;
   

 protected:

  /**
   * Reads the attribute from TokenReader, see AttributeVoid for public wrappers.
   * 
   * If the current token indicates a condition section, the method reads the  
   * condition data from that section. Else it does nothing. Exceptions may only be thrown
   * on invalid data within the condition section. The label argiment is ignored, we 
   * use hardcoded labled "EventCondition" and "StateCondition" to figure the type of 
   * condition. The context argument may be provided to use the appropriate state symbols
   * for stateconditions. 
   *
   * @param rTr
   *   TokenReader to read from
   * @param rLabel
   *   Section to read
   * @param pContext
   *   Read context to provide contextual information
   *
   * @exception Exception
   *   - IO error (id 1)
   */
  virtual void DoRead(TokenReader& rTr, const std::string& rLabel="", const Type* pContext=0);
 
  /**
   * Writes the attribute to TokenWriter, see AttributeVoid for public wrappers.
   *
   * Writes a condition section to include data on state- or event-condition. The label argument
   * is ignored, we use hardcoded keywords "EventCondition" and StateCondition". The context
   * argument is used to provide appropriate state symbol tables.
   *
   * @param rTw
   *   TokenWriter to write to
   * @param rLabel
   *   Section to write
   * @param pContext
   *   Read context to provide contextual information
   *
   * @exception Exception
   *   - IO error (id 2)
   */
  virtual void DoWrite(TokenWriter& rTw,const std::string& rLabel="", const Type* pContext=0) const;



}; // class SimConditionAttribute


/**
 * Set of simulation named conditions 
 *
 * Note: we currently share a symboltabel with the global
 * event set. This will definitely change in a future revision.
 *
 * Note: since we currently dont implement state based conditions
 * we still dont have a ParallelExecutor reference for token io.
 * This will definitely change in a future revision.
 */

class SimConditionSet : public TaNameSet<SimConditionAttribute> {

 public:

  /** Default constructor */
  SimConditionSet(void) : TaNameSet<SimConditionAttribute>() {};

  /** Copy constructor */
  SimConditionSet(const SimConditionSet& rOtherSet) : IndexSet(), TaNameSet<SimConditionAttribute>(rOtherSet) {};

  /**  Virtual destructor */
  virtual ~SimConditionSet(void) {};

  /** Test condition for enabled */
  bool Enabled(Idx cond) const { return Attribute(cond).IsEnabled(); };

  /** Set condition enabled */
  void Enabled(Idx cond, bool on) { Attributep(cond)->Enabled(on); };

  /** Get set of enabled conditions */
  SimConditionSet EnabledConditions(void);

  /** Reset all condition states (incl name of this set) */
  void Reset(void);

};



/** @} doxygen group */

} // namespace

#endif
