/** @file agenerator.h Classes TaGenerator and Generator */

/* FAU Discrete Event Systems Library (libfaudes)

Copyright (C) 2006  Bernd Opitz
Copyright (C) 2007  Thomas Moor
Exclusive copyright is granted to Klaus Schmidt

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNES FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */


#ifndef FAUDES_GENERATOR_H
#define FAUDES_GENERATOR_H

#include "definitions.h"
#include "exception.h"
#include "symboltable.h"
#include "indexset.h"
#include "nameset.h"
#include "transset.h"
#include "token.h"
#include "tokenreader.h"
#include "tokenwriter.h"
#include "vgenerator.h"
#include <map>
#include <set>
#include <sstream>
#include <cstdlib>
#include <assert.h>

namespace faudes {

/**
 * Model of the common five-tupel G := (X, Sigma, Delta, X_0, X_m).
 * 
 * \section GeneratorMembers Core Members
 *
 * Generators are be used to represent the closed and the marked language L(G) 
 * and L_m(G) respectively. The core members of a Generator are
 *
 * @param mAlphabet
 *   the Alphabet Sigma, type faudes::EventSet
 * @param mStates
 *   the stateset X, type faudes::StateSet
 * @param mTransRel
 *   the transitionrelation Delta, type faudes::TransSet 
 * @param mInitStates
 *   the set of initial states X_0, type faudes::StateSet
 * @param mMarkedStates
 *   the set of marked statesX_m, type faudes::StateSet
 *  
 * The main purpose of this class is to provide functions that allow read and write access 
 * to the core mebmers, ie inserting and  deleting of events, states and transitions. There
 * are also functions for reachability properties eg trimming the generator. 
 *
 *
 * States, events and transitions in a generator can be addressed in three alternative methods:
 *
 * - by index, involves a search on a sorted set (fast)
 * - by name, involves two searches on sorted set (not quite so fast)
 * - by iterator, involves pointer dereferencing  (very fast)
 *
 * For read access, const refererences to sets are provided. In order to allow for consistency checks,
 * write access is via generator methods only. When the compiletime option FAUDES_CHECKED is 
 * defined, write methods throw an exception on inconsistent data, eg. setting an initial state
 * that is not an element of the state set, or introducung a transition with an event label that
 * is not in the alphabet. 
 *
 *
 * \section SecEventsVersusStates Events Versus States
 *
 * While both events and states are represented by the integer type faudes::Idx, there
 * is a fundamental distinction between both. Events are global enteties in the sense
 * that whether two generators share events or not does matter. Thus, each two events are 
 * related regardless the generators in which they occur. States however, are local to the 
 * generator where they are defined and whether states from different generators  
 * have or dont have the same index is irrelevant. 
 *
 * In consequence of the different charakter of events and states, there is one global
 * sybmoltable that provides event names, while there is a local state symbol table for 
 * each generator. Furthermore, state names are considered cosmetic and hence are optional, while
 * event names are mandatory. 
 *
 * Example: two machines both refer to the event name "alpha" that models the process of passing 
 * a workpiece from one machine to the other. In libFAUDES this is indeed modelled as one event 
 * which is represented by one index. However, both machines may have a state "Idle" which indicates
 * that the respective machine is idle. In libFAUDES the two states are treated locally to the
 * generators and whether or not they have the same index is regarded irrelevant.
 * 
 * The Generator class carries a flag to indicate that functions with result type
 * Generator shall attach names to the newly created states, perhaps based on the state names
 * of the respective arguments. Turning of this feature and avoiding state names alltogether
 * considerably increases libFAUDES performance.
 *
 * \section GeneratorFileFormat File IO
 *
 * Generators inherit the standard token IO interface from Type, so you may use Read and Write
 * functions for Generators. The fileformat consists of a Generator section that includes
 * one subsection for each core member. It is illustrated by the below example
 * 
 * @code
 * <Generator>
 * % libFAUDES Generator for the simple machine
 * "simple machine"                   
 *
 * <Alphabet>                         
 * "alpha" "beta" "mue" "lambda"      
 * </Alphabet>
 *
 * <States>                           
 * "idle" "busy" "down"
 * </States>
 *
 * <TransRel>
 * "idle" "alpha" "busy"
 * "busy" "beta" "idle"
 * "busy" "mue" "down"
 * "down" "lambda" "idle"
 * </TransRel>
 * 
 * <InitStates>
 * "idle"
 * </InitStates>
 * 
 * <MarkedStates>
 * "idle"
 * </MarkedStates>
 * </Generator>
 * @endcode
 *
 * Note that for file IO states may be represented either ba symbolic name or index.
 * However, when writing a file indices will be chosen consecutively and beginning at 1.
 * Thus, state indices are not maintained by file IO. Events are always represented by their 
 * symbolic name.
 *
 * \section Attributes
 *
 * The TaGenerator takes four template parameters for attribute classes. There is a
 * dummy class faudes::AttributeVoid that effectively results in a generator with no attributes. 
 * A convenience typedef defines the type Generator as a TaGenerator with void attributes
 * and this is the intended type to represent plain finite state machines.  
 * 
 * In the context of a TaGenerator, also non-trivial attributes attributes have only minimal 
 * sematics: they can be accessed in a per event, state and transition manner and they 
 * can have default or non-default value. The minimum interface that an attribute class 
 * parameter must provide, is given in faudes::AttributeVoid. Derived classes can provide 
 * addtional semantics, eg faudes::AttributeFlags for boolean flags and faudes::AttributeCFlags 
 * for controllability properties. The TaGenerator transparently support extended attribute
 * semantics, buit does not provide taylored acces function. This is done in TaGenerator
 * derivates eg TcGenerator.
 *
 *
 * @ingroup GeneratorClasses
 */

template <class GlobalAttr, class StateAttr, class EventAttr, class TransAttr>
class TaGenerator : public vGenerator {
 public:
    
  /** Convenience typdef for member transiton set */
  typedef TaTransSet<TransAttr> ATransSet;

  /*****************************************
   *****************************************
   *****************************************
   *****************************************/

  /** @name Constructors & Destructor */
  /** @{ doxygen group */

  /**
   * Construct an emtpy Generator
   */
  TaGenerator(void);

  /** 
   * Copy-constructor (from TaGenerator, incl attributes)
   *
   * @param rOtherGen
   */
  TaGenerator(const TaGenerator& rOtherGen);

  /** 
   * Copy-constructor (from vGenerator, set attributes to default)
   *
   * @param rOtherGen
   */
  TaGenerator(const vGenerator& rOtherGen);

  /**
   * Construct from file. This constructor
   * uses the Read(TokenReader&) function to read.
   *
   * @param rFileName
   *   Name of file
   *
   * @exception Exception
   *   - IO errors (id 1)
   *   - token mismatch (id 50, 51, 52, 80, 85)
   */
  TaGenerator(const std::string& rFileName);

  /**
   * Construct on heap. 
   * Technically not a constructor, this function creates a TaGenerator with the
   * same event symboltable and the same attribute type. Since NewP() is defined as virtual vGenerator
   * function, it can be accessed from all generator classes. It is the callers reponsebilty to
   * delete the object when no longer needed.
   *
   * @return 
   *   new Generator 
   */
  TaGenerator* NewP(void) const;

  /**
   * Construct on stack.
   * Technically not a constructor, this function creates a TaGenerator with the
   * same event symboltable and the same attribute type. 
   *
   * @return 
   *   new Generator 
   */
  virtual TaGenerator NewAGen(void) const;

  /**
   * Destructor
   */
  virtual ~TaGenerator(void);

  /** @} doxygen group */

  /*****************************************
   *****************************************
   *****************************************
   *****************************************/

  /** @name Copy and Assignment */
  /** @{ doxygen group */


  /**
   * Copy to other TaGenerator (incl attributes) 
   *
   * @param rGen
   *   Destination for copy operation.
   */
  virtual void Copy(TaGenerator& rGen) const; 

  /**
   * Copy to other vGenerator (set attributes to default) 
   *
   * @param rGen
   *   Destination for copy operation.
   */
  virtual void Copy(vGenerator& rGen) const; 

  /**
   * Destructive copy to other TaGenerator (incl attributes). 
   * This is the intended interface definition for a copy method with
   * increased performance at the cost of clearing the source data. However,
   * the current implementation calls std copy.
   * 
   *
   * @param rGen
   *   Destination for copy operation.
   */
  virtual void Move(TaGenerator& rGen); 

  /**
   * Destructive copy to other vGenerator (set attributes to default). 
   * This is the intended interface definition for a copy method with
   * increased performance at the cost of clearing the source data. However,
   * the current implementation calls std copy.
   * 
   *
   * @param rGen
   *   Destination for copy operation.
   */
  virtual void Move(vGenerator& rGen); 

  /**
   * Assignment operator (uses Copy(TaGenerator&) )
   *
   * @param rOtherGen
   *   Other generator
   */
  virtual TaGenerator& operator = (const TaGenerator& rOtherGen);

  /**
   * Assignment operator (uses Copy(vGenerator&) )
   *
   * @param rOtherGen
   *   Other generator
   */
  virtual TaGenerator& operator = (const vGenerator& rOtherGen);

  /**
   * Create another version of this generator.
   * Assembles a copy of this generator, however, with versioned events.
   * The new event names are created by appending an underscore and a numeric index.
   * State names and indices as well as any attributes are maintained.
   *
   * @param version
   *   Numeric value to be appended to event names
   * @param rResGen
   *   Resulting versioned generator
   */
  void Version(Idx version, TaGenerator& rResGen);

  /**
   * Create another version of this generator.
   * Assembles a copy of this generator, however, with versioned events.
   * The new event names are created by appending an underscore and a specified string.
   * State names and indices as well as any attributes are maintained.
   *
   * @param rVersion
   *   String value to be appended to event names
   * @param rResGen
   *   Resulting versioned generator
   */
  void Version(const std::string& rVersion, TaGenerator& rResGen);


  /** @} doxygen group */

  /*****************************************
   *****************************************
   *****************************************
   *****************************************/

  /** @name File IO */
  /** @{ doxygen group */
 
  /**
   * Write generators alphabet to console
   */
  void WriteAlphabet(void) const;

  /**
   * Write generators alphabet to string
   *
   * @return
   *   std::string
   * @exception Exception
   *   - IO errors (id 2)
   */
  std::string AlphabetToString(void) const;

  /**
   * Write generators alphabet to tokenwriter
   *
   * @param rTw
   *   Reference to TokenWriter
   *
   * @exception Exception
   *   - IO errors (id 2)
   */
  void WriteAlphabet(TokenWriter& rTw) const;


  /**
   * Write a stateset to console. 
   * Uses WriteStateSet(TokenWriter& rTw, const TaStateSet&) const to write
   * the specified state set to console referring to this generators state names.
   *
   * @param rStateSet
   *   Reference to stateset
   */
  template<class OtherAttr>
    void WriteStateSet(const TaStateSet<OtherAttr>& rStateSet) const;

  /**
   * Write a stateset to console.
   * Uses WriteStateSet(TokenWriter& rTw, const StateSet&) const to write
   * the specified state set to console  referring to this generators state names.
   *
   * @param rStateSet
   *   Reference to stateset
   */
  void WriteStateSet(const StateSet& rStateSet) const;

  /**
   * Write a stateset to string.
   * Uses WriteStateSet(TokenWriter& rTw, const TaStateSet&) const to write
   * the specified state set to a string  referring to this generators state names.
   *
   * @param rStateSet
   *   Reference to stateset
   * @return
   *   std::string
   * @exception Exception
   *   - IO errors (id 2)
   */
  template<class OtherAttr>
    std::string StateSetToString(const TaStateSet<OtherAttr>& rStateSet) const;

  /**
   * Write a stateset to string. 
   * Uses WriteStateSet(TokenWriter& rTw, const StateSet&) const to write
   * the specified state set to a string  referring to this generators state names.
   *
   * @param rStateSet
   *   Reference to stateset
   * @return
   *   std::string
   * @exception Exception
   *   - IO errors (id 2)
   */
  std::string StateSetToString(const StateSet& rStateSet) const;

  /**
   * Write a stateset to TokenWriter. 
   * All output of state sets with  attributes is done with this function.
   * Recall that a StateSet technically is a set of plain indices, no reference
   * to symbolic names. Thus, it is only the context of a Generator that provides
   * the symbolic names for file output. 
   * 
   * Output of state sets always uses the mMinStateIndexMap to re-index states. 
   * However, this map is only set up automatically for file output. If You require
   * re-indexed output to eg a string, you must set up the map by calling SetMinStateIndexMap().
   * To ensure that no re-indexing takes place, call ClearMinStateIndexMap().
   *
   * @param rTw
   *   Reference to TokenWriter
   * @param rStateSet
   *   Reference to stateset
   *
   * @exception Exception
   *   - IO errors (id 2)
   */
  template<class OtherAttr>
    void WriteStateSet(TokenWriter& rTw, const TaStateSet<OtherAttr>& rStateSet) const;


  /**
   * Write a stateset to TokenWriter.
   * All output of statesets with no attributes is done with this function.
   * See WriteStateSet(TokenWriter&, const TaStateSet&) const for further details.
   *
   * @param rTw
   *   Reference to TokenWriter
   * @param rStateSet
   *   Reference to stateset
   *
   * @exception Exception
   *   - IO errors (id 2)
   */
  void WriteStateSet(TokenWriter& rTw, const StateSet& rStateSet) const;


  /**
   * Write a stateset to TokenWriter (debug version)
   * 
   *
   * @param rTw
   *   Reference to TokenWriter
   * @param rStateSet
   *   Reference to stateset
   *
   * @exception Exception
   *   - IO errors (id 2)
   */
  template<class OtherAttr>
    void DWriteStateSet(TokenWriter& rTw, const TaStateSet<OtherAttr>& rStateSet) const;

  /**
   * Write a stateset to TokenWriter (debug version, no  re-indexing)
   *
   * @param rTw
   *   Reference to TokenWriter
   * @param rStateSet
   *   Reference to stateset
   *
   * @exception Exception
   *   - IO errors (id 2)
   */
  void DWriteStateSet(TokenWriter& rTw, const StateSet& rStateSet) const;


  /**
   * Write stateset of this generator to a string (no  re-indexing)
   *
   * @return
   *   std::string
   * @exception Exception
   *   - IO errors (id 2)
   */
  std::string StatesToString(void) const;

  /**
   * Write set of marked states to a string (no  re-indexing)
   *
   * @return
   *   std::string
   * @exception Exception
   *   - IO errors (id 2)
   */
  std::string MarkedStatesToString(void) const;

  /**
   * Write set of initial states to a string (no  re-indexing)
   *
   * @return
   *   std::string
   * @exception Exception
   *   - IO errors (id 2)
   */
  std::string InitStatesToString(void) const;

  /**
   * Write transition relation to console (no  re-indexing)
   */
  void WriteTransRel(void) const;

  /**
   * Write transition relation to string (no  re-indexing)
   */
  std::string TransRelToString(void) const;

  /**
   * Write transition relation to tokenwriter (no  re-indexing)
   * Re-indexing and symbolic state names are handled in the same way
   * as with state sets: this function refers to the generators state symboltable to
   * obtain state names and  uses the mMinStateIndexMap to re-index the output.
   *
   * @param rTw
   *   Reference to TokenWriter
   *
   * @exception Exception
   *   - IO errors (id 2)
   */
  void WriteTransRel(TokenWriter& rTw) const;

  /**
   * Write transition relation to tokenwriter (debug version)
   * @param rTw
   *   Reference to TokenWriter
   *
   * @exception Exception
   *   - IO errors (id 2)
   */
  void DWriteTransRel(TokenWriter& rTw) const;

 
  /**
   * Write statistics to console.
   * Produce statistics (number of states etc.) in comment form (lines starting with %)
   */
  void WriteStatistics(void) const;

  /**
   * Write statistics to string.
   * Produce statistics (number of states etc.) in comment form (lines starting with %)   
   */
  std::string StatisticsToString(void) const;

  /**
   * Write statistics to tokenwriter
   * Produce statistics (number of states etc.) in comment form (lines starting with %)   
   * @param rTw
   *   Reference to TokenWriter
   *
   * @exception Exception
   *   - IO errors (id 2)
   */
  void WriteStatistics(TokenWriter& rTw) const;

 
  /**
   * Writes generator to dot input format.
   * The dot file format is specified by the graphiz package; see http://www.graphviz.org.
   * The package includes the dot command line tool to generate a graphical 
   * representation of the generators graph. See also vGenerator::GraphWrite().
   * This functions sets the re-indexing to minimal indices.
   *
   * @param rFileName
   *   File to write
   *
   * @exception Exception
   *   - IO errors (id 2)
   */
  void DotWrite(const std::string& rFileName);

  /**
   * Writes generator to dot input format (no re-indexing)
   * Variant of DotWrite() without re-indexing.
   *
   * @param rFileName
   *   File to write
   *
   * @exception Exception
   *   - IO errors (id 2)
   */
  void DDotWrite(const std::string& rFileName) const;

  /**
   * Writes generator to dot input format (for export to VioLib).
   * Variant of DotWrite() using strategig state and event names
   * to simplify import to VioLib (qt widget for graphical representation
   * of FAUDES generators).
   *
   * @param rFileName
   *   File to write
   * @exception Exception
   *   - IO errors (id 2)
   */
  void XDotWrite(const std::string& rFileName) const;

  /**
   * Read a state set.
   * Refer to the generators state symboltable while reading a state set.
   * Ignore any attributes.
   *
   * @param rTr
   *   Reference to TokenReader
   * @param rLabel
   *   Label of set in source
   * @param rStateSet
   *   Destination state set 
   *
   * @exception Exception
   *   - IO errors (id 1)
   *   - token mismatch (id 50, 51, 52, 80, 85)
   */
  void ReadStateSet(TokenReader& rTr, const std::string& rLabel, StateSet& rStateSet) const;


  /** @} doxygen group */

  /*****************************************
   *****************************************
   *****************************************
   *****************************************/

  /** @name Basic Maintenance */
  /** @{ doxygen group */

  /** 
   * Check if generator is valid 
   *
   * @return 
   *   Success
   */
  bool Valid(void);

  /**
   * Clear alphabet, states and transitions 
   * (initial states and marked states also)
   */
  virtual void Clear(void);


  /**
   * Clear all states and transitions. Alphabet stays untouched.
   */
  void ClearStates(void);

  /**
   * Get number of events in mAlphabet
   *
   * @return
   *   Number of events
   */
  Idx AlphabetSize(void) const;

  /** 
   * Get generator size (# of states) 
   * 
   * @return 
   *   Number of states
   */
  Idx Size(void) const;

  /**
   * Get number of transitions
   *
   * @return
   *   Number of transitions
   */
  Idx TransRelSize(void) const;

  /**
   * Get number of initial states
   *
   * @return
   *   Number of initial states
   */
  Idx InitStatesSize(void) const;

  /**
   * Get number of marked states
   *
   * @return
   *   Number of marked states
   */
  Idx MarkedStatesSize(void) const;

  /** 
   * Check if generator is empty (no states)
   *
   * @return 
   *   True if mStates is empty
   */
  bool Empty(void) const;

  /**
   * Check if mAlphabet is Empty
   *
   * @return
   *   True if mAlphabet is empty
   */
  bool AlphabetEmpty(void) const;

  /**
   * Check if transition relation is empty
   *
   * @return
   *   True if transition relation is empty
   */
  bool TransRelEmpty(void) const;

  /**
   * Check if set of initial states is empty
   *
   * @return
   *   True if mInitStates is empty
   */
  bool InitStatesEmpty(void) const;

  /**
   * Check if set of marked states is empty
   *
   * @return
   *   True if mMarkedStates is empty
   */
  bool MarkedStatesEmpty(void) const;

  /**
   * Return used events (executed in transitions)
   *
   * @return EventSet
   */
  EventSet UsedEvents(void) const;

  /**
   * Return unused events
   *
   * @return EventSet
   */
  EventSet UnusedEvents(void) const;

  /**
   * Return active event set at state x1
   *
   * @param x1
   *   x1 Idx
   *
   * @return EventSet
   */
  EventSet ActiveEventSet(Idx x1) const;

  /**
   * Return active transition set at state x1
   *
   * @param x1
   *   x1 Idx
   *
   * @return EventSet
   */
  TransSet ActiveTransSet(Idx x1) const;

  /**
   * Return the states covered by transitions als indices
   *
   * @return StateSet
   */
  StateSet TransRelStateSpace(void) const;

  /**
   * Return the successor states of state x1
   *
   * @return StateSet
   */
  StateSet TransRelStateSpace(Idx x1) const;


  /**
   * Updates internal attributes. Since a plain generator
   * does not have any internal attributes, this is
   * just an interface. Reimplement to your needs.
   *
   * @return true if value changed
   */
  virtual bool UpdateAttributes(void) {return true;};

  /** @} doxygen group */



  /*****************************************
   *****************************************
   *****************************************
   *****************************************/

  /** @name Read Access to Core Members */
  /** @{ doxygen group */

  /**
   * iterator to Begin() of mAlphabet
   *
   * @return
   *   iterator to begin of mAlphabet
   */
  EventSet::Iterator AlphabetBegin(void) const;

  /**
   * iterator to End() of mAlphabet
   *
   * @return
   *   iterator to end of mAlphabet
   */
  EventSet::Iterator AlphabetEnd(void) const;

  /** 
   * Test existence of event in mAlphabet
   *
   * @param index
   *   Event index
   *
   * @return
   *   true / false
   */
  bool ExistsEvent(Idx index) const;

  /** 
   * Test existence of event in mAlphabet
   *
   * @param rName
   *   Event name
   *
   * @return
   *   true / false
   */
  bool ExistsEvent(const std::string& rName) const;

  /**
   * Returns a iterator to event index in mAlphabet
   *
   * @param index
   *   Index to find
   *
   * @return
   *   EventSet::iterator to event index
   */
  EventSet::Iterator FindEvent(Idx index) const;

  /**
   * Returns a iterator to event index in mAlphabet
   *
   * @param rName
   *   Event name of index to find
   *
   * @return
   *   EventSet::iterator to event index
   */
  EventSet::Iterator FindEvent(const std::string& rName) const;

  /**
   * Return const reference to mAlphabet 
   *
   * @return EventSet
   */
  const TaEventSet<EventAttr>& Alphabet(void) const;

  /**
   * iterator to Begin() of mStates
   *
   * @return
   *   iterator to begin of mStates
   */
  StateSet::Iterator StatesBegin(void) const;

  /**
   * iterator to End() of mStates
   *
   * @return
   *   iterator to end of mStates
   */
  StateSet::Iterator StatesEnd(void) const;

  /**
   * Test existence of state in mStates
   * 
   * @param index
   *   State index
   *
   * @return
   *   true / false
   */
  bool ExistsState(Idx index) const;

  /**
   * Test existence of state in mStates
   * 
   * @param name
   *   State name
   *
   * @return
   *   true / false
   */
  bool ExistsState(const std::string& name) const;

  /**
   * Returns a iterator to state index in mStates 
   *
   * @param index
   *   Index to find
   *
   * @return
   *   StateSet::Iterator to state index
   */
  StateSet::Iterator FindState(Idx index) const;

  /**
   * Returns a iterator to state with specified name
   *
   * @param rName
   *   name of state to find
   *
   * @return
   *   StateSet::Iterator to state 
   */
  StateSet::Iterator FindState(const std::string& rName) const;


  /**
   * Return reference to mStates
   *
   * @return 
   *   StateSet reference
   */
  const TaStateSet<StateAttr>& States(void) const;

  /**
   * Return initial state 
   *
   * @return
   *   Index of initial state
   *
   * @exception Exception
   *  - initial state does not exist uniquely (id 92)
   */
  Idx InitState(void) const;

  /**
   * Iterator to Begin() of mInitStates
   *
   * @return
   *   Iterator to begin of mInitStates
   */
  StateSet::Iterator InitStatesBegin(void) const;

  /**
   * Iterator to End() of mInitStates
   * 
   * @returns
   *   Iterator to end of mInitStates
   */
  StateSet::Iterator InitStatesEnd(void) const;

  /**
   * Test existence of state in mInitStates
   * 
   * @param index
   *   State index
   *
   * @return
   *   true / false
   */
  bool ExistsInitState(Idx index) const;

  /**
   * Returns a iterator to state index in mInitStates 
   *
   * @param index
   *   Index to find
   *
   * @return
   *   StateSet::Iterator to state index
   */
  StateSet::Iterator FindInitState(Idx index) const;

  /**
   * Return const ref to initial states
   *
   * @return StateSet
   */
  const StateSet& InitStates(void) const;

  /**
   * iterator to Begin() of mMarkedStates
   *
   * @returns
   *   iterator to Begin of mMarkedStates
   */
  StateSet::Iterator MarkedStatesBegin(void) const;

  /**
   * iterator to End() of mMarkedStates
   * 
   * @returns
   *   iterator to End of mMarkedStates
   */
  StateSet::Iterator MarkedStatesEnd(void) const;

  /**
   * Test existence of state in mMarkedStates
   * 
   * @param index
   *   State index
   *
   * @return
   *   true / false
   */
  bool ExistsMarkedState(Idx index) const;

  /**
   * Returns a iterator to state index in mMarkedStates
   *
   * @param index
   *   Index to find
   *
   * @return
   *   StateSet::Iterator to state index
   */
  StateSet::Iterator FindMarkedState(Idx index) const;

  /**
   * Return const ref of marked states
   *
   * @return StateSet
   */
  const StateSet& MarkedStates(void) const;

  /**
   * iterator to Begin() of transition relation
   *
   * @return
   *   iterator to Begin of mTransRel
   */
  typename ATransSet::Iterator TransRelBegin(void) const;

  /**
   * iterator to End() of transition relation
   * 
   * @return
   *   iterator to End of mTransRel
   */
  typename ATransSet::Iterator TransRelEnd(void) const;

  /**
   * iterator to begin of transitions with x1 as predecessor state.
   *
   * @param x1
   *   Predecessor state
   *
   * @return
   *   iterator to begin of transitions with x1
   */
  typename ATransSet::Iterator TransRelBegin(Idx x1) const;

  /**
   * iterator to end of transitions with x1 as predecessor state. 
   *
   * Note: Set the End(x1) iterator to a variable, so it won't be 
   * recalculated every iteration.
   *
   * @param x1
   *   Predecessor state
   *
   * @return 
   *   iterator to end of transitions with x1 
   *   (one after last matching transition)
   */
  typename ATransSet::Iterator TransRelEnd(Idx x1) const;

  /**
   * iterator to begin of transitions with x1 as predecessor state and
   * event ev. 
   *
   * @param x1
   *   Predecessor state
   * @param ev
   *   Event
   *
   * @return
   *   iterator to begin of transitions with x1 and ev
   */
  typename ATransSet::Iterator TransRelBegin(Idx x1, Idx ev) const;

  /**
   * iterator to end of transitions with x1 as predecessor state and
   * event ev.
   *
   * Note: Set the End(x1,ev) iterator to a variable, so it won't be 
   * recalculated every iteration.
   * 
   * @param x1
   *   Predecessor state
   * @param ev
   *   Event
   *
   * @return
   *   iterator to end of transitions with x1 and ev 
   *   (one after last matching transition)
   */
  typename ATransSet::Iterator TransRelEnd(Idx x1, Idx ev) const;

  /**
   * iterator to transition given by x1, ev, x2
   *
   * 
   * @param rX1
   *   name of Predecessor state
   * @param rEv
   *   name of Event
   * @param rX2
   *   name of Successor state
   *
   * @return
   *   iterator to transition or end() if not exists
   */
  typename ATransSet::Iterator FindTransition(
    const std::string& rX1, const std::string& rEv, const std::string& rX2) const;

  /**
   * iterator to transition given by x1, ev, x2
   *
   * 
   * @param x1
   *   Predecessor state
   * @param ev
   *   Event
   * @param x2
   *   Successor state
   *
   * @return
   *   iterator to transition or End() if not exists
   */
  typename ATransSet::Iterator FindTransition(Idx x1, Idx ev, Idx x2) const;

  /**
   * iterator to transition 
   *
   * 
   * @param rTrans
   *   transition
   *
   * @return
   *   iterator to transition or end() if not exists
   */
  typename ATransSet::Iterator FindTransition(const Transition& rTrans) const;

  /**
   * test for transition given by x1, ev, x2
   *
   * 
   * @param rX1
   *   name of Predecessor state
   * @param rEv
   *   name of Event
   * @param rX2
   *   name of Successor state
   *
   * @return
   *   true / false
   */
  bool ExistsTransition(
    const std::string& rX1, const std::string& rEv, const std::string& rX2) const;

  /**
   * test for transition given by x1, ev, x2
   *
   * 
   * @param x1
   *   Predecessor state
   * @param ev
   *   Event
   * @param x2
   *   Successor state
   *
   * @return
   *   true / false
   */
  bool ExistsTransition(Idx x1, Idx ev, Idx x2) const;

  /**
   * test for transition 
   *
   * 
   * @param rTrans
   *   transition
   *
   * @return
   *   true / false
   */
  bool ExistsTransition(const Transition& rTrans) const;

  /**
   * Return reference to transition relation
   *
   * @return TransRel
   */
  const ATransSet& TransRel(void) const;

  /**
   * Get copy of trantision relation  sorted by other compare 
   * operator, e.g. "x2,ev,x1"
   * 
   * @param res
   *   resulting transition relation 
   */
  void TransRel(TransSetX1EvX2& res) const;
  void TransRel(TransSetEvX1X2& res) const;
  void TransRel(TransSetEvX2X1& res) const;
  void TransRel(TransSetX2EvX1& res) const;
  void TransRel(TransSetX2X1Ev& res) const;
  void TransRel(TransSetX1X2Ev& res) const;

  /** @} doxygen group */



  /*****************************************
   *****************************************
   *****************************************
   *****************************************/

  /** @name Write Access to Core Members */
  /** @{ doxygen group */


  /**
   * Add an existing event to mAlphabet by index
   *
   * @param index
   *   Event index
   * @return
   *   true, if event was new to alphabet
   */
  bool InsEvent(Idx index);

  /**
   * Add named event to generator. An entry in the mpEventSymbolTable will
   * be made if event is new.
   *
   * @param rName
   *   Name of the event to add
   *
   * @return 
   *   New unique index
   */
  Idx InsEvent(const std::string& rName);

  /**
   * Add an existing event to mAlphabet by index, with attribute
   *
   * @param attr
   *   attribute of event
   * @param index
   *   Event index
   * @return
   *   true, if event was new to alphabet
   */
  bool InsEvent(Idx index, const EventAttr& attr);

  /**
   * Add named event with attribute to generator. An entry in the 
   * mpEventSymbolTable will be made if event is new.
   *
   * @param rName
   *   Name of the event to add
   * @param attr
   *   attribute of event
   *
   * @return 
   *   New unique index
   */
  Idx InsEvent(const std::string& rName, const EventAttr& attr);

  /**
   * Add new named events to generator.
   *
   * @param events
   *   EventSet
   */
  void InsEvents(const EventSet& events);

  /**
   * Delete event from generator by index. mpEventSymbolTable stays untouched.
   * Transitions containing event will be removed too.
   *
   * @param index
   *   Index of event
   * @return
   *   true, if event was in alphabet
   *
   */
  bool DelEvent(Idx index);

  /**
   * Delete event from generator by name. mpEventSymbolTable stays untouched.
   * Transitions containing event will be removed too.
   *
   * @param rName
   *    Name of event
   * @return
   *   true, if event was in alphabet
   *
   */
  bool DelEvent(const std::string& rName);

  /** 
   * Delete a set of events from generator. mpEventSymbolTable stays untouched.
   * Transitions containing events will be removed too.
   * 
   * @param rEvents
   *   EventSet containing events to remove
   */
  void DelEvents(const EventSet& rEvents);

  /** 
   * Delete event from Alphabet without consistency check. The event is only
   * deleted from mAlphabet but not from transition relation.
   *
   * @param index
   *   Index of event
   * @return
   *   true, if event was in alphabet
   *
   */
  bool DelEventFromAlphabet(Idx index);

  /**
   * Set mAlphabet without consistency check.
   *
   * @param rNewalphabet
   *   EventSet with new alphabet
   */
  void InjectAlphabet(const EventSet& rNewalphabet);

  /**
   * Set mAlphabet without consistency check (incl attributes)
   *
   * @param rNewalphabet
   *   EventSet with new alphabet
   */
  void InjectAlphabet(const TaEventSet<EventAttr>& rNewalphabet);

  /** 
   * Add new anonymous state to generator 
   * 
   * @return 
   *   Index of new unique state
   */
  Idx InsState(void);

  /** 
   * Add new anonymous state with attribute to generator 
   * 
   * @param attr
   *   attribute of new state
   *
   * @return 
   *   Index of new unique state
   */
  Idx InsState(const StateAttr& attr);

  /** 
   * Add (perhaps new) state to generator 
   * 
   * @return 
   *   true to indicate that state was new to generator
   */
  bool InsState(Idx index);

  /** 
   * Add new named state to generator. 
   *
   * @param rName
   *   Name of the state to add
   *
   * @return 
   *   Index of new unique state
   *
   * @exception Exception
   *   Name already exists (id 44)
   */
  Idx InsState(const std::string& rName);

  /** 
   * Add new named state with attribute to generator. 
   *
   * @param rName
   *   Name of the state to add
   * @param attr
   *   attribute of new state
   *
   * @return 
   *   Index of new unique state
   * @exception Exception
   *   Name already exists (id 44)
   */
  Idx InsState(const std::string& rName, const StateAttr& attr);

  /** 
   * Add (perhaps new) state with attribute to generator. 
   *
   * @param index
   *   index of state to add
   * @param attr
   *   attribute of new state
   *
   * @return
   *   true, if event was new to alphabet
   *
   */
  bool InsState(Idx index, const StateAttr& attr);

  /** 
   * Delete a state from generator by index. 
   * Cleans mStates, mInitStates, mMarkedStates, mTransRel and mpStateSymbolTable.
   *
   * @param index
   *   Index of state to delete. Will be erased in mpStateSymbolTable too.
   * @return
   *   true, if state was in stateset
   */
  bool DelState(Idx index);

  /** 
   * Delete a state from generator by name.
   * Cleans mStates, mInitStates, mMarkedStates, mTransRel and mpStateSymbolTable.
   *
   * @param rName
   *   Name of state to delete. Will be erased in mpStateSymbolTable too
   * @return
   *   true, if state was in stateset
   *
   * @exception Exception
   *   - symbolic name not known (id 90)
   */
  bool DelState(const std::string& rName);

  /**
   * Delete a set of states 
   * Cleans mStates, mInitStates, mMarkedStates, mTransrel, and mpStateSymboltable
   *
   * @param rDelStates
   *   StateSet containing states to delete
   */
  void DelStates(const StateSet& rDelStates);


  /**
   * Delete a state from generator without consistency check. This removes the
   * state from mStates and mpStateSymbolTable but doesn't touch mTransRel,
   * mInitStates and mMarkedStates.
   *
   * @param index
   *   Index of state to delete. Will be erased in mpStateSymbolTable too
   * @return
   *   true, if state was in stateset
   *
   */
  bool DelStateFromStates(Idx index);

  /**
   * Delete a state from generator without consistency check. This removes the
   * state from mStates and mpStateSymbolTable but doesn't touch mTransRel,
   * mInitStates and mMarkedStates.
   * Index to delete is given by iterator. 
   *
   * @param pos
   *   StateSet::iterator
   * @return
   *   iteraror to next state
   */
  StateSet::Iterator DelStateFromStates(StateSet::Iterator pos);

  /** 
   * Inject an existing state index into generators mStates
   * Use with care! For use in optimized functions.
   *
   * @param index
   *   State index to inject
   */
  void InjectState(Idx index);

  /**
   * Inject a complete mStates without consistency checks (without attributes)
   *
   * @param rNewStates
   *    StateSet
   */
  void InjectStates(const StateSet& rNewStates);


  /**
   * Create new anonymous state and set as initial state
   *
   * @return 
   *   Index of new unique
   */
  Idx InsInitState(void);

  /**
   * Create a new named state and set as initial state
   *
   * @param rName
   *   Name of the state to add
   *
   * @return 
   *   Index of new unique state
   */
  Idx InsInitState(const std::string& rName);

  /**
   * Create new anonymous state and set as marked state
   *
   * @return 
   *   Index of new unique state
   */
  Idx InsMarkedState(void);

  /**
   * Create a new named state and set as marked state
   *
   * @param rName
   *   Name of the state to add
   *
   * @return 
   *   Index of new unique state
   */
  Idx InsMarkedState(const std::string& rName);

  /**
   * Set an existing state as initial state by index.
   *
   *
   * @param index
   *   Index of state to set as initial state
   * @exception Exception
   *   - State index not found in generator (id 91)
   */
  void SetInitState(Idx index);

  /**
   * Set an existing state as initial state by name. 
   * (only for user interactions)
   *
   * @param rName
   *   Name of state to set as initial state
   *
   * @exception Exception
   *   - State name not known in generator (id 90)
   */
  void SetInitState(const std::string& rName);

  /**
   * Replace mInitStates with StateSet given as parameter without consistency checks.
   *
   * @param rNewInitStates
   *   StateSet containing new mInitStates
   */
  void InjectInitStates(const StateSet& rNewInitStates);

  /**
   * Unset an existing state as initial state by index
   *
   * Define FAUDES_CHECKED for consistency checks.
   *
   * @param index
   *   State index
   *
   * @exception Exception
   *   - State index not found in generator (id 91)
   */
  void ClrInitState(Idx index);

  /**
   * Unset an existing state as initial state by name (only for user
   * interactions)
   *
   * @param rName
   *   State name
   *
   * @exception Exception
   *   - State name not known in generator (id 90)
   */
  void ClrInitState(const std::string& rName);

  /**
   * Unset an existing state as initial state by iterator
   *
   * @param pos
   *   StateSet::iterator
   */
  void ClrInitState(StateSet::Iterator pos);

  /**
   * Clear all mInitStates
   */
  void ClearInitStates(void); 

  /**
   * Set an existing state as marked state by index
   *
   * Define FAUDES_CHECKED for consistency checks.
   *
   * @param index
   *   Index of state to set as initial state
   * @exception Exception
   *   - State index not found in generator (id 91)
   */
  void SetMarkedState(Idx index);

  /**
   * Set an existing state as marked state by name. 
   * (only for user interactions)
   *
   * @param rName
   *   Name of state to set as marked state
   *
   * @exception Exception
   *   - State name not known in generator (id 90)
   */
  void SetMarkedState(const std::string& rName);

  /**
   * Unset an existing state as marked state by index
   *
   * Define FAUDES_CHECKED for consistency checks.
   *
   * @param index
   *   State index
   *
   * @exception Exception
   *   - State index not found in generator (id 91)
   */
  void ClrMarkedState(Idx index);

  /**
   * Unset an existing state as marked state by name (only for user
   * interactions)
   *
   * @param rName
   *   State name
   *
   * @exception Exception
   *   - State index not found in generator (id 91)
   */
  void ClrMarkedState(const std::string& rName);

  /**
   * Unset an existing state as marked state by iterator
   *
   * @param pos
   *   StateSet::iterator
   */
  void ClrMarkedState(StateSet::Iterator pos);

  /**
   * Clear all marked states
   */
  void ClearMarkedStates(void);

  /**
   * Replace mMarkedStates with StateSet given as parameter without consistency checks.
   *
   * @param rNewMarkedStates
   *   StateSet containing new marked states
   */
  void InjectMarkedStates(const StateSet& rNewMarkedStates);

  /** 
   * Add a transition to generator by indices. States and event
   * must already exist!
   *
   * Define FAUDES_CHECKED for consistency checks.
   *
   * @param x1 
   *   Predecessor state index 
   * @param ev 
   *   Event index 
   * @param x2
   *   Successor state index
   *
   * @return
   *   True, if the transition was new the generator
   *
   * @exception Exception
   *   - state or event not in generator (id 95)
   */
  bool SetTransition(Idx x1, Idx ev, Idx x2);

  /** 
   * Add a transition to generator by names. Statename and eventname
   * must already exist!
   *
   * @param rX1
   *   Predecessor state name
   * @param rEv
   *   Event name
   * @param rX2
   *   Successor state name
   *
   * @return
   *   True, if the transition was new the generator
   *
   * @exception Exception
   *   - state or event not in generator (id 95)
   *   - state name not known (id 90)
   *   - event name not known (id 66)
   */
  bool SetTransition(const std::string& rX1, const std::string& rEv, 
				   const std::string& rX2);

  /** 
   * Add a transition to generator. States and event
   * must already exist!
   *
   *
   * @param rTransition
   *   Transition
   *
   * @return
   *   True, if the transition was new the generator
   * @exception Exception
   *   - state or event not in generator (id 95)
   */
  bool SetTransition(const Transition& rTransition);

  /** 
   * Add a transition with attribute to generator. States and event
   * must already exist!
   *
   *
   * @param rTransition
   *   transition
   * @param rAttr
   *   attribute
   * @return
   *   True, if the transition was new the generator
   * @exception Exception
   *   - state or event not in generator (id 95)
   *
   */
  bool SetTransition(const Transition& rTransition, const TransAttr& rAttr);

  /**
   * Remove a transition by indices
   *
   * @param x1
   *   Predecessor state index
   * @param ev
   *   Event index
   * @param x2
   *   Successor state index
   */
  void ClrTransition(Idx x1, Idx ev, Idx x2);

  /**
   * Remove a transition by transition object
   *
   * @param rTrans
   *   Transition object
   */
  void ClrTransition(const Transition& rTrans);

  /**
   * Remove a transition by iterator
   *
   * @param it
   *   TransSet::iterator 
   * @return
   *   iterator to next transition
   */
  typename ATransSet::Iterator ClrTransition(typename ATransSet::Iterator it);

  /**
   * Clear all transitions
   */
  void ClearTransRel(void);

  /**
   * Set transition without consistency check.
   *
   * @param rTrans
   *   Transition to insert
   */
  void InjectTransition(const Transition& rTrans);

  /**
   * Set mTransRel without consistency check.
   *
   * @param rNewtransrel
   *   EventSet with new alphabet
   */
  void InjectTransRel(const TransSet& rNewtransrel);

  /** @} doxygen group */



  /*****************************************
   *****************************************
   *****************************************
   *****************************************/

  /** @name Attributes */
  /** @{ doxygen group */


  /**
   * Clear Attributes
   */
  void ClearAttributes(void);

  /**
   * Determine and set those attributes, that are 
   * a conseqence of other given data. E.g. you may set
   * particular state flag, if this state is reachable.
   * 
   * This method does nothing and may be reimplemented 
   * by a class that adds semantics to attributes.
   */
  void DetermineAttributes(void) {};

  /**
   * Clear event attributes
   */
  void ClearEventAttributes(void);

  /**
   * Clear attribute for existing event
   *
   * @param index
   *   Event index
   */
  void ClrEventAttribute(Idx index);

  /**
   * Set attribute for existing event
   *
   * @param index
   *   Event index
   * @param rAttr
   *   New attribute
   *
   * @exception Exception
   *   Index not found in EventSymbolMap (id 42)
   */
  void EventAttribute(Idx index, const EventAttr& rAttr);

  /** 
   * Event attribute lookup 
   *
   * @param index
   *
   * @return 
   *   reference to attribute
   */
  const EventAttr& EventAttribute(Idx index) const;

  /** 
   * Event attribute lookup 
   *
   * @param rName
   *
   * @return 
   *   reference to attribute
   */
  const EventAttr& EventAttribute(const std::string& rName) const;

  /** 
   * Event attribute pointer (to access Attribute methods)
   * note: may insert explicit default attribute
   *
   * @param index
   *
   * @return 
   *   pointer to attribute
   */
  EventAttr* EventAttributep(Idx index);

  /** 
   * Event attribute pointer (to access Attribute methods)
   * note: may insert explicit default attribute
   *
   * @param rName
   *
   * @return 
   *   pointer to attribute
   */
  EventAttr* EventAttributep(const std::string& rName);


  /**
   * Clear state attributes
   */
  void ClearStateAttributes(void);

  /**
   * Clear attribute for existing state
   *
   * @param index
   *   State index
   */
  void ClrStateAttribute(Idx index);

  /** 
   * Set attribute for existing state
   *
   * @param index
   *   Index
   * @param rAttr
   *   attriute
   *
   * @exception Exception
   *   Name already associated with another index (id 44)
   */
  void StateAttribute(Idx index, const StateAttr& rAttr);

  /** 
   * State attribute lookup 
   *
   * @param index
   *
   * @return ref to attribute of state
   */
  const StateAttr& StateAttribute(Idx index) const;

  /** 
   * State attribute pointer (to access Attribute methods) 
   * note: may insert explicit default attribute
   *
   * @param index
   *
   * @return pointer to attribute of state
   */
  StateAttr* StateAttributep(Idx index);

  /**
   * Clear transition attributes
   */
  void ClearTransAttributes(void);

  /**
   * Set attribute for existing transition
   *
   * @param rTrans
   *   transition
   * @param rAttr
   *   New attribute
   *
   */
  void TransAttribute(const Transition& rTrans, const TransAttr& rAttr);

  /**
   * Clear attribute for existing transition
   *
   * @param rTrans
   *   transition
   */
  void ClrTransAttribute(const Transition& rTrans);

  /**
   * get attribute for existing transition
   *
   * @return 
   *   attribute
   *
   */
  const TransAttr& TransAttribute(const Transition& rTrans) const;

  /**
   * get attribute pointer for existing transition
   * note: may insert explicit default attribute
   *
   * @return 
   *   attribute pointer
   *
   */
  TransAttr* TransAttributep(const Transition& rTrans);

  /**
   * Clear global attribute
   */
  void ClearGlobalAttribute(void);

  /** 
   * Set global attribute 
   *
   * @param rAttr
   *   attribute
   */
  void GlobalAttribute(const GlobalAttr& rAttr) {mGlobalAttribute=rAttr;};

  /** 
   * Get attribute ref
   */
  const GlobalAttr& GlobalAttribute(void) const {return mGlobalAttribute;};


  /** 
   * Get attribute pointer 
   */
  GlobalAttr* GlobalAttributep(void) {return &mGlobalAttribute;};


  /** @} doxygen group */



  /*****************************************
   *****************************************
   *****************************************
   *****************************************/

  /** @name Reachability */
  /** @{ doxygen group */


  /**
   * Compute set of accessible states
   */
  StateSet AccessibleSet(void) const;

  /**
   * Make generator accessible
   *
   * @return
   *   True if generator contains at least one initial state
   */
  bool Accessible(void);

  /**
   * Check if generator is accessible
   *
   * @return
   *   True if generator contains at least one initial state
   */
  bool IsAccessible(void) const;

  /**
   * Compute set of Coaccessible states
   */
  StateSet CoaccessibleSet(void) const;

  /**
   * Make generator Coaccessible
   *
   * @return
   *   True if generator contains at least one marked state
   */
  bool Coaccessible(void);

  /**
   * Check if generator is Coaccessible
   *
   * @return
   *   True if generator contains at least one marked state
   */
  bool IsCoaccessible(void) const;

  /**
   * Computer set of trim states
   */
  StateSet TrimSet(void) const;

  /**
   * Make generator trim
   *
   * @return 
   *   True if generator contains at least one initial state and
   *   one marked state after internally calling Accessible() and
   *   Coaccessible().
   */
  bool Trim(void);

  /**
   * Check if generator is trim
   *
   * @return
   *   True if generator is trim
   */
  bool IsTrim(void) const;


  /**
   * Computer set of blocking states
   */
  StateSet BlockingSet(void) const;

  /** @} doxygen group */



  /*****************************************
   *****************************************
   *****************************************
   *****************************************/

  /** @name Misc */
  /** @{ doxygen group */


  /**
   * Set the alphabet to events executed in transitions
   */
  void MinimizeAlphabet(void);

  /**
   * Check if generator is deterministique
   * We insit in exactly one initial state and for each state and event at most 
   * one transition (to some successor state). This definition matches the situation
   * where a generator is defined to have one inital state as opposed to a set set of initial
   * states (which may be empty). In particular, the generated language of a determinitstic 
   * generator allways includes the empty string and therefor is not the empty set. When
   * focuus is on the marked language only, this issue does not exists.
   *
   * @return
   *   True if Generator is deterministic
   */
  bool IsDeterministic(void) const;

  /**
   * Set minimal index map for file io of generator states
   *
   * This function is implemented as fake-const to allow for
   * const Write function.
   *
   */
  void SetMinStateIndexMap(void) const;

  /**
   * Clear minimal index map for 1:1 file io 
   *
   */
  void ClearMinStateIndexMap(void) const;

  /**
   * Get state index as is it will be written to file
   *
   * @param index
   *   state index
   * @return
   *   minimal index
   */
  Idx MinStateIndex(Idx index) const;


  /**
   * Get state index translation map
   *
   * @return
   *   minimal index map
   */
  const std::map<Idx,Idx>& MinStateIndexMap(void) const;


  /**
   * Get Pointer to EventSymbolTable.
   *
   * @return
   *   Pointer to EventSymbolTable
   */
  SymbolTable* EventSymbolTablep(void) const;

  /**
   * Set Pointer to EventSymbolTable. 
   * This function sets the reference to the event symboltable. The current implementation
   * clears the generator, future versions may implement a re-indexing.
   *
   * @param pSymTab
   *   Pointer to SymbolTable
   */
  void EventSymbolTablep(SymbolTable* pSymTab);

  /**
   * Set Pointer to EventSymbolTable. 
   * This function sets the reference to the global event symboltable according to
   * the one used in the specified generator. The current implementation
   * clears the generator, future versions may implement a re-indexing.
   *
   * @param rOtherGen
   *   Generator to provide global event symboltable 
   *   
   */
  virtual void EventSymbolTablep(const vGenerator& rOtherGen);

  /** @} doxygen group */


 protected:
  /** Generator alphabet */
  TaEventSet<EventAttr> mAlphabet;

  /** Generator state set */
  TaStateSet<StateAttr> mStates;

  /** Transition relation */
  ATransSet mTransRel;

  /** Initial states */
  StateSet mInitStates;

  /** Marked states */
  StateSet mMarkedStates;

  /** Map State indices to consecutive indices */
  std::map<Idx,Idx> mMinStateIndexMap;

  /** global attribute */
  GlobalAttr mGlobalAttribute;

  /**
   * Read generator object from TokenReader, see Type::Read for public wrappers.
   *
   * Virtual function for std token io interface. Context is ignored,
   * label defaults to "Generator".
   *
   * @param rTr
   *   TokenReader to read from
   * @param rLabel
   *   Section to read
   * @param pContext
   *   Read context to provide contextual information (ignored)
   *
   * @exception Exception
   *   - token mismatch (id 50, 51, 52, 80, 85)
   *   - IO error (id 1)
   */
  virtual void DoRead(TokenReader& rTr,  const std::string& rLabel = "", const Type* pContext=0);
 
  /**
   * Write generator to TokenWriter, see Type::Write for public wrappers.
   *
   * Virtual function for std token io interface. Context is ignored,
   * label defaults to "Generator". If the tokenwriter writes to a file,
   * state indices will be re-indext to start from 1.
   *
   * @param rTw
   *   Reference to TokenWriter
   * @param rLabel
   *   Label of section to write
   * @param pContext
   *   Write context to provide contextual information (ignored)
   *
   * @exception Exception 
   *   - IO errors (id 2)
   */
  virtual void DoWrite(TokenWriter& rTw, const std::string& rLabel="",const Type* pContext=0) const;

  /**
   * Write generator in debugging format to TokenWriter, see Type::DWrite for public wrappers.
   *
   * Reimplement this method in derived classes to provide the std token io
   * interface defined in the public section of Type.
   *
   * @param rTw
   *   Reference to TokenWriter
   * @param rLabel
   *   Label of section to write
   * @param pContext
   *   Write context to provide contextual information (ignored)
   *
   * @exception Exception 
   *   - IO errors (id 2)
   */
  virtual void DoDWrite(TokenWriter& rTw, const std::string& rLabel="",const Type* pContext=0) const;

  /**
   * Read the generator's name from a file
   * 
   * @param rFileName
   *   File to read from
   *
   * @exception Exception
   *   - IO errors (id 1)
   *   - token mismatch (id 50, 51, 52)
   */
  void ReadGeneratorName(const std::string& rFileName);

  /**
   * Read the generator's name from a TokenReader
   * 
   * @param rTr
   *   Reference to TokenReader
   *
   * @exception Exception
   *   - IO errors (id 1)
   *   - token mismatch (id 50, 51, 52)
   */
  void ReadGeneratorName(TokenReader& rTr);

  /**
   * Read the generator's alphabet from a file.
   *
   * @param rFileName
   *   File to read from
   *
   * @exception Exception
   *   - IO errors (id 1)
   *   - token mismatch (id 50, 51, 52)
   */
  void ReadAlphabet(const std::string& rFileName);

  /**
   * Read the generator's alphabet from a TokenReader
   *
   * @param rTr
   *   Reference to TokenReader
   *
   * @exception Exception
   *   - IO errors (id 1)
   *   - token mismatch (id 50, 51, 52)
   */
  void ReadAlphabet(TokenReader& rTr);

  /**
   * Read the generator's mStates from a file. 
   * This sets up the StateSymbolTable
   *
   * @param rFileName
   *   File to read from
   *
   * @exception Exception
   *   - IO errors (id 1)
   *   - token mismatch (id 50, 51, 52)
   */
  void ReadStates(const std::string& rFileName);

  /**
   * Read the generator's stateset from a TokenReader. 
   * This sets up the StateSymbolTable
   *
   * @param rTr
   *   Reference to TokenReader
   *
   * @exception Exception
   *   - IO errors (id 1)
   *   - token mismatch (id 50, 51, 52)
   */
  void ReadStates(TokenReader& rTr);

  /**
   * Read the generator's transition relation from a file. 
   *
   * @param rFileName
   *   File to read from
   *
   * @exception Exception
   *   - IO errors (id 1)
   *   - token mismatch (id 50, 51, 52)
   */
  void ReadTransRel(const std::string& rFileName);

  /**
   * Read the generator's transition relation from a TokenReader. 
   *
   * @param rTr
   *   Reference to TokenReader
   *
   * @exception Exception
   *   - IO errors (id 1)
   *   - token mismatch (id 50, 51, 52)
   */
  void ReadTransRel(TokenReader& rTr);

  /**
   * Get accessible states by filling accessibleset recursive
   *
   * @param accessibleset
   *   Reference to StateSet with accessibel states
   * @param startState
   *   State index to start with
   */
  void CheckAccessible(StateSet& accessibleset, Idx startState) const;

  /**
   * Get Coaccessible states by filling Coaccessibleset recursive
   *
   * @param Coaccessibleset
   *   Reference to StateSet with coaccessibel states
   * @param rtrel
   *   Reverse transition relation where x1 <-> x2
   * @param startState
   *   State index to start with
   */
  void CheckCoaccessible(StateSet& Coaccessibleset, const TransSetX2EvX1& rtrel, Idx startState) const;


};


/** 
 * Plain generator, typedef for generator with no attributes. 
 * \ingroup GeneratorClasses
 */
typedef TaGenerator<AttributeVoid,AttributeVoid,AttributeVoid,AttributeVoid> Generator; 


/* convenience access to relevant scopes */
#define THIS TaGenerator<GlobalAttr, StateAttr, EventAttr, TransAttr>
#define TEMP template <class GlobalAttr, class StateAttr, class EventAttr, class TransAttr>
#define BASE vGenerator

// Generator::Generator(void)
TEMP THIS::TaGenerator(void) : 
  // init base
  vGenerator() 
{  
  FD_DG("TaGenerator(" << this << ")::TaGenerator()");
  // set basic members
  mStates.Name("States");
  mInitStates.Name("InitStates");
  mMarkedStates.Name("MarkedStates");
  mAlphabet.Name("Alphabet");
}

// Generator::Generator(rOtherGen)
TEMP THIS::TaGenerator(const TaGenerator& rOtherGen) : 
  // copy base
  vGenerator(rOtherGen)
{
  FD_DG("TaGenerator(" << this << ")::TaGenerator(" << &rOtherGen << ")");
  // have  a 1:1 copy (incl sym tables)
  rOtherGen.Copy(*this);
}


// Generator::Generator(rOtherGen)
TEMP THIS::TaGenerator(const vGenerator& rOtherGen) : 
  // copy base
  vGenerator(rOtherGen)
{
  FD_DG("TaGenerator(" << this << ")::TaGenerator([v]" << &rOtherGen << ")");
  // have  a 1:1 copy (incl sym tables)
  rOtherGen.Copy(*this);
}

// Generator::Generator(rFileName)
TEMP THIS::TaGenerator(const std::string& rFileName) : 
  // init base
  vGenerator()
{
  FD_DG("TaGenerator(" << this << ")::TaGenerator(" << rFileName << ")");
  // set some defaults
  mStateNamesEnabled=true;
  // do read
  Read(rFileName);
}

// Copy(gen)  
TEMP void THIS::Copy(TaGenerator& rGen) const {
  FD_DG("TaGenerator(" << this << ")::Copy(" << &rGen << ")");
  // prepare result (call clear for virtual stuff)
  rGen.Clear();
  // have same event symboltable
  rGen.EventSymbolTablep(mpEventSymbolTable);
  // copy state symboltable
  rGen.StateSymbolTable(mStateSymbolTable);
  // set other members
  rGen.Name(Name());
  rGen.StateNamesEnabled(StateNamesEnabled());
  rGen.mStates= mStates;
  rGen.mInitStates = mInitStates;
  rGen.mMarkedStates = mMarkedStates;
  rGen.mAlphabet = mAlphabet;
  rGen.mTransRel=mTransRel;
  rGen.mStateNamesEnabled=mStateNamesEnabled;
  rGen.mGlobalAttribute=mGlobalAttribute;
  // copy add on stuff
  rGen.mMinStateIndexMap=mMinStateIndexMap;

#ifdef FAUDES_CHECKED
  if(!rGen.Valid()) {
    FD_DG("TaGenerator()::Copy(): invalid generator");
    rGen.DWrite(); 
    abort();
  }
#endif
} 

// Copy(gen)  
TEMP void THIS::Copy(vGenerator& rGen) const {
  FD_DG("TaGenerator(" << this << ")::Copy([v]" << &rGen << ")");
  // prepare result (call clear for virtual stuff)
  rGen.Clear();
  // have same event symboltable
  rGen.EventSymbolTablep(mpEventSymbolTable);
  // copy state symboltable
  rGen.StateSymbolTable(mStateSymbolTable);
  // set other members
  rGen.Name(Name());
  rGen.StateNamesEnabled(StateNamesEnabled());
  rGen.InjectStates(mStates);
  rGen.InjectInitStates(mInitStates);
  rGen.InjectMarkedStates(mMarkedStates);
  rGen.InjectAlphabet(mAlphabet);
  rGen.InjectTransRel(mTransRel);
#ifdef FAUDES_CHECKED
  if(!rGen.Valid()) {
    FD_DG("TaGenerator()::Copy([v]): invalid generator");
    rGen.DWrite(); 
    abort();
  }
#endif
} 


// Move(gen) destructive copy, not implemented
TEMP void THIS::Move(TaGenerator& rGen) {
  FD_DG("TaGenerator(" << this << ")::Move(" << &rGen << ")");
  Copy(rGen);
  Clear();
}


// Move(gen) destructive copy, not implemented 
TEMP void THIS::Move(vGenerator& rGen) {
  FD_DG("TaGenerator(" << this << ")::Move([v]" << &rGen << ")");
  Copy(rGen);
  Clear();
}

// Generator::~Generator
TEMP THIS::~TaGenerator(void) {
  FD_DG("TaGenerator(" << this << ")::~TaGenerator()");
}

// NewP
TEMP THIS* THIS::NewP(void) const {
  // allocate
  THIS* res = new THIS;
  // fix base data
  res->EventSymbolTablep(BASE::mpEventSymbolTable);
  res->mStateNamesEnabled=BASE::mStateNamesEnabled;
  return res;
}

// NewAGen()
TEMP THIS THIS::NewAGen(void) const {
  THIS res;
  // fix base data
  res.EventSymbolTablep(BASE::mpEventSymbolTable);
  res.mStateNamesEnabled=BASE::mStateNamesEnabled;
  return res;
}


// operator =
TEMP TaGenerator<GlobalAttr,StateAttr,EventAttr,TransAttr>& THIS::operator = (const TaGenerator& rOtherGen) {
  FD_DG("TaGenerator(" << this << ")::operator = " << &rOtherGen);
  rOtherGen.Copy(*this);
  return *this;
}

// operator =
TEMP TaGenerator<GlobalAttr,StateAttr,EventAttr,TransAttr>& THIS::operator = (const vGenerator& rOtherGen) {
  FD_DG("TaGenerator(" << this << ")::operator = [v]" << &rOtherGen);
  rOtherGen.Copy(*this);
  return *this;
}

// EventSymbolTablep() 
TEMP SymbolTable* THIS::EventSymbolTablep(void) const {
  return BASE::EventSymbolTablep();
}

// EventSymbolTablep(pSymTab) 
TEMP void THIS::EventSymbolTablep(SymbolTable* pSymTab) {
  BASE::EventSymbolTablep(pSymTab);
  // todo: reindex rather than clear stuff
  Clear();
  mAlphabet.SymbolTablep(mpEventSymbolTable);
}

// EventSymbolTablep(rOtherGen) 
TEMP void THIS::EventSymbolTablep(const vGenerator& rOtherGen) {
  BASE::EventSymbolTablep(rOtherGen);
}

// DoWrite()
TEMP void THIS::DoWrite(TokenWriter& rTw, const std::string& rLabel, const Type* pContext) const {
  (void) pContext;
  // figure section
  std::string label=rLabel;
  if(label=="") label="Generator"; 
  FD_DG("TaGenerator(" << this << ")::DoWrite(): section " << label);
  // reindex for file output
  if(rTw.DestMode()==TokenWriter::File) SetMinStateIndexMap();
  // write generator
  rTw.WriteBegin(label);
  rTw << mMyName;
  rTw << "\n";
  rTw << "\n";
  WriteStatistics(rTw);
  rTw << "\n";
  mAlphabet.Write(rTw);
  rTw << "\n";
  WriteStateSet(rTw, mStates);
  rTw << "\n";
  WriteTransRel(rTw);
  rTw << "\n";
  WriteStateSet(rTw, mInitStates);
  rTw << "\n";
  WriteStateSet(rTw, mMarkedStates);
  rTw << "\n";
  mGlobalAttribute.Write(rTw);
  rTw << "\n";
  rTw.WriteEnd(label);
  // end of reindex
  ClearMinStateIndexMap();
}

// DoDWrite()
TEMP void THIS::DoDWrite(TokenWriter& rTw, const std::string& rLabel, const Type* pContext) const {
  (void) pContext;
  // figure section
  std::string label=rLabel;
  if(label=="") label="Generator"; 
  FD_DG("TaGenerator(" << this << ")::DoDWrite(): section " << label);
  // write generator
  rTw.WriteBegin(label);
  rTw << mMyName;
  rTw << "\n";
  rTw << "\n";
  WriteStatistics(rTw);
  rTw << "\n";
  mAlphabet.DWrite(rTw);
  rTw << "\n";
  DWriteStateSet(rTw, mStates);
  rTw << "\n";
  DWriteTransRel(rTw);
  rTw << "\n";
  DWriteStateSet(rTw, mInitStates);
  rTw << "\n";
  DWriteStateSet(rTw, mMarkedStates);
  rTw << "\n";
  mGlobalAttribute.Write(rTw);
  rTw << "\n";
  rTw.WriteEnd(label);
  rTw << "\n";
}

// WriteAlphabet() 
TEMP void THIS::WriteAlphabet(void) const {
  TokenWriter tw(TokenWriter::Stdout);
  WriteAlphabet(tw);
}

// AlphabetToString()
TEMP std::string THIS::AlphabetToString(void) const {
  TokenWriter tw(TokenWriter::String);
  WriteAlphabet(tw);
  return tw.Str();
}

// WriteAlphabet(rTw&) 
TEMP void THIS::WriteAlphabet(TokenWriter& rTw) const { 
  mAlphabet.Write(rTw);
}

// WriteStateSet(rStateSet&) 
TEMP template<class OtherAttr>
void THIS::WriteStateSet(const TaStateSet<OtherAttr>&  rStateSet) const {
  TokenWriter tw(TokenWriter::Stdout);
  WriteStateSet(tw,rStateSet);
}

// WriteStateSet(rStateSet&) 
TEMP void THIS::WriteStateSet(const StateSet&  rStateSet) const {
  TaStateSet<AttributeVoid> vcopy = rStateSet;
  WriteStateSet(vcopy);
}

// StateSetToString()
TEMP template<class OtherAttr>
std::string THIS::StateSetToString(const TaStateSet<OtherAttr>&  rStateSet) const {
  TokenWriter tw(TokenWriter::String);
  WriteStateSet(tw,rStateSet);
  return tw.Str();
}

// StateSetToString()
TEMP std::string THIS::StateSetToString(const StateSet&  rStateSet) const {
  TaStateSet<AttributeVoid> vcopy = rStateSet; 
  return StateSetToString(vcopy);
}

// WriteStateSet(rTw&, rStateSet&) 
TEMP template<class OtherAttr>
void THIS::WriteStateSet(TokenWriter& rTw, const TaStateSet<OtherAttr>&  rStateSet) const {
  rTw.WriteBegin(rStateSet.Name());
  // build reverse index map of states to write ( fileidx->idx )
  // (this ensures named states to be wtritten first; see SetMinStateIndexMap())
  std::map<Idx,Idx> reversemap;
  std::map<Idx,Idx>::const_iterator minit;
  StateSet::Iterator sit;
  for (sit = rStateSet.Begin(); sit != rStateSet.End(); ++sit) {
    reversemap[MinStateIndex(*sit)] = *sit;
  }
  // iterate states to write 
  for(minit = reversemap.begin(); minit != reversemap.end(); ++minit) {
    // identify anonymous block (consecutive state indices)
    std::map<Idx,Idx>::const_iterator conit=minit;
    Idx start = conit->first;
    Idx anoncount = 0;
    for(; conit != reversemap.end(); ++conit) {
      if(StateName(conit->second) != "") break;
      if(!StateAttribute(conit->second).IsDefault()) break;
      if(conit->first != start+anoncount) break;
      ++anoncount;
    }
    // write anonymous block
    if (anoncount > FD_CONSECUTIVE) {
      rTw.WriteBegin("Consecutive");
      rTw << start;
      rTw << start+anoncount-1;
      rTw.WriteEnd("Consecutive");
      minit=conit;
    } 
    // break loop
    if(minit == reversemap.end() )
      break;
    // write non anonymous state name/idx
    std::string statename = StateName(minit->second);
    if (statename != "") {
      rTw << statename;
    } else {
      rTw << minit->first;
    }
    // write state attribute
    OtherAttr attr=rStateSet.Attribute(minit->second);
    attr.Write(rTw);
  }
  rTw.WriteEnd(rStateSet.Name());
}

// WriteStateSet(rTw&, rStateSet&) 
TEMP void THIS::WriteStateSet(TokenWriter& rTw, const StateSet&  rStateSet) const {
  TaStateSet<AttributeVoid> vcopy=rStateSet;
  WriteStateSet(rTw,vcopy);
}

// DWriteStateSet(rTw&, rStateSet&) 
TEMP template<class OtherAttr>
void THIS::DWriteStateSet(TokenWriter& rTw, const TaStateSet<OtherAttr>& rStateSet) const {
  rTw.WriteBegin(rStateSet.Name());
  StateSet::Iterator lit;
  std::map<Idx,Idx>::const_iterator minit;
  for(lit = rStateSet.Begin(); lit != rStateSet.End(); ++lit) {
    rTw << SStr(*lit);
    OtherAttr attr=rStateSet.Attribute(*lit);
    attr.Write(rTw);
  }
  rTw.WriteEnd(rStateSet.Name());
}


// DWriteStateSet(rTw&, rStateSet&) 
TEMP void THIS::DWriteStateSet(TokenWriter& rTw, const StateSet&  rStateSet) const {
  TaStateSet<AttributeVoid> vcopy=rStateSet;
  DWriteStateSet(rTw,vcopy);
}

// StatesToString()
TEMP std::string THIS::StatesToString(void) const {
  return StateSetToString(mStates);
}

// MarkedStatesToString()
TEMP std::string THIS::MarkedStatesToString(void) const {
  return StateSetToString(mMarkedStates);
}

// InitStatesToString()
TEMP std::string THIS::InitStatesToString(void) const {
  return StateSetToString(mInitStates);
}


// WriteTransRel() 
TEMP void THIS::WriteTransRel(void) const {
  TokenWriter tw(TokenWriter::Stdout);
  WriteTransRel(tw);
}

// TransRelToString()
TEMP std::string THIS::TransRelToString(void) const {
  TokenWriter tw(TokenWriter::String);
  WriteTransRel(tw);
  return tw.Str();
}

// WriteTransRel(rTw&)
TEMP void THIS::WriteTransRel(TokenWriter& rTw) const {
  typename ATransSet::Iterator tit;
  int oldcolumns = rTw.Columns();
  rTw.Columns(3);
  rTw.WriteBegin("TransRel");
  bool smalltransrel = (Size() < FD_SMALLTRANSREL);

  for (tit = mTransRel.Begin(); tit != mTransRel.End(); ++tit) {

    // write x1
    Idx x1=MinStateIndex(tit->X1);
    if (smalltransrel) {
      std::string x1name = StateName(tit->X1);
      if (x1name != "") {
	rTw << x1name;
      } else {
	rTw << x1;
      }
    } else {
      rTw << x1;
    }

    // write ev
    rTw << EventName(tit->Ev);

    // write x2
    Idx x2=MinStateIndex(tit->X2);
    if (smalltransrel) {
      std::string x2name = StateName(tit->X2);
      if (x2name != "") {
	rTw << x2name;
      } else {
	rTw << x2;
      }
    } else {
      rTw << x2;
    }

    // write attributes
    TransAttribute(*tit).Write(rTw);

  }
  rTw.WriteEnd("TransRel");
  rTw.Columns(oldcolumns);
}

// DWriteTransRel(rTw&)
TEMP void THIS::DWriteTransRel(TokenWriter& rTw) const {
  typename ATransSet::Iterator tit;
  int oldcolumns = rTw.Columns();
  rTw.Columns(1);
  rTw.WriteBegin("TransRel");
  for (tit = mTransRel.Begin(); tit != mTransRel.End(); ++tit) {
    rTw << TStr(*tit);
    TransAttr attr=mTransRel.Attribute(*tit);
    attr.Write(rTw);
  }
  rTw.WriteEnd("TransRel");
  rTw.Columns(oldcolumns);
}



// WriteStatistics() 
TEMP
void THIS::WriteStatistics(void) const  
{
  TokenWriter tw(TokenWriter::Stdout);
  WriteStatistics(tw);
}

// StatisticsToString()
TEMP std::string THIS::StatisticsToString(void) const {
  TokenWriter tw(TokenWriter::String);
  WriteStatistics(tw);
  return tw.Str();
}

// WriteStatistics(rTw&)
TEMP void THIS::WriteStatistics(TokenWriter& rTw) const
{
  rTw.WriteComment("");
  rTw.WriteComment(" Statistics for " + Name());
  rTw.WriteComment(" States:        " + ToStringInteger(Size()) );
  rTw.WriteComment(" Init/Marked:   " + ToStringInteger(mInitStates.Size()) 
	      + "/" + ToStringInteger(mMarkedStates.Size()));
  rTw.WriteComment(" Events:        " + ToStringInteger(mAlphabet.Size()) );
  rTw.WriteComment(" Transitions:   " + ToStringInteger(mTransRel.Size()) );
  rTw.WriteComment(" StateSymbols:  " + ToStringInteger(mpStateSymbolTable->Size()) );
  rTw.WriteComment(" Attrib. E/S/T: " + ToStringInteger(mAlphabet.AttributesSize()) 
	      + "/" + ToStringInteger(mStates.AttributesSize())
	      + "/" + ToStringInteger(mTransRel.AttributesSize()) );
  rTw.WriteComment("");
}

// DotWrite(rFileName)
TEMP void THIS::DotWrite(const std::string& rFileName) {
  FD_DG("TaGenerator(" << this << ")::DotWrite(" << rFileName << ")");
  SetMinStateIndexMap();
  StateSet::Iterator lit;
  typename ATransSet::Iterator tit;
  try {
    std::ofstream stream;
    stream.exceptions(std::ios::badbit|std::ios::failbit);
    stream.open(rFileName.c_str());
    stream << "digraph \"" << Name() << "\" {" << std::endl;
    stream << "  rankdir=LR" << std::endl;
    stream << "  node [shape=circle];" << std::endl;
    stream << std::endl;
    stream << "  // istates" << std::endl;
    int i = 1;
    for (lit = InitStatesBegin(); lit != InitStatesEnd(); ++lit) {
      std::string xname= StateName(*lit);
      if(xname=="") xname=ToStringInteger(MinStateIndex(*lit));
      stream << "  dot_dummyinit_" << i << " [shape=none, label=\"\" ];" << std::endl;
      stream << "  dot_dummyinit_" << i << " -> \"" << xname << "\";" << std::endl; 
      i++;
    }
    stream << std::endl;
    stream << "  // mstates" << std::endl;
    for (lit = MarkedStatesBegin(); lit != MarkedStatesEnd(); ++lit) {
      std::string xname= StateName(*lit);
      if(xname=="") xname=ToStringInteger(MinStateIndex(*lit));
      stream << "  \"" << xname << "\" [shape=doublecircle];" << std::endl;
    }
    stream << std::endl;
    stream << "  // rest of stateset" << std::endl;
    for (lit = StatesBegin(); lit != StatesEnd(); ++lit) {
      if (! (ExistsInitState(*lit) || ExistsMarkedState(*lit)) ) {
	std::string xname= StateName(*lit);
	if(xname=="") xname=ToStringInteger(MinStateIndex(*lit));
	stream << "  \"" << xname << "\";" << std::endl;
      }
    }
    stream << std::endl;
    stream << "  // transition relation" << std::endl;
    for (tit = TransRelBegin(); tit != TransRelEnd(); ++tit) {
      std::string x1name= StateName(tit->X1);
      if(x1name=="") x1name=ToStringInteger(MinStateIndex(tit->X1));
      std::string x2name= StateName(tit->X2);
      if(x2name=="") x2name=ToStringInteger(MinStateIndex(tit->X2));
      stream << "  \"" << x1name  << "\" -> \"" << x2name
	     << "\" [label=\"" << EventName(tit->Ev) << "\"];" << std::endl;
    }
    stream << "};" << std::endl;
    stream.close();
  }
  catch (std::ios::failure&) {
    throw Exception("TaGenerator::DotWrite", 
		    "Exception opening/writing dotfile \""+rFileName+"\"", 2);
  }
  ClearMinStateIndexMap();
}

// DDotWrite(rFileName)
TEMP void THIS::DDotWrite(const std::string& rFileName) const {
  FD_DG("TaGenerator(" << this << ")::DDotWrite(" << rFileName << ")");
  StateSet::Iterator lit;
  typename ATransSet::Iterator tit;
  try {
    std::ofstream stream;
    stream.exceptions(std::ios::badbit|std::ios::failbit);
    stream.open(rFileName.c_str());
    stream << "digraph \"" << Name() << "\" {" << std::endl;
    stream << "  rankdir=LR" << std::endl;
    stream << "  node [shape=circle];" << std::endl;
    stream << std::endl;
    stream << "  // istates" << std::endl;
    int i = 1;
    for (lit = InitStatesBegin(); lit != InitStatesEnd(); ++lit) {
      stream << "  dot_dummyinit_" << i << " [shape=none, label=\"\" ];" << std::endl;
      stream << "  dot_dummyinit_" << i << " -> \"" 
	     << SStr(*lit)  << "\";" << std::endl; 
      i++;
    }
    stream << std::endl;
    stream << "  // mstates" << std::endl;
    for (lit = MarkedStatesBegin(); lit != MarkedStatesEnd(); ++lit) {
      stream << "  \"" << SStr(*lit) << "\" [shape=doublecircle];" << std::endl;
    }
    stream << std::endl;
    stream << "  // rest of stateset" << std::endl;
    for (lit = StatesBegin(); lit != StatesEnd(); ++lit) {
      // if not in mInitStates or mMarkedStates
      if (! (ExistsInitState(*lit) || ExistsMarkedState(*lit)) ) {
	stream << "  \"" << SStr(*lit) << "\";" << std::endl;
      }
    }
    stream << std::endl;
    stream << "  // transition relation" << std::endl;
    for (tit = TransRelBegin(); tit != TransRelEnd(); ++tit) {
      stream << "  \"" << SStr(tit->X1) 
	     << "\" -> \"" << SStr(tit->X2)
	     << "\" [label=\"" << EventName(tit->Ev) << "\"];" << std::endl;
    }
    stream << "};" << std::endl;
    stream.close();
  }
  catch (std::ios::failure&) {
    throw Exception("TaGenerator::DotWrite", 
		    "Exception opening/writing dotfile \""+rFileName+"\"", 2);
  }
}


// XDotWrite(rFileName)
TEMP void THIS::XDotWrite(const std::string& rFileName) const {
  FD_DG("TaGenerator(" << this << ")::XDotWrite(" << rFileName << ")");
  StateSet::Iterator lit;
  typename ATransSet::Iterator tit;
  try {
    std::ofstream stream;
    stream.exceptions(std::ios::badbit|std::ios::failbit);
    stream.open(rFileName.c_str());
    stream << "digraph \"___" << Name() << "___\" {" << std::endl;
    stream << "  rankdir=LR" << std::endl;
    stream << "  node [shape=circle];" << std::endl;
    stream << std::endl;
    stream << "  // stateset" << std::endl;
    for (lit = StatesBegin(); lit != StatesEnd(); ++lit) {
      stream << "  \"s" << *lit << "\";" << std::endl;
    }
    stream << std::endl;
    stream << "  // transition relation" << std::endl;
    for (tit = TransRelBegin(); tit != TransRelEnd(); ++tit) {
      stream << "  \"s" << tit->X1 
	     << "\" -> \"s" << tit->X2
	     << "\" [label=\"e" << tit->Ev << "\" " << "polyline" << "];" << std::endl;
    }
    stream << "};" << std::endl;
    stream.close();
  }
  catch (std::ios::failure&) {
    throw Exception("TaGenerator::XDotWrite", 
		    "Exception opening/writing dotfile \""+rFileName+"\"", 2);
  }
}

// DoRead(tr)
TEMP void THIS::DoRead(TokenReader& rTr,  const std::string& rLabel, const Type* pContext) {
  (void) pContext;
  std::string label=rLabel;
  if(label=="") label="Generator"; 
  FD_DG("TaGenerator(" << this << ")::DoRead(): file " << rTr.FileName() << "  section " << label);
  // Clear old stuff
  Clear();
  // find Genertor in any of our current file formats
  rTr.SeekBegin(label);
  // get the Generator "name" token
  ReadGeneratorName(rTr);
  // read mAlphabet
  ReadAlphabet(rTr);
  // read stateset
  ReadStates(rTr);
  // read transrel
  ReadTransRel(rTr);
  // read istates
  ReadStateSet(rTr, "InitStates", mInitStates);
  // read mstates
  ReadStateSet(rTr, "MarkedStates", mMarkedStates);
  // read attribute
  mGlobalAttribute.Read(rTr);
  // read end 
  rTr.SeekEnd(label);
  FD_DG("TaGenerator(" << this << ")::DoRead(): done");
}

// ReadGeneratorName(rFileName)
TEMP void THIS::ReadGeneratorName(const std::string& rFileName) {
  TokenReader tr(rFileName);
  tr.SeekBegin("Generator");  // mimique old behaviour .. use "ReadBegin" instead   
  ReadGeneratorName(tr);
}

// ReadGeneratorName(tr)
TEMP void THIS::ReadGeneratorName(TokenReader& rTr) {
  FD_DG("TaGenerator(" << this << ")::ReadGeneratorName(\"" 
	<< rTr.FileName() << "\")");
   mMyName=rTr.ReadString();
}

// ReadAlphabet(rFileName)
TEMP void THIS::ReadAlphabet(const std::string& rFileName) {
  FD_DG("TaGenerator(" << this << ")::ReadAlphabet(\"" << rFileName << "\")");
  mAlphabet.Read(rFileName,"Alphabet");
}

// ReadAlphabet(rTr) 
TEMP void THIS::ReadAlphabet(TokenReader& rTr) {
  FD_DG("TaGenerator(" << this << ")::ReadAlphabet(\"" 
	<< rTr.FileName() << "\")");
  mAlphabet.Read(rTr,"Alphabet");
}

// ReadStates(rFileName) 
TEMP void THIS::ReadStates(const std::string& rFileName) {
  TokenReader tr(rFileName);
  ReadStates(tr);
}

// ReadStates(tr) 
TEMP void THIS::ReadStates(TokenReader& rTr) {
  FD_DG("TaGenerator(" << this << ")::ReadStates(\"" << rTr.FileName() << "\")");
  // HELPERS:
  Token token;
  // ALGORITHM:
  mStates.Clear();
  mStates.Name("States");
  mStateSymbolTable.Clear();
  rTr.SeekBegin("States");  
  while(rTr.Peek(token)) {
    // break end of section
    if (token.Type() == Token::End) {
      break;
    }
    // read state by index
    if (token.Type() == Token::Integer) {
      rTr.Get(token);
      Idx index = token.IntegerValue();
      FD_DG("TaGenerator(" << this << ")::ReadStates(\"" << rTr.FileName() << "\"): by index " << index);
      if(mStates.Exists(index)) {
	std::stringstream errstr;
	errstr << "Token " << token.IntegerValue() << " appears twice in stateset"
	       << rTr.FileLine();
	throw Exception("TaGenerator::ReadStates", errstr.str(), 80);
      }
      StateAttr sattr;
      sattr.Read(rTr);
      InsState(index,sattr);
      continue;
    } 
    // read state by name
    if (token.Type() == Token::String) {
      rTr.Get(token);
      FD_DG("TaGenerator(" << this << ")::ReadStates(\"" << rTr.FileName() << "\"): by name " << token.StringValue());
      if(ExistsState(token.StringValue())) {
	std::stringstream errstr;
	errstr << "Token " << token.StringValue() << " appears twice in stateset"
	       << rTr.FileLine();
	throw Exception("TaGenerator::ReadStates", errstr.str(), 80);
      }
      StateAttr sattr;
      sattr.Read(rTr);
      InsState(token.StringValue(),sattr);
      continue;
    } 
    // read consecutive block of anonymous states
    if (token.Type() == Token::Begin && token.StringValue() == "Consecutive") {
      rTr.ReadBegin("Consecutive");
      Token token1,token2;
      rTr.Get(token1);
      rTr.Get(token2);
      FD_DG("TaGenerator(" << this << ")::ReadStates(\"" << rTr.FileName() << "\"): consecutive range");
      if ((token1.Type() != Token::Integer) || (token2.Type() != Token::Integer)) {
	std::stringstream errstr;
	errstr << "Invalid range of consecutive states"  << rTr.FileLine();
	throw Exception("TaGenerator::ReadStates", errstr.str(), 80);
      }
      for(Idx index = (Idx) token1.IntegerValue(); index <= (Idx) token2.IntegerValue(); ++index) {
	if(mStates.Exists(index)) {
	  std::stringstream errstr;
	  errstr << "Index " << index << " appears twice in stateset"
		 << rTr.FileLine();
	  throw Exception("TaGenerator::ReadStates", errstr.str(), 80);
	}
	InsState(index);
      }
      rTr.ReadEnd("Consecutive");
      continue;
    }
    // cannot process token
    std::stringstream errstr;
    errstr << "Invalid token" << rTr.FileLine();
    throw Exception("TaGenerator::ReadStates", errstr.str(), 80);
  }
  rTr.SeekEnd("States");
  FD_DG("TaGenerator(" << this << ")::ReadStates(\"" << rTr.FileName() << "\"): done");
}


// ReadStateSet(tr, rLabel, rStateSet) 
TEMP void THIS::ReadStateSet(TokenReader& rTr, const std::string& rLabel, StateSet& rStateSet)  const {
  FD_DG("TaGenerator(" << this << ")::ReadStateSet(\"" << rLabel<< "\")");
  // HELPERS:
  Token token;
  // ALGORITHM:
  rStateSet.Clear();
  rTr.SeekBegin(rLabel);  
  rStateSet.Name(rLabel);
  while(rTr.Peek(token)) {
    // break end of section
    if (token.Type() == Token::End) {
      break;
    }
    // read state by index
    if (token.Type() == Token::Integer) {
      rTr.Get(token);
      Idx index = token.IntegerValue();
      if(!ExistsState(index)) {
	std::stringstream errstr;
	errstr << "Token " << token.IntegerValue() << " not in stateset"
	       << rTr.FileLine();
	throw Exception("TaGenerator::ReadStateSet", errstr.str(), 80);
      }
      AttributeVoid vattr;
      vattr.Read(rTr);
      rStateSet.Insert(index);
      continue;
    } 
    // read state by name
    if (token.Type() == Token::String) {
      rTr.Get(token);
      Idx index =StateIndex(token.StringValue());
      if(index==0) {
	std::stringstream errstr;
	errstr << "Token " << token.StringValue() << " not in stateset"
	       << rTr.FileLine();
	throw Exception("TaGenerator::ReadStateSet", errstr.str(), 80);
      }
      AttributeVoid vattr;
      vattr.Read(rTr);
      rStateSet.Insert(index);
      continue;
    } 
    // read consecutve block of anonymous states
    if (token.Type() == Token::Begin && token.StringValue() == "Consecutive") {
      rTr.ReadBegin("Consecutive");
      Token token1,token2;
      rTr.Get(token1);
      rTr.Get(token2);
      if ((token1.Type() != Token::Integer) || (token2.Type() != Token::Integer)) {
	std::stringstream errstr;
	errstr << "Invalid range of consecutive states"  << rTr.FileLine();
	throw Exception("TaGenerator::ReadStateSet", errstr.str(), 80);
      }
      for(Idx index = (Idx) token1.IntegerValue(); index <= (Idx) token2.IntegerValue(); ++index) {
	if(!ExistsState(index)) {
	  std::stringstream errstr;
	  errstr << "Token " << token.IntegerValue() << " not in stateset"
		 << rTr.FileLine();
	  throw Exception("TaGenerator::ReadStateSet", errstr.str(), 80);
	}
	rStateSet.Insert(index);
      }
      rTr.ReadEnd("Consecutive");
      continue;
    }
    // cannot process token
    std::stringstream errstr;
    errstr << "Invalid token" << rTr.FileLine();
    throw Exception("TaGenerator::ReadStateSet", errstr.str(), 50);
  }
  rTr.SeekEnd(rLabel);
}

// ReadTransRel(rFileName)
TEMP void THIS::ReadTransRel(const std::string& rFileName) {
  TokenReader tr(rFileName);
  ReadTransRel(tr);
}

// ReadTransRel(tr) 
TEMP void THIS::ReadTransRel(TokenReader& rTr) {
  FD_DG("TaGenerator(" << this << ")::ReadTransRel(\"" << rTr.FileName() << "\")");
  try {
    Token token;
    Idx x1 = 0, ev = 0, x2 = 0;
    rTr.SeekBegin("TransRel");
    // Read tokens
    while (rTr.Peek(token)) {
      // 0: end of transition relation
      if ((token.Type() == Token::End)) break;

      // 1: the x1 token
      rTr >> token;
      if (token.Type() == Token::Integer) {
	x1=token.IntegerValue();
      } else if (token.Type() == Token::String) {
	token.StringValue();
	x1=StateIndex(token.StringValue());
      } else break;

      // 2: the event token
      rTr >> token;
      if (token.Type() == Token::Integer) {
	ev=token.IntegerValue();
      } else if (token.Type() == Token::String) {
	token.StringValue();
	ev=EventIndex(token.StringValue());
      } else break;

      // 3: the x1 token
      rTr >> token;
      if (token.Type() == Token::Integer) {
	x2=token.IntegerValue();
      } else if (token.Type() == Token::String) {
	token.StringValue();
	x2=StateIndex(token.StringValue());
      } else break;

      // 4: attributes
      TransAttr attr; 
      attr.Read(rTr);       

      // check values
      if( (!ExistsState(x1)) || (!ExistsState(x2)) ){
	std::stringstream errstr;
	errstr << "invalid state" << rTr.FileLine();
	throw Exception("TaGenerator::ReadTransRel", errstr.str(), 85);
      } 
      if(!ExistsEvent(ev)) {
	std::stringstream errstr;
	errstr << "invalid event" << rTr.FileLine();
	throw Exception("TaGenerator::ReadTransRel", errstr.str(), 85);
      } 

      // insert transition
      SetTransition(Transition(x1, ev, x2),attr);

    } // end while
    rTr.SeekEnd("TransRel");   
  }
  catch (faudes::Exception&) {
    std::stringstream errstr;
    errstr << "Reading TransRel failed in " << rTr.FileLine();
    throw Exception("TaGenerator::ReadTransRel", errstr.str(), 50); 
  }
  FD_DG("TaGenerator(" << this << ")::ReadTransRel(\"" << rTr.FileName() << "\"): sone");
}

// Valid()
TEMP bool THIS::Valid(void) {
  FD_DG("TaGenerator(" << this << ")::Valid()");
  typename ATransSet::Iterator tit;
  StateSet::Iterator lit;
  for (tit = TransRelBegin(); tit != TransRelEnd(); ++tit) {
    if (! ExistsState(tit->X1)) return false;
    if (! ExistsEvent(tit->Ev)) return false;
    if (! ExistsState(tit->X2)) return false;
  }
  for (lit = InitStatesBegin(); lit != InitStatesEnd(); ++lit) {
    if (! ExistsState(static_cast<Idx>(*lit))) return false;
  }
  for (lit = MarkedStatesBegin(); lit != MarkedStatesEnd(); ++lit) {
    if (! ExistsState(static_cast<Idx>(*lit))) return false;
  }
  if (mAlphabet.Name() != "Alphabet") return false;
  if (mStates.Name() != "States") return false;
  if (mInitStates.Name() != "InitStates") return false;
  if (mMarkedStates.Name() != "MarkedStates") return false;
  return true;
}

// AlphabetSize()
TEMP Idx THIS::AlphabetSize(void) const {
  return mAlphabet.Size();
}

// Size()
TEMP Idx THIS::Size(void) const {
  return mStates.Size();
}

// virtual
// Clear()
TEMP void THIS::Clear(void) {
  FD_DG("TaGenerator(" << this << ")::Clear()");
  mAlphabet.Clear();
  mStates.Clear();
  BASE::mpStateSymbolTable->Clear();
  mTransRel.Clear();
  mInitStates.Clear();
  mMarkedStates.Clear();
  ClearGlobalAttribute();
}

// ClearGlobalAttribute()
TEMP void THIS::ClearGlobalAttribute(void) {
  mGlobalAttribute = GlobalAttr();
}

// ClearStateAttributes()
TEMP void THIS::ClearStateAttributes(void) {
  mStates.ClearAttributes();
}

// ClearEventAttributes()
TEMP void THIS::ClearEventAttributes(void) {
  mAlphabet.ClearAttributes();
}

// ClearTransAttributes()
TEMP void THIS::ClearTransAttributes(void) {
  mTransRel.ClearAttributes();
}

// ClearAttributes()
TEMP void THIS::ClearAttributes(void) {
  ClearGlobalAttribute();
  ClearStateAttributes();
  ClearEventAttributes();
  ClearTransAttributes();
}


// ClearStates()
TEMP void THIS::ClearStates(void) {
  mStates.Clear();
  mTransRel.Clear();
  mInitStates.Clear();
  mMarkedStates.Clear();
  mpStateSymbolTable->Clear();
}

// TransRelSize()
TEMP Idx THIS::TransRelSize(void) const {
  return mTransRel.Size();
}

// InitStatesSize()
TEMP Idx THIS::InitStatesSize(void) const {
  return mInitStates.Size();
}

// MarkedStatesSize()
TEMP Idx THIS::MarkedStatesSize(void) const {
  return mMarkedStates.Size();
}

// AlphabetEmpty()
TEMP bool THIS::AlphabetEmpty(void) const {
  return mAlphabet.Empty();
}

// Empty()
TEMP bool THIS::Empty(void) const {
  return mStates.Empty();
}

// TransRelEmpty()
TEMP bool THIS::TransRelEmpty(void) const {
  return mTransRel.Empty();
}

// InitStatesEmpty()
TEMP bool THIS::InitStatesEmpty(void) const {
  return mInitStates.Empty();
}

// MarkedStatesEmpty()
TEMP bool THIS::MarkedStatesEmpty(void) const {
  return mMarkedStates.Empty();
}

// iterator AlphabetBegin() const
TEMP EventSet::Iterator THIS::AlphabetBegin(void) const {
  return mAlphabet.Begin();
}

// iterator AlphabetEnd() const
TEMP EventSet::Iterator THIS::AlphabetEnd(void) const {
  return mAlphabet.End();
}

// iterator StatesBegin() const
TEMP StateSet::Iterator THIS::StatesBegin(void) const {
  return mStates.Begin();
}

// iterator StatesEnd() const
TEMP StateSet::Iterator THIS::StatesEnd(void) const {
  return mStates.End();
}

// iterator TransRelBegin() const
TEMP typename THIS::ATransSet::Iterator 
THIS::TransRelBegin(void) const {
  return mTransRel.Begin();
}

// iterator TransRelEnd() const
TEMP typename THIS::ATransSet::Iterator 
THIS::TransRelEnd(void) const {
  return mTransRel.End();
}

// iterator TransRelBegin(x1) const
TEMP typename THIS::ATransSet::Iterator 
THIS::TransRelBegin(Idx x1) const {
  return mTransRel.Begin(x1);
}

// iterator TransRelEnd(x1) const
TEMP typename THIS::ATransSet::Iterator 
THIS::TransRelEnd(Idx x1) const {
  return mTransRel.End(x1);
}

// iterator TransRelBegin(x1, ev) const
TEMP typename THIS::ATransSet::Iterator 
THIS::TransRelBegin(Idx x1, Idx ev) const {
  return mTransRel.Begin(x1, ev);
}

// iterator TransRelEnd(x1, ev) const
TEMP typename THIS::ATransSet::Iterator 
THIS::TransRelEnd(Idx x1, Idx ev) const {
  return mTransRel.End(x1, ev);
}

// iterator FindTransition(trans)
TEMP typename THIS::ATransSet::Iterator 
THIS::FindTransition(const Transition& rTrans) const {
  return mTransRel.Find(rTrans);
}

// iterator FindTransition(x1, ex x2)
TEMP typename THIS::ATransSet::Iterator 
THIS::FindTransition(Idx x1, Idx ev, Idx x2) const {
  return mTransRel.Find(Transition(x1, ev, x2));
}

// iterator FindTransition(x1, ev, x2)
TEMP typename THIS::ATransSet::Iterator THIS::FindTransition(
							  const std::string& rX1, const std::string& rEv, const std::string& rX2) const 
{
  return mTransRel.Find(StateIndex(rX1), EventIndex(rEv), StateIndex(rX2));
}

// iterator ExistsTransition(trans)
TEMP bool THIS::ExistsTransition(const Transition& rTrans) const {
  return mTransRel.Exists(rTrans);
}

// iterator ExistsTransition(x1, ex x2)
TEMP bool THIS::ExistsTransition(Idx x1, Idx ev, Idx x2) const {
  return mTransRel.Exists(Transition(x1, ev, x2));
}

// iterator ExistsTransition(x1, ev, x2)
TEMP bool THIS::ExistsTransition(
				 const std::string& rX1, const std::string& rEv, const std::string& rX2) const 
{
  return mTransRel.Exists(StateIndex(rX1), EventIndex(rEv), StateIndex(rX2));
}


// idx InitState() const
TEMP Idx THIS::InitState(void) const {
#ifdef FAUDES_CHECKED
  if(mInitStates.Size()!=1) {    
    std::stringstream errstr;
    errstr << "init state does not exist uniquely" << std::endl;
    throw Exception("TaGenerator::InitSTate", errstr.str(), 92);
  }
#endif
  return *mInitStates.Begin();
}

// iterator InitStatesBegin() const
TEMP StateSet::Iterator THIS::InitStatesBegin(void) const {
  return mInitStates.Begin();
}

// iterator InitStatesEnd() const
TEMP StateSet::Iterator THIS::InitStatesEnd(void) const {
  return mInitStates.End();
}

// iterator MarkedStatesBegin()
TEMP StateSet::Iterator THIS::MarkedStatesBegin(void) const {
  return mMarkedStates.Begin();
}

// iterator MarkedStatesEnd() const
TEMP StateSet::Iterator THIS::MarkedStatesEnd(void) const {
  return mMarkedStates.End(); 
}

// InjectAlphabet(newalphabet)
TEMP void THIS::InjectAlphabet(const EventSet& newalphabet) {
  FD_DG("TaGenerator::InjectAlphabet()  " << newalphabet.ToString());
#ifdef FAUDES_CHECKED
  if(newalphabet.SymbolTablep()!=mpEventSymbolTable) {
    std::stringstream errstr;
    errstr << "symboltable mismatch aka not implemented" << std::endl;
    throw Exception("TaGenerator::InjectAlphabet", errstr.str(), 88);
  }
#endif
  mAlphabet = newalphabet;
  mAlphabet.Name("Alphabet");
}

// InjectAlphabet(newalphabet)
TEMP void THIS::InjectAlphabet(const TaEventSet<EventAttr>& newalphabet) {
  FD_DG("TaGenerator::InjectAlphabet(TaEventSet<EventAttr>)  " << newalphabet.ToString());
#ifdef FAUDES_CHECKED
  if(newalphabet.SymbolTablep()!=mpEventSymbolTable) {
    std::stringstream errstr;
    errstr << "symboltable mismatch aka not implemented" << std::endl;
    throw Exception("TaGenerator::InjectAlphabet", errstr.str(), 88);
  }
#endif
  mAlphabet = newalphabet;
  mAlphabet.Name("Alphabet");
}

// InsEvent(index)
TEMP bool THIS::InsEvent(Idx index) {
  FD_DG("TaGenerator(" << this << ")::InsEvent(" << index << ")");
  return mAlphabet.Insert(index);
}

// InsEvent(rName)
TEMP Idx THIS::InsEvent(const std::string& rName) {
  FD_DG("TaGenerator(" << this << ")::InsEvent(\"" << rName << "\")");
  return mAlphabet.Insert(rName);
}

// InsEvent(index, attr)
TEMP bool THIS::InsEvent(Idx index, const EventAttr& attr) {
  FD_DG("TaGenerator(" << this << ")::InsEvent(" << index << " " << attr.ToString() << ")");
  return mAlphabet.Insert(index, attr);
}

// InsEvent(rName)
TEMP Idx THIS::InsEvent(const std::string& rName, const EventAttr& attr) {
  FD_DG("TaGenerator(" << this << ")::InsEvent(\"" << rName << attr.ToString() << "\")");
  return mAlphabet.Insert(rName, attr);
}

// InsEvents(events)
TEMP void THIS::InsEvents(const EventSet& events) {
  mAlphabet.InsertSet(events);
}

// DelEvent(index)
TEMP bool THIS::DelEvent(Idx index) {
  FD_DG("TaGenerator(" << this << ")::DelEvent(" << index << ")");
  mTransRel.EraseByEv(index);
  return mAlphabet.Erase(index);
}

// DelEvent(rName)
TEMP bool THIS::DelEvent(const std::string& rName) {
  FD_DG("TaGenerator(" << this << ")::DelEvent(\"" << rName << "\")");
  Idx index = mAlphabet.Index(rName);
  mTransRel.EraseByEv(index);
  return mAlphabet.Erase(index);
}

// DelEvents(events)
TEMP void THIS::DelEvents(const EventSet& rEvents) {
  FD_DG("TaGenerator(" << this << ")::DelEvents(\"" 
	<< rEvents.ToString() << "\")");
  EventSet::Iterator it;
  for (it = rEvents.Begin(); it != rEvents.End(); ++it) {
    DelEvent(*it);
  }
}

// DelEventFromAlphabet(index)
TEMP bool THIS::DelEventFromAlphabet(Idx index) {
  FD_DG("TaGenerator(" << this << ")::DelEventFromAlphabet(" 
	<< index << ")");
  return mAlphabet.Erase(index);
}

// InsState()
TEMP Idx THIS::InsState(void) {
  FD_DG("TaGenerator(" << this << ")::InsState()");
  return mStates.Insert();
}

// InsState(index)
TEMP bool THIS::InsState(Idx index) {
  FD_DG("TaGenerator(" << this << ")::InsState(" << index << ")");
  return mStates.Insert(index);
}

// InsState(index, attr)
TEMP bool THIS::InsState(Idx index, const StateAttr& attr) {
  FD_DG("TaGenerator(" << this << ")::InsState(" << index << ",attr)");
  return mStates.Insert(index,attr);
}

// InsState(rName)
TEMP Idx THIS::InsState(const std::string& rName) {
  FD_DG("TaGenerator(" << this << ")::InsState(\"" << rName << "\")");
  Idx index=mStates.Insert();
  StateName(index,rName);
  return index;
}

// InsState(rName, attr)
TEMP Idx THIS::InsState(const std::string& rName, const StateAttr& attr) {
  FD_DG("TaGenerator(" << this << ")::InsState(\"" << rName << "\", attr)");
  Idx index=mStates.Insert();
  StateName(index,rName);
  StateAttribute(index,attr);
  return index;
}


// InjectState(index)
TEMP void THIS::InjectState(Idx index) {
  FD_DG("TaGenerator(" << this << ")::InjectState(\"" 
	<< SStr(index) << "\")");
  mStates.Insert(index);
}

// InjectStates(rNewStates)
TEMP void THIS::InjectStates(const StateSet& rNewStates) {
  FD_DG("TaGenerator(" << this << ")::InjectStates(" 
	<< rNewStates.ToString() << ")");
  mStates = rNewStates;
  mStates.Name("States");
}

// InsInitState()
TEMP Idx THIS::InsInitState(void) {
  FD_DG("TaGenerator(" << this << ")::InsInitState()");
  Idx index;
  index = InsState();
  mInitStates.Insert(index);
  return index;
}

// InsInitState(name)
TEMP Idx THIS::InsInitState(const std::string& rName) {
  FD_DG("TaGenerator(" << this << ")::InsInitState(\"" << rName << "\")");
  Idx index;
  index = InsState(rName);
  mInitStates.Insert(index);
  return index;
}

// InsMarkedState()
TEMP Idx THIS::InsMarkedState(void) {
  FD_DG("TaGenerator(" << this << ")::InsMarkedState()");
  Idx index;
  index = InsState();
  mMarkedStates.Insert(index);
  return index;
}

// InsMarkedState(rName)
TEMP Idx THIS::InsMarkedState(const std::string& rName) {
  FD_DG("TaGenerator(" << this << ")::InsMarkedState(\"" << rName << "\")");
  Idx index;
  index = InsState(rName);
  mMarkedStates.Insert(index);
  return index;
}

// DelState(index)
TEMP bool THIS::DelState(Idx index) {
  FD_DG("TaGenerator(" << this << ")::DelState(" << index << ")");
  // mInitStates
  mInitStates.Erase(index);
  // mstates
  mMarkedStates.Erase(index);
  // transrel 
  mTransRel.EraseByX1OrX2(index);
  // mStates + global
  mStateSymbolTable.ClrEntry(index);
  return mStates.Erase(index);
}

// DelState(rName)
TEMP bool THIS::DelState(const std::string& rName) {
  FD_DG("TaGenerator(" << this << ")::DelState(\"" << rName << "\")");
  Idx index;
  index = StateIndex(rName);
#ifdef FAUDES_CHECKED
  if (index == 0) {
    std::stringstream errstr;
    errstr << "state name \"" << rName << "\" not found in generator \""
	   << Name() << "\"";
    throw Exception("TaGenerator::DelState(name)", errstr.str(), 90);
  }
#endif
  return DelState(index);
}

// DelStates(rDelStates)
TEMP void THIS::DelStates(const StateSet& rDelStates) {
  FD_DG("TaGenerator(" << this << ")::DelStates(" 
	<< rDelStates.ToString() << ")");
  StateSet::Iterator cit;
  StateSet::Iterator cit_end;
  // symbolic state names
  if (StateNamesEnabled()) {
    for (cit = rDelStates.Begin(); cit != rDelStates.End(); ++cit) {
      mpStateSymbolTable->ClrEntry(*cit);
    }
  }
  // statesets
  mStates.EraseSet(rDelStates);
  mInitStates.EraseSet(rDelStates);
  mMarkedStates.EraseSet(rDelStates);
  // mTransRel:
  // should reimplement: iterate over transition rather than states
  for (cit = rDelStates.Begin(); cit != rDelStates.End(); ++cit) {
    mTransRel.EraseByX1OrX2(*cit);
  }
}

// DelStateFromStates(index)
TEMP bool THIS::DelStateFromStates(Idx index) {
  FD_DG("TaGenerator(" << this << ")::DelStateFromStates(" << index << ")");
  // mStates + global
  return mStates.Erase(index);
}

// DelStateFromStates(pos)
TEMP StateSet::Iterator THIS::DelStateFromStates(StateSet::Iterator pos) {
  FD_DG("TaGenerator(" << this << ")::DelState(" << *pos << ")");
  return mStates.Erase(pos);
}

// SetInitState(index)
TEMP void THIS::SetInitState(Idx index) {
  FD_DG("TaGenerator(" << this << ")::SetInitState(" << index << ")");
#ifdef FAUDES_CHECKED
  if (! mStates.Exists(index)) {
    std::stringstream errstr;
    errstr << "TaGenerator::SetMarkedState: index " << index 
	   << " not in stateset";
    throw Exception("TaGenerator::SetInitState(..)", errstr.str(), 91);
  }
#endif
  mInitStates.Insert(index);
}

// SetInitState(rName)
TEMP void THIS::SetInitState(const std::string& rName) {
  FD_DG("TaGenerator(" << this << ")::SetInitState(\"" << rName << "\")");
  Idx index = StateIndex(rName);
#ifdef FAUDES_CHECKED
  if (index == 0) {
    std::stringstream errstr;
    errstr << "State name \"" << rName << "\" not known in Generator";
    throw Exception("TaGenerator::SetInitState(..)", errstr.str(), 90);
  }
#endif
  SetInitState(index);
}

// InjectInitStates(rNewInitStates)
TEMP void THIS::InjectInitStates(const StateSet& rNewInitStates) {
  FD_DG("TaGenerator(" << this << ")::InjectInitStates(" 
	<< rNewInitStates.ToString() << ")");
  mInitStates = rNewInitStates;
  mInitStates.Name("InitStates");
}

// ClrInitState(index)
TEMP void THIS::ClrInitState(Idx index) {
  FD_DG("TaGenerator(" << this << ")::ClrInitState(" << index << ")");
#ifdef FAUDES_CHECKED
  if (! mStates.Exists(index)) {
    std::stringstream errstr;
    errstr << "TaGenerator::SetMarkedState: index " << index 
	   << " not in stateset";
    throw Exception("TaGenerator::ClrInitState(..)", errstr.str(), 91);
  }
#endif
  mInitStates.Erase(index);
}

// ClrInitState(rName)
TEMP void THIS::ClrInitState(const std::string& rName) {
  FD_DG("TaGenerator(" << this << ")::ClrInitState(\"" << rName << "\")");
  Idx index = StateIndex(rName);
#ifdef FAUDES_CHECKED
  if (index == 0) {
    std::stringstream errstr;
    errstr << "State name \"" << rName << "\" not known in Generator";
    throw Exception("TaGenerator::ClrInitState(..)", errstr.str(), 90);
  }
#endif
  ClrInitState(index);
}

// ClrInitState(pos)
TEMP void THIS::ClrInitState(StateSet::Iterator pos) {
  FD_DG("TaGenerator(" << this << ")::ClrInitState(" << *pos << ")");
  mInitStates.Erase(pos);
}

// ClearInitStates()
TEMP void THIS::ClearInitStates(void) {
  mInitStates.Clear();
}

// SetMarkedState(index)
TEMP void THIS::SetMarkedState(Idx index) {
  FD_DG("TaGenerator(" << this << ")::SetMarkedState(" << index << ")");
#ifdef FAUDES_CHECKED
  if (! mStates.Exists(index)) {
    std::stringstream errstr;
    errstr << "TaGenerator::SetMarkedState: index " << index 
	   << " not in stateset";
    throw Exception("TaGenerator::SetMarkedState(..)", errstr.str(), 91);
  }
#endif
  mMarkedStates.Insert(index);
}

// SetMarkedState(rName)
TEMP void THIS::SetMarkedState(const std::string& rName) {
  FD_DG("TaGenerator(" << this << ")::SetMarkedState(\"" << rName << "\")");
  Idx index = StateIndex(rName);
#ifdef FAUDES_CHECKED
  if (index == 0) {
    std::stringstream errstr;
    errstr << "State name \"" << rName << "\" not known in Generator";
    throw Exception("TaGenerator::SetMarkedState(..)", errstr.str(), 90);
  }
#endif
  SetMarkedState(index);
}

// InjectMarkedStates(rNewMarkedStates)
TEMP void THIS::InjectMarkedStates(const StateSet& rNewMarkedStates) {
  FD_DG("TaGenerator(" << this << ")::InjectMarkedStates(" 
	<< rNewMarkedStates.ToString() << ")");
  mMarkedStates = rNewMarkedStates;
  mMarkedStates.Name("MarkedStates");
}

// ClrMarkedState(index)
TEMP void THIS::ClrMarkedState(Idx index) {
  FD_DG("TaGenerator(" << this << ")::ClrMarkedState(" << index << ")");
#ifdef FAUDES_CHECKED
  if (! mStates.Exists(index)) {
    std::stringstream errstr;
    errstr << "TaGenerator::ClrMarkedState: index " << index 
	   << " not in stateset";
    throw Exception("TaGenerator::ClrMarkedState(..)", errstr.str(), 91);
  }
#endif
  mMarkedStates.Erase(index);
}

// ClrMarkedState(rName)
TEMP void THIS::ClrMarkedState(const std::string& rName) {
  FD_DG("TaGenerator(" << this << ")::ClrMarkedState(\"" << rName << "\")");
  Idx index = StateIndex(rName);
#ifdef FAUDES_CHECKED
  if (index == 0) {
    std::stringstream errstr;
    errstr << "State name \"" << rName << "\" not known in Generator";
    throw Exception("TaGenerator::ClrMarkedState(..)", errstr.str(), 90);
  }
#endif
  ClrMarkedState(index);
}

// ClrMarkedState(pos)
TEMP void THIS::ClrMarkedState(StateSet::Iterator pos) {
  FD_DG("TaGenerator(" << this << ")::ClrMarkedState(" << *pos << ")");
  mMarkedStates.Erase(pos);
}

// ClearMarkedStates()
TEMP void THIS::ClearMarkedStates(void) {
  mMarkedStates.Clear();
}

// InjectTransition(newtrans)
TEMP void THIS::InjectTransition(const Transition& rTrans) {
  FD_DG("TaGenerator::InjectTransition(" << TStr(rTrans) << ")");
  mTransRel.Insert(rTrans);
}

// InjectTransRel(newtransrel)
TEMP void THIS::InjectTransRel(const TransSet& newtransrel) {
  FD_DG("TaGenerator::InjectTransRel(...)");
  mTransRel=newtransrel;
}

// SetTransition(rX1, rEv, rX2)
TEMP bool THIS::SetTransition(const std::string& rX1, const std::string& rEv, const std::string& rX2) {
  FD_DG("TaGenerator(" << this << ")::SetTransition(\""
	<< rX1 << "\", \"" << rEv << "\", \"" << rX2 << "\")");
  Idx x1 = StateIndex(rX1);
  Idx x2 = StateIndex(rX2);
#ifdef FAUDES_CHECKED
  if (x1 == 0) {
    FD_ERR("TaGenerator::SetTransition: state " << rX1 
	   << " not in stateset");
    std::stringstream errstr;
    errstr << "State name " << rX1 << " not found in Generator";
    throw Exception("TaGenerator::SetTransition(..)", errstr.str(), 90);
  }
  if (! mAlphabet.Exists(rEv)) {
    FD_ERR("TaGenerator::SetTransition: event " << rEv << " not in alphabet");
    std::stringstream errstr;
    errstr << "Event name " << rEv << " not found in event domain of Generator";
    throw Exception("TaGenerator::SetTransition(..)", errstr.str(), 95);
  }
  if (x2 == 0) {
    FD_ERR("TaGenerator::SetTransition: state " << rX2 << " not in stateset");
    std::stringstream errstr;
    errstr << "State name " << rX2 << " not found in Generator";
    throw Exception("TaGenerator::SetTransition(..)", errstr.str(), 90);
  }
#endif
  return SetTransition(Transition(x1, EventIndex(rEv), x2));
}


// SetTransition(x1, ev, x2)
TEMP bool THIS::SetTransition(Idx x1, Idx ev, Idx x2) {
  return SetTransition(Transition(x1,ev,x2));
}

// SetTransition(rTransition)
TEMP bool THIS::SetTransition(const Transition& rTransition) {
  FD_DG("TaGenerator(" << this << ")::SetTransition(" << rTransition.X1 << "," 
	<< rTransition.Ev << "," << rTransition.X2 << ")");
#ifdef FAUDES_CHECKED
  if (! mStates.Exists(rTransition.X1)) {
    std::stringstream errstr;
    errstr << "TaGenerator::SetTransition: state " << rTransition.X1 
	   << " not in stateset";
    throw Exception("TaGenerator::SetTransition(..)", errstr.str(), 95);
  }
  if (! mAlphabet.Exists(rTransition.Ev)) {
    std::stringstream errstr;
    errstr << "TaGenerator::SetTransition: event " << rTransition.Ev 
	   << " not in alphabet ";
    throw Exception("TaGenerator::SetTransition(..)", errstr.str(), 95);
  }
  if (! mStates.Exists(rTransition.X2)) {
    std::stringstream errstr;
    errstr << "TaGenerator::SetTransition: state " << rTransition.X2 
	   << " not in stateset";
    throw Exception("TaGenerator::SetTransition(..)", errstr.str(), 95);
  }
#endif
  return mTransRel.Insert(rTransition);
}

// SetTransition(rTransition, rAttr)
TEMP bool THIS::SetTransition(const Transition& rTransition, const TransAttr& rAttr) {
  FD_DG("TaGenerator(" << this << ")::SetTransition(" << rTransition.X1 << "," 
	<< rTransition.Ev << "," << rTransition.X2 << ", [attr:]" << rAttr.ToString() << ")");
#ifdef FAUDES_CHECKED
  if (! mStates.Exists(rTransition.X1)) {
    std::stringstream errstr;
    errstr << "TaGenerator::SetTransition: state " << rTransition.X1 
	   << " not in stateset";
    throw Exception("TaGenerator::SetTransition(..)", errstr.str(), 95);
  }
  if (! mAlphabet.Exists(rTransition.Ev)) {
    std::stringstream errstr;
    errstr << "TaGenerator::SetTransition: event " << rTransition.Ev 
	   << " not in alphabet ";
    throw Exception("TaGenerator::SetTransition(..)", errstr.str(), 95);
  }
  if (! mStates.Exists(rTransition.X2)) {
    std::stringstream errstr;
    errstr << "TaGenerator::SetTransition: state " << rTransition.X2 
	   << " not in stateset";
    throw Exception("TaGenerator::SetTransition(..)", errstr.str(), 95);
  }
#endif
  return mTransRel.Insert(rTransition,rAttr);
}

// ClrTransition.X1, ev, x2)
TEMP void THIS::ClrTransition(Idx x1, Idx ev, Idx x2) {
  FD_DG("TaGenerator(" << this << ")::ClrTransition(" 
	<< x1 << "," << ev << "," << x2 << ")");
  mTransRel.Erase(x1, ev, x2);
}

// ClrTransition(rTransition)
TEMP void THIS::ClrTransition(const Transition& rTransition) {
  FD_DG("TaGenerator(" << this << ")::ClrTransition(" << TStr(rTransition) << ")");
  mTransRel.Erase(rTransition);
}

// ClrTransition(it)
TEMP typename THIS::ATransSet::Iterator THIS::ClrTransition(typename ATransSet::Iterator it) {
  FD_DG("TaGenerator(" << this << ")::ClrTransition(" << TStr(*it)<< ")" );
  return mTransRel.Erase(it);
}


// TransAttribute(trans, attr)
TEMP void THIS::TransAttribute(const Transition& rTrans, const TransAttr& rAttr)  {
  FD_DG("TaGenerator(" << this << ")::TransAttribute(" 
	<< TStr(rTrans) << ",\"" << rAttr.ToString() << "\")");
  mTransRel.Attribute(rTrans, rAttr);
}

// TransAttributep(trans)
TEMP TransAttr* THIS::TransAttributep(const Transition& rTrans)  {
  return mTransRel.Attributep(rTrans);
}


// TransAttribute(trans)
TEMP const TransAttr& THIS::TransAttribute(const Transition& rTrans) const {
  return mTransRel.Attribute(rTrans);
}

// ClrTransAttribute(trans)
TEMP void THIS::ClrTransAttribute(const Transition& rTrans)  {
  mTransRel.ClrAttribute(rTrans);
}


// ClearTransRel()
TEMP void THIS::ClearTransRel(void) {
  mTransRel.Clear();
}

// EventAttribute(index, attr)
TEMP void THIS::EventAttribute(Idx index, const EventAttr& rAttr) {
  FD_DG("TaGenerator(" << this << ")::EventAttribute(" 
	<< EStr(index) << ",\"" << rAttr.ToString() << "\")");
  mAlphabet.Attribute(index, rAttr);
}

// ClrEventAttribute(index)
TEMP void THIS::ClrEventAttribute(Idx index) {
  FD_DG("TaGenerator(" << this << ")::ClrEventAttribute(\"" << EStr(index) << "\")");
  mAlphabet.ClrAttribute(index);
}

// StateAttribute(index, attr)
TEMP void THIS::StateAttribute(Idx index, const StateAttr& rAttr) {
  FD_DG("TaGenerator(" << this << ")::StateAttribute(" 
	<< index << ",\"" << rAttr.ToString() << "\")");
  mStates.Attribute(index, rAttr);
}

// ClrStateAttribute(index)
TEMP void THIS::ClrStateAttribute(Idx index) {
  FD_DG("TaGenerator(" << this << ")::ClrStateAttribute(\"" << index << "\")");
  mStates.ClrAttribute(index);
}

// ExistsEvent(index)
TEMP bool THIS::ExistsEvent(Idx index) const {
  return mAlphabet.Exists(index);
}

// ExistsEvent(rName)
TEMP bool THIS::ExistsEvent(const std::string& rName) const {
  return mAlphabet.Exists(rName);
}

// FindEvent(index) const
TEMP EventSet::Iterator THIS::FindEvent(Idx index) const {
  return mAlphabet.Find(index);
}

// FindEvent(rName)
TEMP EventSet::Iterator THIS::FindEvent(const std::string& rName) const {
  return mAlphabet.Find(rName);
}

// ExistsState(index)
TEMP bool THIS::ExistsState(Idx index) const {
  return mStates.Exists(index);
}

// ExistsName(name)
TEMP bool THIS::ExistsState(const std::string& rName) const {
  return ExistsState(StateIndex(rName));
}

// FindState(rName) const
TEMP StateSet::Iterator THIS::FindState(const std::string& rName) const {
  return mStates.Find(mpStateSymbolTable->Index(rName));
}

// FindState(index) const
TEMP StateSet::Iterator THIS::FindState(Idx index) const {
  return mStates.Find(index);
}

// ExistsInitState(index)
TEMP bool THIS::ExistsInitState(Idx index) const {
  return mInitStates.Exists(index);
}

// FindInitState(index)
TEMP StateSet::Iterator THIS::FindInitState(Idx index) const {
  return mInitStates.Find(index);
}

// ExistsMarkedState(index)
TEMP bool THIS::ExistsMarkedState(Idx index) const {
  return mMarkedStates.Exists(index);
}

// FindMarkedState(index)
TEMP StateSet::Iterator THIS::FindMarkedState(Idx index) const {
  return mMarkedStates.Find(index);
}

// EventAttribute(index)
TEMP const EventAttr&  THIS::EventAttribute(Idx index) const {
  return mAlphabet.Attribute(index);
}

// EventAttributep(index)
TEMP EventAttr* THIS::EventAttributep(Idx index) {
  return mAlphabet.Attributep(index);
}

// EventAttribute(rName)
TEMP const EventAttr&  THIS::EventAttribute(const std::string& rName) const {
  return EventAttribute(EventIndex(rName));
}

// EventAttributep(rName)
TEMP EventAttr* THIS::EventAttributep(const std::string& rName) {
  return EventAttributep(EventIndex(rName));
}
// StateAttribute(index)
TEMP const StateAttr&  THIS::StateAttribute(Idx index) const {
  return mStates.Attribute(index);
}

// StateAttributep(index)
TEMP StateAttr*  THIS::StateAttributep(Idx index) {
  return mStates.Attributep(index);
}

// Alphabet()
TEMP const TaEventSet<EventAttr>& THIS::Alphabet(void) const {
  return mAlphabet;
}

// States()
TEMP const TaStateSet<StateAttr>& THIS::States(void) const {
  return mStates;
}

// TransRel()
TEMP const typename THIS::ATransSet& THIS::TransRel(void) const {
  return mTransRel;
}


// TransRel(res)
TEMP void THIS::TransRel(TransSetX1EvX2& res) const { mTransRel.ReSort(res); }
TEMP void THIS::TransRel(TransSetEvX1X2& res) const { mTransRel.ReSort(res); }
TEMP void THIS::TransRel(TransSetEvX2X1& res) const { mTransRel.ReSort(res); }
TEMP void THIS::TransRel(TransSetX2EvX1& res) const { mTransRel.ReSort(res); }
TEMP void THIS::TransRel(TransSetX2X1Ev& res) const { mTransRel.ReSort(res); }
TEMP void THIS::TransRel(TransSetX1X2Ev& res) const { mTransRel.ReSort(res); }


// InitStates()
TEMP const StateSet& THIS::InitStates(void) const {
  return mInitStates;
}

// MarkedStates()
TEMP const StateSet& THIS::MarkedStates(void) const {
  return mMarkedStates;
}

// MinimizeAlphabet()
TEMP void THIS::MinimizeAlphabet(void) {
  mAlphabet.NameSet::EraseSet(UnusedEvents());
}

// UsedEvents()
TEMP EventSet THIS::UsedEvents(void) const {
  EventSet resultset = NewEventSet();
  typename ATransSet::Iterator it;
  for (it = mTransRel.Begin(); it != mTransRel.End(); ++it) {
    resultset.Insert(it->Ev);
  }
  return resultset;
}

// UnusedEvents()
TEMP EventSet THIS::UnusedEvents(void) const {
  return mAlphabet - UsedEvents();
}

// ActiveEventSet(x1)
TEMP EventSet THIS::ActiveEventSet(Idx x1) const {
  EventSet result = NewEventSet();
  typename ATransSet::Iterator it;
  for (it = TransRelBegin(x1); it != TransRelEnd(x1); ++it) {
    result.Insert(it->Ev);
  }
  return result;
}

// ActiveTransSet(x1)
TEMP TransSet THIS::ActiveTransSet(Idx x1) const {
  TransSet result;
  typename ATransSet::Iterator it;
  for (it = TransRelBegin(x1); it != TransRelEnd(x1); ++it) {
    result.Insert(*it);
  }
  return result;
}

// TransRelStateSpace()
TEMP StateSet THIS::TransRelStateSpace(void) const {
  StateSet states = StateSet();
  typename ATransSet::Iterator it;
  for (it=mTransRel.Begin(); it != mTransRel.End(); ++it) {
    states.Insert(it->X1);
    states.Insert(it->X2);
  }
  return states;
}

// TransRelStateSpace(x1)
TEMP StateSet THIS::TransRelStateSpace(Idx x1) const {
  StateSet states = StateSet();
  typename ATransSet::Iterator it = mTransRel.Begin(x1);
  typename ATransSet::Iterator it_end = mTransRel.End(x1);
  for (; it != it_end; ++it) {
    states.Insert(it->X2);
  }
  return states;
}

// Version(idx)
TEMP void THIS::Version(Idx version, TaGenerator& rResGen) {
  FD_DG("TaGenerator(" << this << ")::Version(" << version << ")");
  std::ostringstream o;
  o << version;
  Version(o.str(),rResGen);
}

// Version(string)
TEMP void THIS::Version(const std::string& rVersion, TaGenerator& rResGen) {
  FD_DG("TaGenerator(" << this << ")::Version(" << rVersion << ")");
  // second arg must not be us if(this==&rResGen) {exception ..}
  if (this==&rResGen) {
    FD_ERR("TaGenerator::Version(): rResGen must not be this generator");
    abort();
  }
  // prepare Empty generator
  rResGen.Clear();
  rResGen.mGlobalAttribute=mGlobalAttribute;
  EventSet::Iterator eit;
  StateSet::Iterator lit;
  typename ATransSet::Iterator tit;
  std::map<Idx,Idx> eventoldnewmap;
  rResGen.Name(Name()+"_"+rVersion);
  // create versioned mAlphabet
  for (eit = AlphabetBegin(); eit != AlphabetEnd(); ++eit) {
    eventoldnewmap[*eit] = rResGen.InsEvent(EventName(*eit)+"_"+rVersion, EventAttribute(*eit)); 
  }
  // create new stateset
  rResGen.mStates=mStates;
  for (lit = StatesBegin(); lit != StatesEnd(); ++lit) {
    if (StateName(*lit) != "") {
      rResGen.StateName(*lit,StateName(*lit));
    }
  }
  // created versioned transrel
  for (tit = TransRelBegin(); tit != TransRelEnd(); ++tit) {
    rResGen.SetTransition(Transition(tit->X1, eventoldnewmap[tit->Ev], tit->X2), TransAttribute(*tit));
  }
  // set i/m states
  rResGen.mInitStates=mInitStates;
  rResGen.mMarkedStates=mMarkedStates;

  rResGen.mStateNamesEnabled = mStateNamesEnabled;
}

// AccessibleSet()
TEMP StateSet THIS::AccessibleSet(void) const {
  StateSet accessibleset = StateSet();
  StateSet::Iterator it;
  for (it = InitStatesBegin(); it != InitStatesEnd(); ++it) {
    CheckAccessible(accessibleset, *it);
  }
  accessibleset.Name("accessibleset");
  return accessibleset;
}

// Accessible()
TEMP bool THIS::Accessible(void) {
  StateSet accessibleset = AccessibleSet();
  StateSet not_accessible = mStates - accessibleset;
  StateSet::Iterator it;
  for(it = not_accessible.Begin(); it != not_accessible.End(); ++it){
    mTransRel.EraseByX1OrX2(*it);
  }
  // mStates
  mStates = accessibleset;
  mStates.Name("States");
  // mInitStates
  mInitStates = mInitStates * accessibleset;
  mInitStates.Name("InitStates");
  // mMarkedStates
  mMarkedStates = mMarkedStates * accessibleset;
  mMarkedStates.Name("MarkedStates");
  // symbolic state name handling
  if (StateNamesEnabled()) {
    for (it = not_accessible.Begin(); it != not_accessible.End(); ++it) {
      mpStateSymbolTable->ClrEntry(*it);
    }
  }
  // return true if there is an initial state
  if (! mInitStates.Empty()) {
    FD_DF("TaGenerator::accessible: generator is accessible");
    return true;
  }
  else {
    FD_DF("TaGenerator::accessible: generator is accessible");
    return false;
  }

}

// IsAccessible()
TEMP bool THIS::IsAccessible(void) const {
  if ((AccessibleSet() == mStates) && (! mInitStates.Empty())) {
    FD_DF("TaGenerator::accessible: generator is accessible");
    return true;
  }
  else {
    FD_DF("TaGenerator::accessible: generator is not accessible");
    return false;
  }
}

// CoaccessibleSet()
TEMP StateSet THIS::CoaccessibleSet(void) const {
  StateSet coaccessibleset = StateSet();
  StateSet::Iterator it;
  // build reverse transition relation
  TransSetX2EvX1 rtrel;
  TransRel(rtrel);
  for (it = MarkedStatesBegin(); it != MarkedStatesEnd(); ++it) {
    CheckCoaccessible(coaccessibleset, rtrel, *it);
  }
  coaccessibleset.Name("coaccessibleset");
  return coaccessibleset;
}

// Coaccessible()
TEMP bool THIS::Coaccessible(void) {
  StateSet coaccessibleset = CoaccessibleSet();
  StateSet not_coaccessible = mStates - coaccessibleset;
  StateSet::Iterator it;
  // mTransRel
  for(it = not_coaccessible.Begin(); it != not_coaccessible.End(); ++it){
    mTransRel.EraseByX1OrX2(*it);
  }
  // mStates
  mStates = coaccessibleset;
  mStates.Name("States");
  // mInitStates
  mInitStates = mInitStates * coaccessibleset;
  mInitStates.Name("InitStates");
  // mMarkedStates
  mMarkedStates = mMarkedStates * coaccessibleset;
  mMarkedStates.Name("MarkedStates");
  // symbolic state name handling
  if (StateNamesEnabled()) {
    for (it = not_coaccessible.Begin(); it != not_coaccessible.End(); ++it) {
      mpStateSymbolTable->ClrEntry(*it);
    }
  }
  // return true if there is a marked state
  if (! mMarkedStates.Empty()) {
    FD_DF("TaGenerator::coaccessible: generator is coaccessible");
    return true;
  }
  else {
    FD_DF("TaGenerator::coaccessible: generator is not coaccessible");
    return false;
  }
}

// IsCoaccessible()
TEMP bool THIS::IsCoaccessible(void) const {
  if ((CoaccessibleSet() == mStates) && (! mMarkedStates.Empty())) {
    FD_DF("TaGenerator::coaccessible: generator is coaccessible");
    return true;
  }
  else {
    FD_DF("TaGenerator::coaccessible: generator is not coaccessible");
    return false;
  }
}

// TrimSet()
TEMP StateSet THIS::TrimSet(void) const {
  FD_DF("TaGenerator::trimset: trim states: " 
	<< StateSet(AccessibleSet() * CoaccessibleSet()).ToString());
  return AccessibleSet() * CoaccessibleSet();
}

// Trim()
TEMP bool THIS::Trim(void) {
  FD_DF("TaGenerator::trim: generator states: " << mStates.ToString());
  bool accessiblebool = Accessible();
  bool coaccessiblebool = Coaccessible();
  FD_DF("TaGenerator::trim: trim states: " << mStates.ToString());
  if (accessiblebool && coaccessiblebool) {
    FD_DF("TaGenerator::trim: generator is trim");
    return true;
  }
  else {
    FD_DF("TaGenerator::trim: generator is not trim");
    return false;
  }
}

// IsTrim()
TEMP bool THIS::IsTrim(void) const {
  if (IsAccessible() && IsCoaccessible()) {
    FD_DF("TaGenerator::trim: generator is trim");
    return true;
  }
  else {
    FD_DF("TaGenerator::trim: generator is not trim");
    return false;
  }
}

// BlockingSet()
TEMP StateSet THIS::BlockingSet(void) const {
  FD_DF("TaGenerator::BlockingSet: blocking states: " 
	<< StateSet(AccessibleSet() - CoaccessibleSet()).ToString());
  return AccessibleSet() - CoaccessibleSet();
}

// CheckAccessible(accessibleset, startState)
TEMP void THIS::CheckAccessible(StateSet& accessibleset, Idx startState) const {
  if (! accessibleset.Exists(startState)) {
    accessibleset.Insert(startState);
    typename ATransSet::Iterator it = TransRelBegin(startState);
    typename ATransSet::Iterator it_end = TransRelEnd(startState);
    for (; it != it_end; ++it) {
      CheckAccessible(accessibleset, it->X2);
    }
  }
}

// CheckCoaccessible(coaccessibleset, rtransrel, startState)
TEMP void THIS::CheckCoaccessible(StateSet& coaccessibleset, 
				  const TransSetX2EvX1& rtrel, Idx startState) const {
  if (! coaccessibleset.Exists(startState)) {
    coaccessibleset.Insert(startState);
    typename TransSetX2EvX1::Iterator it = rtrel.BeginByX2(startState);
    typename TransSetX2EvX1::Iterator it_end = rtrel.EndByX2(startState);
    for (; it != it_end; ++it) {
      CheckCoaccessible(coaccessibleset, rtrel, it->X1);
    }
  }
}

// IsDeterministic()
TEMP bool THIS::IsDeterministic(void) const {
  // if there is more than one initial state ... nondet
  if (InitStatesSize() != 1) {
    FD_DG("TaGenerator::IsDeterministic: number of initial states not 1");
    return false;
  }
  // if there is a state/event pair with non-unique successor ... nondet
  if (TransRelSize() < 2) return true;
  typename ATransSet::Iterator it1;
  typename ATransSet::Iterator it2;
  for (it1 = TransRelBegin(), it2 = it1++; it1 != TransRelEnd(); it2 = it1++) {
    if ((it1->X1 == it2->X1) && (it1->Ev == it2->Ev)) {
      FD_WP("IsDeterministic: nondeterministic generator. at least one state "
	    << "contains more than on transition with same event: " 
	    << SStr(it1->X1) << ", event " << EStr(it1->Ev));
      return false;
    }
  }
  // in all other cases generator is deterministic
  return true;
}

// ClearMinStateIndexMap()
TEMP void THIS::ClearMinStateIndexMap(void) const {
  FD_DG("TaGenerator::ClearMinStateIndexMap()");
  // fake const
  THIS* fakeconst = const_cast<THIS*>(this);
  fakeconst->mMinStateIndexMap.clear();
}

// MinStateIndex(index)
TEMP Idx THIS::MinStateIndex(Idx index) const {
  std::map<Idx,Idx>::const_iterator minit;
  minit = mMinStateIndexMap.find(index);
  Idx fidx;
  if(minit != mMinStateIndexMap.end()) {
    fidx= minit->second;
  } else {
    fidx=index;
  }
  return fidx;
}

// MinStateIndex()
TEMP const std::map<Idx,Idx>& THIS::MinStateIndexMap(void) const {
  return mMinStateIndexMap;
}



// SetMinStateIndexMap()
TEMP void THIS::SetMinStateIndexMap(void) const {
  FD_DG("TaGenerator::SetMinStateIndexMap()");
  // fake const
  THIS* fakeconst = const_cast<THIS*>(this);
  // clear map
  fakeconst->ClearMinStateIndexMap();
  StateSet::Iterator it;
  Idx minindex = 0;
  // if generator states get names
  if(StateNamesEnabled()) {
    // named initial states first
    for(it = InitStatesBegin(); it != InitStatesEnd(); ++it) {
      if(StateName(static_cast<Idx>(*it)) != "") {
	fakeconst->mMinStateIndexMap[*it] = ++minindex;
      }
    }
    // then all other named states
    for(it = StatesBegin(); it != StatesEnd(); ++it) {
      if(mMinStateIndexMap.count(*it) == 0) {
	if(StateName(static_cast<Idx>(*it)) != "") {
	  fakeconst->mMinStateIndexMap[*it] = ++minindex;
	}
      }
    }
    // at last all anonymous states
    for(it = StatesBegin(); it != StatesEnd(); ++it) {
      if(mMinStateIndexMap.count(*it) == 0) {
	fakeconst->mMinStateIndexMap[*it] = ++minindex;
      }
    }
  }
  // if generator states are all anonymous
  else {
    // all initial states first
    for(it = InitStatesBegin(); it != InitStatesEnd(); ++it) {
      fakeconst->mMinStateIndexMap[*it] = ++minindex;
    }
    // then the rest
    for(it = StatesBegin(); it != StatesEnd(); ++it) {
      if(mMinStateIndexMap.count(*it) == 0) {
	fakeconst->mMinStateIndexMap[*it] = ++minindex;
      }
    }
  }
#ifdef FAUDES_DEBUG_CONTAINER
  std::map<Idx,Idx>::const_iterator _it;
  for(_it = mMinStateIndexMap.begin(); _it != mMinStateIndexMap.end(); ++_it) {
    FD_DC("TaGenerator::MinStateIndexMap: " << _it->first
	  << " <-- " << SStr(_it->second));
  }
#endif
}

#undef THIS
#undef TEMP
#undef BASE


/**
 *
 *  A Generator pool maintains a global generator context.
 *
 *  THIS CLASS NOT IMPLEMENTED. ITS CURRENT "INTERFACE CANDIDATE".
 *
 */

class GeneratorPool {
 public:
  /** constructor */
  GeneratorPool(void) {
    mEventSymbolTablep=SymbolTable::GlobalEventSymbolTablep();
  }

  /**
   * Create empty Generator with event symboltable from pool
   * (this will never work)
   *
   * @return
   *   New Generator
   */
  template <class AnyGenerator>
    AnyGenerator NewGenerator(void) { 
    AnyGenerator gen;
    gen.EventSymbolTablep(mEventSymbolTablep);
    return gen;
  }    

  /**
   * Create empty Generator with event symboltable from pool
   * (this will work some day)
   *
   * @return
   *   New Generator
   */
  template <class AnyGenerator>
    AnyGenerator* NewGeneratorp(void) { 
    AnyGenerator* gen = new AnyGenerator;
    gen->EventSymbolTablep(mEventSymbolTablep);
    return gen;
  }    

  /**
   * Create empty Eventset with event symboltable from pool
   *
   * @return
   *   New Alphabet
   */
  template <class AnyEventSet>
    AnyEventSet NewAlphabet(void) { 
    AnyEventSet eset;
    eset.SymbolTablep(mEventSymbolTablep);
    return eset;
  }    

 private:
  /** eventsymbnoltable shared by all generators from this pool */ 
  SymbolTable mEventSymbolTable;
  SymbolTable* mEventSymbolTablep;
};


} // namespace faudes

#endif

