/** @file attributes.cpp Classes AttributeVoid and AttributeFlags  */

/* FAU Discrete Event Systems Library (libfaudes)

   Copyright (C) 2006  Bernd Opitz
   Exclusive copyright is granted to Klaus Schmidt

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */


#include "attributes.h"


namespace faudes {

/***********************************************************************************
 *
 * implementation of AttributeVoid 
 *
 */

// Write(label,context)
void AttributeVoid::Write(const std::string& rLabel, const Type* pContext) const {
  TokenWriter tw(TokenWriter::Stdout);
  DoWrite(tw,rLabel,pContext);
}

// Write(rFileName, label, context, openmode)
void AttributeVoid::Write(const std::string& rFileName, const std::string& rLabel, 
  const Type* pContext, std::ios::openmode openmode) const {
  try {
    TokenWriter tw(rFileName, openmode);
    DoWrite(tw, rLabel, pContext);
  }
  catch (std::ios::failure&) {
    std::stringstream errstr;
    errstr << "Exception opening/writing file \"" << rFileName << "\"";
    throw Exception("AttributeVoid::Write", errstr.str(), 2);
  }
}

// Write(tw,label,context)
void AttributeVoid::Write(TokenWriter& rTw, const std::string& rLabel, const Type* pContext) const {
  DoWrite(rTw,rLabel,pContext);
}

// ToString(label, context)
std::string AttributeVoid::ToString(const std::string& rLabel, const Type* pContext) const {
  TokenWriter tw(TokenWriter::String);
  DoWrite(tw,rLabel,pContext);
  return tw.Str();
}

// Read(rFilename, rLabel,context)
void AttributeVoid::Read(const std::string& rFilename, const std::string& rLabel, const Type* pContext) {
  TokenReader tr(rFilename);
  DoRead(tr,rLabel,pContext);
  Skip(tr);
}

// Read(rTr, rLabel, context)
void AttributeVoid::Read(TokenReader& rTr, const std::string& rLabel, const Type* pContext) {
  DoRead(rTr,rLabel,pContext);
  Skip(rTr);
}


//DoWrite(rTr,rLabel,pContext)
void AttributeVoid::DoWrite(TokenWriter& rTw, const std::string& rLabel, const Type* pContext) const {
  (void) rTw; (void) rLabel; (void) pContext;
  FD_DC("AttributeVoid::DoWrite()");
}

//DoRead(rTr,rLabel,pContext)
void AttributeVoid::DoRead(TokenReader& rTr, const std::string& rLabel, const Type* pContext) {
  (void) rLabel; (void) pContext; (void) rTr;
  FD_DC("AttributeVoid::DoRead()");
}

//Skip(rTr)
void AttributeVoid::Skip(TokenReader& rTr) {
  FD_DC("AttributeVoid::Skip()");
  Token token;
  while(rTr.Peek(token)) {
    // break on index or name, since this is the next element
    if((token.Type()==Token::String) || (token.Type()==Token::Integer)) {
      break;
    }
    // break on end, since this is the end of the set
    if(token.Type()==Token::End) {
      break;
    }
    // break on Consecutive section, since this belongs to the set
    if((token.Type()==Token::Begin) && (token.StringValue() == "Consecutive")) {
      break;
    }
    // skip any attribute section from other file format
    if(token.Type()==Token::Begin){
      rTr.ReadBegin(token.StringValue());
      rTr.ReadEnd(token.StringValue());
      continue;
    }  
    // skip any other token from other file format
    rTr.Get(token);
  }
}



/***********************************************************************************
 *
 * implementation of AttributeFlags 
 *
 */

//DoWrite(rTw)
// Note: you should write attributes in a section, so that
// the AttributeVoid read method can detect and skip them.
// Here, we make an execption of the rule ...
void AttributeFlags::DoWrite(TokenWriter& rTw,const std::string& rLabel, const Type* pContext) const {
  (void) rLabel; (void) pContext;
  if(!IsDefault()) {
    FD_DC("AttributeFlags(" << this << ")::DoWrite(tr)");
    Token token;
    token.SetInteger16(mFlags);
    rTw << token;
  }
}


//DoRead(rTr)
void AttributeFlags::DoRead(TokenReader& rTr, const std::string& rLabel, const Type* pContext) {
  (void) rLabel; (void) pContext;
  FD_DC("AttributeFlag(" << this << ")::DoRead(tr)");
  Token token;
  rTr.Peek(token);
  if(token.Type()==Token::Integer16) { 
    rTr.Get(token);
    mFlags=token.IntegerValue();
  } else {
    mFlags=mDefFlags;
  }
}


} // namespace
