/** @file attributes.h Classes AttributeVoid and AttributeFlags  */


/* FAU Discrete Event Systems Library (libfaudes)

Copyright (C) 2007  Thomas Moor
Exclusive copyright is granted to Klaus Schmidt

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */


#ifndef FAUDES_ATTRIBUTES_H
#define FAUDES_ATTRIBUTES_H

#include "interface.h"
#include "tokenreader.h"
#include "tokenwriter.h"

namespace faudes {


/**
 * Minimal Attribute. Attributes are used as template parameters for
 * faudes containers and generators and facilitate the modelling of customized  
 * properties of events, states and transitions.  The AttributeVoid class defines
 * the minimal interface of faudes attributes and therefore is the designated
 * base class for all attribute implementations. The AttributeVoid class does
 * not define any actual properties. See AttributeFlags for a non-trivial example.
 */

class AttributeVoid {
 public:

  /** Constructor */
  AttributeVoid(void) {};

  /** Destructor */
  virtual ~AttributeVoid(void) {};

  /**
   * Write attribute to console.
   * Note: all write functions use the virtual function DoWrite(), to be 
   * reimplemented by derived attributes.
   *
   * @param rLabel
   *   Label of section to write
   * @param pContext
   *   Write context to provide contextual information
   *
   */
  void Write(const std::string& rLabel="", const Type* pContext=0) const;

  /**
   * Write attribute to a file. 
   * Note: all write functions use the virtual function DoWrite(), to be 
   * reimplemented by derived attributes.
   *
   * @param pFileName
   *   Name of file
   * @param rLabel
   *   Label of section to write
   * @param pContext
   *   Write context to provide contextual information
   * @param openmode
   *   ios::openmode 
   *
   * @exception Exception
   *   - IO errors (id 2)
   */
  void Write(const std::string& pFileName, const std::string& rLabel="",
	     const Type* pContext=0, std::ios::openmode openmode = std::ios::out|std::ios::trunc) const;

  /**
   * Write attribute to TokenWriter.
   * Note: all write functions use the virtual function DoWrite(), to be 
   * reimplemented by derived attributes.
   *
   * @param rTw
   *   Reference to TokenWriter
   * @param rLabel
   *   Label of section to write
   * @param pContext
   *   Write context to provide contextual information
   *
   * @exception Exception 
   *   - IO errors (id 2)
   */
  void Write(TokenWriter& rTw, const std::string& rLabel="",const Type* pContext=0) const;

  /**
   * Write attribute to a string.
   * Note: all write functions use the virtual function DoWrite(), to be 
   * reimplemented by derived attributes.
   *
   * @param rLabel
   *   Label of section to write
   * @param pContext
   *   Write context to provide contextual information
   * @return
   *   output string
   * @exception Exception
   *   - IO errors (id 2)
   */
  std::string ToString(const std::string& rLabel="", const Type* pContext=0) const;

  /**
   * Read from file with label specified.
   * Note: all read functions use the virtual function DoRead(), to be 
   * reimplemented for by derived attribute classes.
   *
   * @param rFileName
   *   Name of file
   * @param rLabel
   *   Section to read from 
   * @param pContext
   *   Read context to provide contextual information
   *
   * @exception Exception
   *   - IO errors (id 1)
   *   - token mismatch from DoRead()
   */
  void Read(const std::string& rFileName, const std::string& rLabel = "", const Type* pContext=0);

  /**
   * Read from TokenReader with label sepcified.
   * Note: all read functions use the virtual function DoRead(), to be 
   * reimplemented for by derived attribute classes.
   *
   * @param rTr
   *   Reference to tokenreader
   * @param rLabel
   *   Section to read
   * @param pContext
   *   Read context to provide contextual information
   *
   * @exception Exception
   *   - IO errors (id 1)
   *   - token mismatch from DoRead()
   */
  void Read(TokenReader& rTr, const std::string& rLabel = "", const Type* pContext=0);


  /** 
   * Test for default value
   */
  bool IsDefault(void) const {return true;};

  /**
   *   Set to fefault value
   */
  void SetDefault(void) {};

  /**
   *   Synonym for SetDefault to match std interface
   */
  void Clear(void) { SetDefault(); };

 protected:

  /**
   * Actual read method to read attribute from tokenreader.
   *
   * For the base class AttributeVoid, this method skips all tokens until a String or 
   * Integer token indicates the end of all attribute values. 
   *
   * For derived attributue classes, this method must either read all tokens that
   * belong to the respective attribute, or none. It may thorw exceptions on token
   * mismatch within the relevant attribute, but it may not throw exceptions when
   * it encounters tokens that possibly belong to another attribute. 
   *
   * @param rTr
   *   TokenReader to read from
   * @param rLabel
   *   Section to read
   * @param pContext
   *   Read context to provide contextual information
   *
   * @exception Exception
   *   - IO error (id 1)
   */
  virtual void DoRead(TokenReader& rTr,  const std::string& rLabel = "", const Type* pContext=0);
 
  /**
   * Actual write method to write the attribute to a TokenWriter.
   *
   * Reimplement this method for derived attribute classes to define the token io 
   * format. 
   *
   * @param rTw
   *   Reference to TokenWriter
   * @param rLabel
   *   Label of section to write
   * @param pContext
   *   Write context to provide contextual information
   *
   * @exception Exception 
   *   - IO errors (id 2)
   */
  virtual void DoWrite(TokenWriter& rTw, const std::string& rLabel="",const Type* pContext=0) const;

  /**
   * Skip attribute tokens.
   *
   * Helper method to be called after all sttribute derived classes had their
   * to read their data. It skips all tokens and sections until it reached a 
   * String or decimal Integer.
   *
   * @param rTr
   *   TokenReader to read from
   * @exception Exception
   *   - IO error (id 1)
   */
  void Skip(TokenReader& rTr);

 

}; // class AttributeVoid


/** Convenience typdef flag data */
typedef unsigned long int fType;

/**
 * Boolean flags Attribute. This attribute class uses a flag word to represent 
 * boolean values. The current implementation uses a long int type and hence handles
 * up to 32 flags. Each flag is accessed by the according bit mask. 
 *
 * The current version
 * of libFAUDES uses bits 0,1,2 of event attributes for controllability properties 
 * (see AttributeCFlags) and the bits 31 and 32 of state attributes for the Qt based
 * GUI project. Further versions may use the lower 8 bits of the event aflags and the higher 8
 * bits of state flags. You are free to use any flags for your application, but if possible try
 * to avoid the above mentioned. For extensive use of flags, you may also consider to
 * define a separate attribute class, perhaps with a different fType.
 */



class AttributeFlags : public AttributeVoid {
 public:

  /** Default constructor */
  AttributeFlags(void) : mFlags(mDefFlags) {};

  /** Destructor */
  virtual ~AttributeFlags(void) {};

  /**
   * Test a flag
   */
  bool Test(fType mask) const { return ( (mFlags & mask) != 0 ); };

  /**
   * Test multible flags, combine by "and"
   */
  bool TestAll(fType mask) const { return ( (mFlags & mask) == mask ); };

  /**
   * Test multible flags, combine by "or"
   */
  bool TestSome(fType mask) const { return ( (mFlags & mask) != 0 ); };

  /**
   * Test multible flags, combine by "not or"
   */
  bool TestNone(fType mask) const { return ( (mFlags & mask) == 0 ); };

  /**
   * Set multiple flags
   */
  void Set(fType mask) {mFlags |= mask; };

  /**
   * Clear multiple flags
   */
  void Clr(fType mask) {mFlags &= ~mask; };

  /** 
   * Test for default value
   */
  bool IsDefault(void) const {return mFlags==mDefFlags;};

  /** Flags (public access for convenience)  */
  fType mFlags;

  /* default flags */
  const static fType mDefFlags=0x0;


protected:

  /**
   * Reads attribute from TokenReader, see AttributeVoid for public wrappers.
   *
   * Test whether the current token is an integer with base 16. If so, read the token to the
   * flags value. Else, dont take any tokens. The label and context arguments are ignored.
   *
   * @param rTr
   *   TokenReader to read from
   * @param rLabel
   *   Section to read
   * @param pContext
   *   Read context to provide contextual information
   *
   * @exception Exception
   *   - IO error (id 1)
   */
  virtual void DoRead(TokenReader& rTr,  const std::string& rLabel = "", const Type* pContext=0);
 
  /**
   * Write to TokenWriter, see AttributeVoid for public wrappers.
   *
   * If not the defult value, write the flag value as base 16 integer token. Else,
   * do nothing. The label and context arguments are ignored.
   *
   * @param rTw
   *   Reference to TokenWriter
   * @param rLabel
   *   Label of section to write
   * @param pContext
   *   Write context to provide contextual information
   *
   * @exception Exception 
   *   - IO errors (id 2)
   */
  virtual void DoWrite(TokenWriter& rTw, const std::string& rLabel="",const Type* pContext=0) const;



}; // class AttributeFlags


} // namespace faudes

#endif
