/** @file baseset.h @brief Class TBaseSet */


/* FAU Discrete Event Systems Library (libfaudes)

   Copyright (C) 2008  Thomas Moor
   Exclusive copyright is granted to Klaus Schmidt

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */



#ifndef FAUDES_BASESET_H
#define FAUDES_BASESET_H

#include "definitions.h"
#include "interface.h"
#include "tokenwriter.h"
#include "tokenreader.h"
#include <set>
#include <algorithm>

namespace faudes {

/** @addtogroup ContainerClasses */
/** @{*/

/**
 * STL style set template.
 * This class template is built on top of the STL set template. It provides essentials of 
 * the STL interface relevant to libFAUDES plus a deferred copy mechanism, aka copy-on-write. 
 * TBaseSet serves as a base class for all libFaudes containers: 
 * - IndexSet (indices), 
 * - TaIndexSet (indices with attributes), 
 * - SymbolSet (symbolic names), 
 * - NameSet (indices with symbolic names), 
 * - TaNameSet (indices with attributes and symbolic names), 
 * - TTransSet (transitions in a sepcific order),
 * - TaTransSet (transitions is std order with attribute).
 *
 * The public functions of a TBaseSet provide the high-level api, with the intention
 * to organize the deferred copy machanism in a transparent manner. Since STL iterators
 * refer to a particular STL container, they become invalid when the internal container 
 * is copied. Therefor TBaseSet tracks iterators and fixes them when the actual copy takes place.
 * Clearly, this introduces some overhead, in particular when your application represents
 * subsets as sets of iterators. You may use the public method Lock() to enforce 
 * a full copy and to prevent any further re-allocation.
 *
 * Alternatively to the high-level api, a protected low-level api is provided with direct 
 * access to the internal STL set. When using this api, it is up to the derived class to ensure
 * that the BaseSet gets detached from its refernces befor write operations can take
 * place. 
 *
 * The virtual function TBaseSet<T>::Valid() is used to indicate whether a candidate
 * element is valid as a set member. If the macro FAUDES_CHECKED
 * is defined, the attempt to insert an invalid element triggers an exception (id 61). Invalid
 * iterators throw an exception (id 62) when used as an argument to a BaseSet function. 
 *
 *
 * Note on a boring technical detail: 
 * since STL sets are sorted, effectively all set iterators should be const.
 * However, there is a minor issue whether or not the erase function should use a 
 * const iterator as argument. SGI derived STL implementations (as used on most GNU systems) 
 * avoid this issue by defining const and non-const iterators on sets as identical types. 
 * MS implementation (used in VS C++ by default) differ in this aspect. The class 
 * TBaseSet::Iterator hides the issue from the faudes API but it is still present internaly: 
 * in its current implementation, libfaudes will not compile with MS STL. At this stage we are 
 * aware of two workarounds, one simple but on cost of performance, one a little bit awkward. 
 * Let us know if MS STL is missioncrititical for your libfaudes application.
 */

template<class T, class Cmp=std::less<T> >
  class TBaseSet : FAUDES_TYPE {

public:

  /**
   * Constructor. 
   */
  TBaseSet(void);

  /**
   * Copy-constructor. 
   *
   * @param rOtherSet 
   *    Source to copy from
   */
  TBaseSet(const TBaseSet& rOtherSet);

  /**
   * Virtual destructor
   */
  virtual ~TBaseSet(void);

  /** 
   * Return name of TBaseSet 
   * 
   * @return
   *   Name of TBaseSet
   */
  std::string Name(void) const;
        
  /**
   * Set name of TBaseSet
   *
   * @param rName
   *   Name to set
   */
  void Name(const std::string& rName);

  /** 
   * Clear all set
   */
  virtual void Clear(void);
      
  /**
   * Get Size of TBaseSet
   *
   * @return 
   *   Number of indices in TBaseSet
   */
  Idx Size(void) const;

  /** 
   * Check if the TBaseSet ist Empty 
   *
   * @return 
   *   True if empty
   */
  bool Empty(void) const;

  /** 
   * Iterator class for high-level api to TBaseSet.
   *
   */
  class Iterator;

  /** 
   * Iterator to the begin of set
   *
   * @return 
   *   Iterator
   */
  Iterator Begin(void) const;
        
  /** 
   * Iterator to the end of set
   *
   * @return 
   *   Iterator
   */
  Iterator End(void) const;

  /**
   * Test validty of candidate element.
   *
   * Reimplement this function for particular type T of elements,
   * eg for an index set with T=Idx indicate 0 an invalid index.
   *
   * @param rElem
   *    Candidate to test
   * @return
   *    True if element is valid
   */
  virtual bool Valid(const T& rElem) const;
   

  /** 
   * Erase element by reference
   *
   * @param rElem
   *    Element to erase
   * @return
   *    True if element used to exist
   */
  virtual bool Erase(const T& rElem);


  /** 
   * Erase element by iterator 
   *
   * @param pos
   *    Iterator to specify element
   * @return 
   *    Iterator to next element (or End if no such)
   */
  virtual Iterator Erase(const Iterator& pos); 


  /** 
   * Erase elements given by other set 
   *
   * @param rOtherSet 
   *    Set of elements to erase
   */
  virtual void EraseSet(const TBaseSet& rOtherSet);


  /** 
   * Insert specified element
   *
   * @param rElem
   *    Element to insert
   * @return
   *   True if index was new to set 
   */
  virtual bool Insert(const T& rElem);

  /**
   * Insert elements given by rOtherSet 
   *
   *
   * @param rOtherSet
   *   Set of elements to insert
   */
  virtual void InsertSet(const TBaseSet& rOtherSet);

  /**
   * Set Union, result is accumulated in this set.
   *
   * @param rOtherSet
   *   Other TBaseSet
   */
  virtual void SetUnion(const TBaseSet& rOtherSet);

  /**
   * Set Intersection, result is stored in this set.
   * 
   * @param rOtherSet
   *   Other TBaseSet
   */
  virtual void SetIntersection(const TBaseSet& rOtherSet);


  /**
   * Test existence of element
   *
   * @param rElem
   *   Element to test
   *
   * @return 
   *   True if element exists in set
   */
  bool Exists(const T& rElem) const;

  /**
   * Find element and return iterator.
   *
   * @param rElem
   *   Element to find
   *
   * @return 
   *   Iterator to element or End() if nonexistent
   */
  Iterator Find(const T& rElem) const;

  /**
   * Set union operator
   *
   * @return 
   *   Union Set
   */
   TBaseSet operator + (const TBaseSet& rOtherSet) const;

  /**
   * Set difference operator
   *
   * @return 
   *   Difference Set
   */
   TBaseSet operator - (const TBaseSet& rOtherSet) const;

  /**
   * Set intersection operator
   *
   * @return 
   *   Intersection Set
   */
  TBaseSet operator * (const TBaseSet& rOtherSet) const;


  /** Test for equality  */
  bool operator == (const TBaseSet& rOtherSet) const;

  /** Test for inequality  */
  bool operator != (const TBaseSet& rOtherSet) const;

  /** Test for subset  */
  bool operator <= (const TBaseSet& rOtherSet) const;

  /** Test for superset */
  bool operator >= (const TBaseSet& rOtherSet) const;

  /** Order for sorting containers of TBaseSet */
  bool operator < (const TBaseSet& rOtherSet) const;

  /** Assignment operator (fake copy) */
  virtual const TBaseSet& operator=(const TBaseSet& rSrc) {return Assign(rSrc);};

  /** Some validation of deferred copy mechanism (provole abort)*/
  void DValid(const std::string& rMessage="") const;

  /** Detach from extern storage (incl allocation and true copy) */
  void Detach(void) const;

  /** Detach and lock any further reallocation */
  void Lock(void) const;

  /** 
   * Iterator class for high-level api to TBaseSet.
   * This class is derived from STL interators to additionally provide
   * a reference of the container to iterate on. This feature
   * is used to adjust iterators when the actual set gets reallocated due to a Detach()
   * operation. Inheritance is private to ensure that all high-level api functions maintain
   * iteretor refernces consistently. Currently, high-level api iterators support
   * the operators -&gt; ,*,  =, ++, --, ==, !=.
   * 
   * HotFix: turned to public inheritance to compile with stl/g++ 4.3. Should investigate
   * this in detail and explicitly make the required data public while having
   * inheritance private in general.
   */
   class Iterator : public std::set<T,Cmp>::const_iterator {
     public: 
     /** Default contructor, container unknown */
     Iterator() : 
       std::set<T,Cmp>::const_iterator() , 
       pBaseSet(NULL),
       mAttached(false) 
     {};

     /** Construct by members, used by TBaseSet to create temporary iterators */
     Iterator(
       const TBaseSet<T,Cmp>* pBaseSet,
       const typename std::set<T,Cmp>::const_iterator& sit) : 
       std::set<T,Cmp>::const_iterator(sit), 
       pBaseSet(pBaseSet),
       mAttached(false)  
     {};

     /** Copy constructor, copies container reference, incl attach */
     Iterator(const Iterator& fit) : 
       std::set<T,Cmp>::const_iterator(fit),
       pBaseSet(fit.pBaseSet), 
       mAttached(false) 
     {
       if(pBaseSet) {
          pBaseSet->AttachIterator(this);
          mAttached=true;
       }
     };  

     /** Destructor */
     ~Iterator(void) { 
       if(mAttached) pBaseSet->DetachIterator(this);
     }; 

     /** Assigment, tracks container */
     const Iterator& operator= (const Iterator& rSrc) {
#ifdef FAUDES_DEBUG_CODE
       if(rSrc.pBaseSet==NULL) {
         FD_ERR("TBaseSet<T,Cmp>::Iterator(" << this << "):operator= invalid iterator: no baseset");
         abort();
       }
#endif
       // performance relevant std case 
       if(mAttached) if(pBaseSet==rSrc.pBaseSet) {
         std::set<T,Cmp>::const_iterator::operator= (rSrc);
         return *this;
       }
       // general case
       if(mAttached) pBaseSet->DetachIterator(this);
       std::set<T,Cmp>::const_iterator::operator= (rSrc);
       pBaseSet = rSrc.pBaseSet;
       if(pBaseSet) {
         pBaseSet->AttachIterator(this);
         mAttached=true;
       } else {
 	mAttached=false;
       }
       return *this;
     };

     /** Assign STL iterator only */
     void  StlIterator(const typename std::set<T,Cmp>::const_iterator& sit) {
       std::set<T,Cmp>::const_iterator::operator= (sit);
     };

     /** Get STL iterator only */
     const typename std::set<T,Cmp>::const_iterator& StlIterator(void) const {
       return *this;
     };

     /** Invalidate */
     void  Invalidate(void) {
       pBaseSet=NULL;
       mAttached=false;
     }; 

     /** Detach */
     void  Detach(void) {
       mAttached=false;
     }; 


     /** Check validity (provoke abort error) */
     void DValid(void) const {
      if(pBaseSet==NULL) {
         FD_ERR("TBaseSet<T,Cmp>::Iterator(" << this << "):DValid(): invalid iterator: no baseset");
         abort();
       }
       pBaseSet->DValid();
     }; 

    /** Reimplement dereference */
     const T* operator-> (void) const {
#ifdef FAUDES_DEBUG_CODE
       if(pBaseSet==NULL) {
         FD_ERR("TBaseSet<T,Cmp>::Iterator(" << this << "):operator->: invalid iterator: no baseset");
         abort();
       }
#endif
       return std::set<T,Cmp>::const_iterator::operator-> ();
     };

     /** Reimplement derefernce */
     const T& operator* (void) const {
#ifdef FAUDES_DEBUG_CODE
       if(pBaseSet==NULL) {
         FD_ERR("TBaseSet<T,Cmp>::Iterator(" << this << "):operator*: invalid iterator: no baseset");
         abort();
       }
#endif
       return std::set<T,Cmp>::const_iterator::operator* ();
     };

     /** Reimplement == */
     bool operator == (const Iterator& rOther) const {
#ifdef FAUDES_DEBUG_CODE
       if(pBaseSet==NULL) {
         FD_ERR("TBaseSet<T,Cmp>::Iterator(" << this << "):operator==: invalid iterator: no baseset");
         abort();
       }
#endif
       return std::set<T,Cmp>::const_iterator::operator == (rOther);
     };

     /** Reimplement != */
     bool operator != (const Iterator& rOther) const {
#ifdef FAUDES_DEBUG_CODE
       if(pBaseSet==NULL) {
         FD_ERR("TBaseSet<T,Cmp>::Iterator(" << this << "):operator!=: invalid iterator: no baseset");
         abort();
       }
#endif
       return std::set<T,Cmp>::const_iterator::operator != (rOther);
     };

     /** Reimplement postfix ++ */
     Iterator operator++ (int step) {
#ifdef FAUDES_DEBUG_CODE
       if(pBaseSet==NULL) {
         FD_ERR("TBaseSet<T,Cmp>::Iterator(" << this << "):operator++: invalid iterator: no baseset");
         abort();
       }
#endif
       Iterator old(pBaseSet,*this);
       std::set<T,Cmp>::const_iterator::operator++ (step);
       return old;
     };

     /** Reimplement prefix ++ */
     const Iterator& operator++ (void) {
#ifdef FAUDES_DEBUG_CODE
       if(pBaseSet==NULL) {
         FD_ERR("TBaseSet<T,Cmp>::Iterator(" << this << "):operator++: invalid iterator: no baseset");
         abort();
       }
#endif
       std::set<T,Cmp>::const_iterator::operator++ ();
       return *this;
     };

     /** Reimplement postfix -- */
     Iterator operator-- (int step) {
#ifdef FAUDES_DEBUG_CODE
       if(pBaseSet==NULL) {
         FD_ERR("TBaseSet<T,Cmp>::Iterator(" << this << "):operator--: invalid iterator: no baseset");
         abort();
       }
#endif
       Iterator old(pBaseSet, *this);
       std::set<T,Cmp>::const_iterator::operator-- (step);
       return old;
     };

     /** Reimplement prefix -- */
     const Iterator& operator-- (void) {
#ifdef FAUDES_DEBUG_CODE
       if(pBaseSet==NULL) {
         FD_ERR("TBaseSet<T,Cmp>::Iterator(" << this << "):operator--: invalid iterator: no baseset");
         abort();
       }
#endif
       std::set<T,Cmp>::const_iterator::operator-- ();
       return *this;
     };


     /** Order by reference for containers of Iterators < */
     /*
     bool operator < (const Iterator& rOther) const {
       return this < &rOther;
     };
     */

     /** Maintaine container reference */
     const TBaseSet<T,Cmp>* pBaseSet;

     /** Indocate that this iterator is attached to some baseset */
     bool mAttached;
   };

#ifdef DONT_TRACK_REFERENCES

  /** 
   * Iterator class for high-level api to TBaseSet.
   * This version is a dummy and does not provide any additional features.
   */
   class Iterator : public std::set<T,Cmp>::const_iterator {
     public:

     /** Default contructor */
     Iterator(void) : 
       std::set<T,Cmp>::const_iterator()  
     {}; 

     /** Copy constructor  */
     Iterator(const Iterator& fit) : 
       std::set<T,Cmp>::const_iterator(fit)
     {};

     /** Copy constructor  */
     Iterator(const typename std::set<T,Cmp>::const_iterator& sit) : 
       std::set<T,Cmp>::const_iterator(sit)
     {};

     /** Copy constructor, compatibility  */
     Iterator(
       const TBaseSet<T,Cmp>* pBaseSet,
       const typename std::set<T,Cmp>::const_iterator& sit) : 
       std::set<T,Cmp>::const_iterator(sit) 
     {};

     /** Assign STL iterator only, compatibility */
     void  StlIterator(const typename std::set<T,Cmp>::const_iterator& sit) {
       std::set<T,Cmp>::const_iterator::operator= (sit);
     }; 

     /** Get STL iterator only, compatibility */
     const typename std::set<T,Cmp>::const_iterator& StlIterator(void) const {
       return *this;
     };

     /** Invalidate, compatibility */
     void  Invalidate(void) {};

   };

#endif


protected:


  /** 
   * Token output, debugging see Type::DWrite for public wrappers.
   * Reimplement this function in derived classes for specific
   * specific template parameters.
   * @param rTw
   *   Reference to TokenWriter
   * @param rLabel
   *   Label of section to write, defaults to name of set
   * @param pContext
   *   Write context to provide contextual information
   */
  virtual void DoDWrite(TokenWriter& rTw,const std::string& rLabel="", const Type* pContext=0) const;

  /** 
   * Token output, see Type::DWrite for public wrappers.
   * Reimplement this function in derived classes for specific
   * specific template parameters. By convention, the default label ""
   * should be translated to a) the name of the set or b) some meaningful default, 
   * eg "IndexSet" for a set of indices". The pContext pointer can de type-checked
   * and interpreted, ie as a symboltable to provide symbolic names. It is also
   * passed on to attributes.
   *
   * @param rTw
   *   Reference to TokenWriter
   * @param rLabel
   *   Label of section to write, defaults to name of set
   * @param pContext
   *   Write context to provide contextual information
   */
  virtual void DoWrite(TokenWriter& rTw, const std::string& rLabel="", const Type* pContext=0) const;

  /**
   * Token input, see Type::Read for public wrappers.
   * Reimplement this function in derived classes for specific
   * specific template parameters.
   * By convention, the default label "" should be translated to some meaningful default, 
   * eg "IndexSet" for a set of indices". The pContext pointer can de type-checked
   * and interpreted, ie as a symboltable to provide symbolic names. It is also
   * passed on to attributes.
   *
   * @param rTr
   *   Reference to TokenReader
   * @param rLabel
   *   Label of section to read, defaults to name of set
   * @param pContext
   *   Read context to provide contextual information
   */
  virtual void DoRead(TokenReader& rTr, const std::string& rLabel = "", const Type* pContext=0);

  /** Assignment function (fake copy, no storage allocated) */
  const TBaseSet<T,Cmp>& Assign(const TBaseSet<T,Cmp>& rOtherSet);

  /** STL set, if the object owns content (else NULL) */
  std::set<T,Cmp>* mpSet;

  /** static empty STL set for default constructor */
  static std::set<T,Cmp> mEmptySet;

  /** Pointer on STL set to operate on */
  std::set<T,Cmp>* pSet;

  /** STL iterator, non-const version */
  typedef typename std::set<T,Cmp>::iterator iterator;

  /** STL iterator, const version */
  typedef typename std::set<T,Cmp>::const_iterator const_iterator;

  /** Convert STL iterator */
  typename TBaseSet<T,Cmp>::Iterator ThisIterator(const typename std::set<T,Cmp>::const_iterator& sit) const;


private:


  /** Name of TBaseSet */
  std::string mMyName;

  /** Other BaseSets, that refer to the STL set provided by this TBaseSet */
  std::set< TBaseSet<T,Cmp>* > mReferences;

  /** Pointer on BaseSet that owns the STL set this BaseSet referrs to */
  TBaseSet<T,Cmp>* pBaseSet;

  /** Indicate "has no references, provides STL set" */
  bool mDetached;

  /** Indicate "dont re-allocate the STL set again" */
  bool mLocked;

  /** Ensure that we do not serve contents to anyone else */
  void RelinkReferences(void);

  /** Record that we serve contents to some other TBaseSet */
  void AttachReference(TBaseSet* pRef) const;

  /** Record that we stop serving contents to some TBaseSet */
  void DetachReference(TBaseSet* pRef) const;

  /** Iterators that refer to this TBaseSet */
  std::set< Iterator* > mIterators;

  /** Record that an iterator refers to this TBaseSet */
  void AttachIterator(Iterator* pFit) const;

  /** Record that an iterator stops to refer to this TBaseSet */
  void DetachIterator(Iterator* pFit) const;

};


/** @} doxygen group */



/*
******************************************************************************************
******************************************************************************************
******************************************************************************************

Implementation of TBaseSet

******************************************************************************************
******************************************************************************************
******************************************************************************************
*/


// template staticxs: empty set
template<class T, class Cmp>
std::set<T,Cmp> TBaseSet<T,Cmp>::mEmptySet=std::set<T,Cmp>();


// TBaseSet()
template<class T, class Cmp>
TBaseSet<T,Cmp>::TBaseSet(void) :
  mpSet(NULL),
  pSet(&mEmptySet),  
  pBaseSet(this),
  mDetached(false), 
  mLocked(false) 
{
  FD_DC("TBaseSet(" << this << ")::TBaseSet()");
  // maintain deferred copy 
  /*
  mpSet= new std::set<T,Cmp>();
  pSet=mpSet;
  mDetached=true;
  */
  // other members
  mMyName="BaseSet";
}



// TBaseSet(rOtherSet)
template<class T, class Cmp>
TBaseSet<T,Cmp>::TBaseSet(const TBaseSet& rOtherSet) : 
  Type(rOtherSet),
  mpSet(NULL),
  pSet(NULL),  
  pBaseSet(this),
  mDetached(false), 
  mLocked(false) 
{
  FD_DC("TBaseSet(" << this << ")::TBaseSet(rOtherSet " << &rOtherSet << "): fake copy construct");
  // maintain deferred copy 
  pBaseSet=rOtherSet.pBaseSet;
  pBaseSet->AttachReference(this);
  pSet=rOtherSet.pSet;
  // other members
  mMyName=rOtherSet.mMyName;
#ifdef FAUDES_DEBUG_CONTAINER
  DValid();
#endif
}

// destructor
template<class T, class Cmp>
TBaseSet<T,Cmp>::~TBaseSet(void) {
  FD_DC("TBaseSet(" << this << ")::~TBaseSet()");
  // maintain deferred copy 
  RelinkReferences();
  pBaseSet->DetachReference(this);
  if(mpSet) delete mpSet;
  // unlink iterators (mandatory, since referenced object is destructed)
  typename std::set< Iterator* >::const_iterator iit;
  for(iit=mIterators.begin(); iit!=mIterators.end(); ++iit) {
    (**iit).Invalidate();
  }
}

// assignement
template<class T, class Cmp> 
const TBaseSet<T,Cmp>& TBaseSet<T,Cmp>::Assign(const  TBaseSet<T,Cmp>& rOtherSet) {
  FD_DC("TBaseSet(" << this << ")::Assign(rOtherSet " << &rOtherSet << "): fake assignment");
  // bail out ojn selfref
  if(this==&rOtherSet) return *this;
  // maintain deferred copy 
  RelinkReferences();
  pBaseSet->DetachReference(this);
  mReferences.clear();
  pBaseSet=rOtherSet.pBaseSet;
  pBaseSet->AttachReference(this);
  pSet=rOtherSet.pSet;
  if(mpSet) {
     delete mpSet;
     mpSet=NULL;
  }
  mDetached=false; 
  mLocked=false;
  // fix iterators (invalidate)
  typename std::set< Iterator* >::iterator iit;
  for(iit=mIterators.begin(); iit!=mIterators.end(); ++iit) {
    (**iit).Invalidate();
  }
  mIterators.clear();
  // other members 
  mMyName=rOtherSet.mMyName;
#ifdef FAUDES_DEBUG_CONTAINER
  DValid();
#endif
  return *this;
}


// Detach()
template<class T, class Cmp> 
void TBaseSet<T,Cmp>::Detach(void) const {
  FD_DC("TBaseSet(" << this << ")::Detach(void)");
#ifdef FAUDES_DEBUG_CONTAINER
  DValid();
#endif

  // nothing todo
  if(mDetached) return;

  // provide fake const
  TBaseSet<T,Cmp>* fake_const = const_cast< TBaseSet<T,Cmp>* >(this);

  // might have overseen reference detach
  if(pBaseSet->mReferences.empty() && pBaseSet->mpSet != NULL) {
    fake_const->mDetached=true;
    return;
  }

  // stragie A: references get the copy; thus, the baseset data does
  // not get reallocated and we dont need to track iterators. on the
  // downside, fixing the references iterators may be more effort.
  if(mLocked==true) {
   
    FD_DC("TBaseSet(" << this << ")::Detach(void): allocate and copy, strategie A");
    // do allocation and copy of stl set
    std::set<T,Cmp>* copy = new std::set<T,Cmp>();
    *copy = *pSet;     
    // first of my reference gets the new data
    TBaseSet<T,Cmp>* storage = *mReferences.begin();
    storage->pBaseSet=storage;
    storage->mpSet=copy; 
    storage->pSet=copy;
    storage->mReferences=mReferences;
    storage->DetachReference(storage);
    // check whether new storage is detached
    if(storage->mReferences.empty()) 
       storage->mDetached=true;
    // set other users to new storage
    typename std::set< TBaseSet<T,Cmp>* >::const_iterator rit;
    for(rit=mReferences.begin(), ++rit ;rit!=mReferences.end(); ++rit) {
      (*rit)->pBaseSet=storage;
      (*rit)->pSet=copy;
    }
    // fix other users iterators (incl new storage)
    for(rit=mReferences.begin(); rit!=mReferences.end(); ++rit) {
      typename std::set< Iterator* >::iterator iit;
      for(iit=(*rit)->mIterators.begin(); iit!=(*rit)->mIterators.end(); ++iit) {
        if((**iit).StlIterator()==pSet->end()) 
          **iit=Iterator(this, copy->end());
        else
          **iit=Iterator(this, copy->find(***iit));
      }
    }
    // make myself own the old data
    fake_const->mpSet=pSet;
    fake_const->mReferences.clear();
    fake_const->mDetached=true;    
    // stop tracking iterators
    typename std::set< Iterator* >::const_iterator iit;
    for(iit=mIterators.begin(); iit!=mIterators.end(); ++iit) {
      (**iit).Detach();
    }
    fake_const->mIterators.clear();


  // stragie B: this baseset gets the copy; thus, the references iterators
  // remain valid and dont need to be fixed; on the downside, we need to
  // continue to track iterators.
  } else {


    FD_DC("TBaseSet(" << this << ")::Detach(void): allocate and copy, strategie B");
    // make someone else handle original data
    fake_const->RelinkReferences();
    // do allocation and copy
    fake_const->mpSet = new std::set<T,Cmp>();
    *fake_const->mpSet= *pSet;     
    // fix iterators
    typename std::set< Iterator* >::iterator iit;
    for(iit=mIterators.begin(); iit!=mIterators.end(); ++iit) {
      if((**iit).StlIterator()==pSet->end()) 
        **iit=Iterator(this, mpSet->end());
      else
        **iit=Iterator(this, mpSet->find(***iit));
    }
    // unlink from old source, link to new source
    pBaseSet->DetachReference(fake_const);
    if(pBaseSet->pBaseSet->mReferences.empty() && pBaseSet->mpSet != NULL)
      pBaseSet->mDetached=true;
    fake_const->pBaseSet=fake_const;
    fake_const->pSet=mpSet;
    fake_const->mDetached=true;

  }

#ifdef FAUDES_DEBUG_CONTAINER
  DValid();
#endif
  FD_DC("TBaseSet(" << this << ")::Detach(void): done");
}

// Lock()
template<class T, class Cmp> 
void TBaseSet<T,Cmp>::Lock(void) const {
  FD_DC("TBaseSet(" << this << ")::Lock(void)");
#ifdef FAUDES_DEBUG_CONTAINER
  DValid();
#endif

  // trigger actual copy
  Detach();

  // provide fake const
  TBaseSet<T,Cmp>* fake_const = const_cast< TBaseSet<T,Cmp>* >(this);

  // stop tracking iterators
  typename std::set< Iterator* >::const_iterator iit;
  for(iit=mIterators.begin(); iit!=mIterators.end(); ++iit) {
    (**iit).Detach();
  }
  fake_const->mIterators.clear();

  // stop detach from reallocating
  fake_const->mLocked=true;
}


// if a am a source to others, make someone else source  
template<class T, class Cmp>
inline void TBaseSet<T,Cmp>::RelinkReferences(void) {
  FD_DC("TBaseSet::RelinkReferences(" << this << ")")
#ifdef FAUDES_DEBUG_CONTAINER
  DValid();
#endif
  // no references aka this is not a source
  if(mReferences.empty()) return;
  // make first ref the new storage
  TBaseSet<T,Cmp>* storage = *mReferences.begin();
#ifdef FAUDES_DEBUG_CODE
  if(storage->pBaseSet!=this) {
    FD_ERR("TBaseSet<T,Cmp>::RelinkRefernces: old reference must have this as storage provider");
    abort();
  }
#endif
  storage->pBaseSet=storage;
  storage->mpSet=mpSet; 
  storage->pSet=pSet;
  storage->mReferences=mReferences;
  storage->DetachReference(storage);
  // set other users to new storage
  typename std::set< TBaseSet<T,Cmp>* >::const_iterator rit;
  for(rit=mReferences.begin(), ++rit;rit!=mReferences.end(); ++rit) {
    (*rit)->pBaseSet=storage;
  }
  // make myself a reference to the new source
  pBaseSet=storage;
  pSet=storage->pSet;
  mpSet=NULL;
  storage->AttachReference(this);
  mReferences.clear();
#ifdef FAUDES_DEBUG_CONTAINER
  DValid();
#endif
  FD_DC("TBaseSet::RelinkReferences(" << this << "): done")
}


// record fake copy
template<class T, class Cmp>
inline void TBaseSet<T,Cmp>::AttachReference(TBaseSet* pRef) const {
   const_cast< TBaseSet<T,Cmp>* >(this)->mReferences.insert(pRef);
   const_cast< TBaseSet<T,Cmp>* >(this)->mDetached=false;
}

// discard fake copy
template<class T, class Cmp>
inline void TBaseSet<T,Cmp>::DetachReference(TBaseSet* pRef) const {
   const_cast< TBaseSet<T,Cmp>* >(this)->mReferences.erase(pRef);
}


// record an iterator
template<class T, class Cmp>
inline void TBaseSet<T,Cmp>::AttachIterator(Iterator* pFit) const {
  if(mLocked) return;
  FD_DC("TBaseSet::AttachIterator(" << this << "):" << pFit)
  const_cast< TBaseSet<T,Cmp>* >(this)->mIterators.insert(pFit);
}

// discard an iterator
template<class T, class Cmp>
inline void TBaseSet<T,Cmp>::DetachIterator(Iterator* pFit) const {
  if(mLocked) return;
  FD_DC("TBaseSet::DetachIterator(" << this << "):" << pFit)
  const_cast< TBaseSet<T,Cmp>* >(this)->mIterators.erase(pFit);
}



// test some validity
template<class T, class Cmp>
void TBaseSet<T,Cmp>::DValid(const std::string& rMessage) const {
  typename std::set< Iterator* >::const_iterator iit;
#ifdef FAUDES_DEBUG_CONTAINER
  std::cout << "TBaseSet<T,Cmp>(" << this << "," << rMessage << ")::DValid:  source " 
     << pBaseSet << " #" << (pBaseSet->pSet==&mEmptySet ? "e" : "f") << " -- list: ";
  typename std::set< TBaseSet<T,Cmp>* >::const_iterator rit;
  for(rit=pBaseSet->mReferences.begin(); rit!=pBaseSet->mReferences.end(); ++rit)
    std::cout << *rit << " ";
  std::cout << "-- iterators: ";
  for(iit=mIterators.begin(); iit!=mIterators.end(); ++iit)
    std::cout << *iit << " ";
  std::cout << "." << std::endl;
#endif
  // iterators, that dont refer to me as basset
  for(iit=mIterators.begin(); iit!=mIterators.end(); ++iit) {
    if((*iit)->pBaseSet!=this) {
      FD_ERR("TBaseSet<T,Cmp>("<< this << "," << rMessage <<"): invalid iterator (baseset): "<< *iit);
      abort();
    }
  }
  // iterators, that are not marked as attechd
  for(iit=mIterators.begin(); iit!=mIterators.end(); ++iit) {
    if(!(*iit)->mAttached) {
      FD_ERR("TBaseSet<T,Cmp>("<< this << "," << rMessage <<"): invalid iterator (attached): "<< *iit);
      abort();
    }
  }
  // iterators, that are invalid stl iterators
  for(iit=mIterators.begin(); iit!=mIterators.end(); ++iit) {
    typename std::set<T,Cmp>::const_iterator vit;
    for(vit=pSet->begin(); vit!= pSet->end(); ++vit) {
      if(vit==(**iit).StlIterator()) break;
    }
    if(vit!=(**iit).StlIterator()) { // end-iterator is fine, too
      FD_ERR("TBaseSet<T,Cmp>("<< this << "," << rMessage <<"): invalid iterator (stl "<< *iit);
      (**iit).StlIterator(pSet->end()); // fix invalid iterator to refer to end()
    }
  }
  // must have some base 
  if(pBaseSet==NULL) {
    FD_ERR("TBaseSet<T,Cmp>(" << this << "," << rMessage << "): no base found");
    abort();
  }
  // base must provide data either way
  if(pBaseSet->pBaseSet != pBaseSet) {
    FD_ERR("TBaseSet<T,Cmp>(" << this << "," << rMessage << "): no source found");
    abort();
  }
  // refers to other base and own data
  if((mpSet!=NULL) && (pBaseSet != this)) {
    FD_ERR("TBaseSet<T,Cmp>(" << this << "," << rMessage << "): double data");
    abort();
  }
  // refers to other base and has references
  if(pBaseSet!=this && !mReferences.empty()) {
    FD_ERR("TBaseSet<T,Cmp>(" << this << "," << rMessage << "): cannot have source and refs");
    abort();
  }
  // is base but has no own data
  if((pBaseSet == this) && (mpSet==NULL) && (pSet!=&mEmptySet)) {
    FD_ERR("TBaseSet<T,Cmp>(" << this << "," << rMessage << "): no data");
    abort();
  }
  // is base but own data pointer mismatch
  if((pBaseSet == this) && (pSet != mpSet) && (pSet!=&mEmptySet)) {
    FD_ERR("TBaseSet<T,Cmp>(" << this << "," << rMessage << "): data pointer mismatch A");
    abort();
  }
  // refers to base with data pointer mismatch
  if((pBaseSet != this) && (pSet != pBaseSet->pSet)) {
    FD_ERR("TBaseSet<T,Cmp>(" << this << "," << rMessage << "): data pointer mismatch B");
    abort();
  }
  // error in detached flag
  if(mDetached && !pBaseSet->mReferences.empty()) {
    FD_ERR("TBaseSet<T,Cmp>(" << this << "," << rMessage << "): invalid detached flag");
    abort();
  }
  // error in detached flag
  if(mDetached && (pSet==&mEmptySet)) {
    FD_ERR("TBaseSet<T,Cmp>(" << this << "," << rMessage << "): detached empty set");
    abort();
  }
  // error in lock flag
  if(pBaseSet!=this && mLocked) {
    FD_ERR("TBaseSet<T,Cmp>(" << this << "," << rMessage << "): locked reference");
    abort();
  }
  // invalid emptyset
  if(!mEmptySet.empty()) {
    FD_ERR("TBaseSet<T,Cmp>(" << this << "," << rMessage << "): invalid empty set");
    abort();
  }
}



// Name
template<class T, class Cmp>
std::string TBaseSet<T,Cmp>::Name(void) const {
  return mMyName;
}
		
// Name
template<class T, class Cmp>
void TBaseSet<T,Cmp>::Name(const std::string& rName) {
  mMyName = rName;
}
  
// Size()
template<class T, class Cmp>
Idx TBaseSet<T,Cmp>::Size(void) const {
  return (Idx) pSet->size();
}

// Empty()
template<class T, class Cmp>
bool TBaseSet<T,Cmp>::Empty(void) const {
  return pSet->empty();
}


// DoWrite(tw,rLabel,cpntext)
template<class T, class Cmp>
void TBaseSet<T,Cmp>::DoWrite(TokenWriter& rTw,const std::string& rLabel, const Type* pContext) const {
  (void) pContext;
  std::string label=rLabel;
  if(label=="") label=Name(); 
  if(label=="") label="BaseSet"; 
  rTw.WriteBegin(label);
  rTw.WriteEnd(label);
}


// DoDWrite(tw, label, context)
template<class T, class Cmp>
void TBaseSet<T,Cmp>::DoDWrite(TokenWriter& rTw, const std::string& rLabel, const Type* pContext) const {
  (void) pContext;
  (void) rLabel;
  rTw.WriteComment("");
  rTw.WriteComment(" Size/Shares/Iterators: " + ToStringInteger(pSet->size()) 
              + "/" + ToStringInteger(pBaseSet->mReferences.size())
	      + "/" + ToStringInteger(mIterators.size()));
  rTw.WriteComment("");
#ifdef FAUDES_DEBUG_CONTAINER
  DValid();
#endif
}

// DoRead(rTr, rLabel, pContext)
template<class T, class Cmp>
void TBaseSet<T,Cmp>::DoRead(TokenReader& rTr, const std::string& rLabel, const Type* pContext) {
  (void) pContext;
  Name(rLabel);
  rTr.SeekBegin(rLabel); 
  rTr.SeekEnd(rLabel); 
}

// ThisIterator
template<class T, class Cmp>
typename TBaseSet<T,Cmp>::Iterator TBaseSet<T,Cmp>::ThisIterator(const typename std::set<T,Cmp>::const_iterator& sit) const {
  return Iterator(this,sit);
}

// Begin() const
template<class T, class Cmp>
inline typename TBaseSet<T,Cmp>::Iterator TBaseSet<T,Cmp>::Begin(void) const {
  return ThisIterator(pSet->begin());
}
		
// End() const
template<class T, class Cmp>
inline typename TBaseSet<T,Cmp>::Iterator TBaseSet<T,Cmp>::End(void) const {
  return ThisIterator(pSet->end());
}


//Clear
template<class T, class Cmp>
void TBaseSet<T,Cmp>::Clear(void) {
  FD_DC("TBaseSet(" << this << ")::Clear()");
#ifdef FAUDES_DEBUG_CONTAINER
  DValid();
#endif
  // special case: empty anyway
  if(pSet==&mEmptySet) return;
  // special case: locked requires a copy
  if(mLocked) Detach();
  // make someone else handle the data
  if(!mReferences.empty()) 
    RelinkReferences();
  // create empty set
  if(pBaseSet!=this) {
    FD_DC("TBaseSet(" << this << ")::Clear(): allocate");
    mpSet = new std::set<T,Cmp>();
    pSet=mpSet;
    pBaseSet->DetachReference(this);
    pBaseSet=this;
    FD_DC("TBaseSet(" << this << ")::Clear(): allocate: done");
  }
  // fix iterators (invalidate)
  typename std::set< Iterator* >::iterator iit;
  for(iit=mIterators.begin(); iit!=mIterators.end(); ++iit) {
    (**iit).Invalidate();
  }
  mIterators.clear();
  mDetached=true;
  // clear the stl set
  pSet->clear();
#ifdef FAUDES_DEBUG_CONTAINER
  DValid();
#endif
}


//Valid(elem)
template<class T, class Cmp>
inline bool  TBaseSet<T,Cmp>::Valid(const T& rElem) const {
  (void) rElem;
  return true;
}

//Insert(elem)
template<class T, class Cmp>
bool TBaseSet<T,Cmp>::Insert(const T& rElem) {
#ifdef FAUDES_CHECKED
  if(!Valid(rElem)) {
    std::stringstream errstr;
    errstr << "cannot insert invalid element" << std::endl;
    throw Exception("TBaseSet<T,Cmp>::Insert", errstr.str(), 61);
  }
#endif
  if(!mDetached) Detach();
  return pSet->insert(rElem).second;
}

// InsertSet(set)
template<class T, class Cmp>
void TBaseSet<T,Cmp>::InsertSet(const TBaseSet& rOtherSet) {
  FD_DC("TBaseSet(" << this << ")::InsertSet(" << &rOtherSet << ")");
  if(!mDetached) Detach();
  // TODO: test and optimize
  iterator it1 = pSet->begin();
  iterator it2 = rOtherSet.pSet->begin();
  while ((it1 != pSet->end()) && (it2 != rOtherSet.pSet->end())) {
    if (*it1 < *it2) {
      ++it1;
    }
    else if (*it1 == *it2) {
      ++it1;
      ++it2;
    }
    else { // (*it1 > *it2)
      pSet->insert(*it2);
      ++it2;
    }
  }
  while (it2 != rOtherSet.pSet->end()) {
    pSet->insert(*it2);
    ++it2;
  }
}
		
//Erase(rElem)
template<class T, class Cmp>
bool TBaseSet<T,Cmp>::Erase(const T& rElem) {
  if(!mDetached) Detach();
  return (pSet->erase(rElem)!=0);
}


//Erase(pos)
template<class T, class Cmp>
typename TBaseSet<T,Cmp>::Iterator TBaseSet<T,Cmp>::Erase(const Iterator& pos) { 
#ifdef FAUDES_CHECKED
  if (pos == End()) {
    std::stringstream errstr;
    errstr << "iterator out of range " << std::endl;
    throw Exception("TBaseSet<T,Cmp>::Erase", errstr.str(), 62);
  }
#endif
  Detach();
  iterator del= pos.StlIterator(); 
  iterator res= del;
  res++; 
  pSet->erase(del);
  return ThisIterator(res); 
}


//EraseSet(set)
template<class T, class Cmp>
void TBaseSet<T,Cmp>::EraseSet(const TBaseSet& rOtherSet) {
  FD_DC("TBaseSet(" << this << ")::EraseSet(" << &rOtherSet << ")");
  if(!mDetached) Detach();
  // TODO: test and optimize 
  iterator tmpit;
  iterator it = pSet->begin();
  iterator oit = rOtherSet.pSet->begin();
  while ((it != pSet->end()) && (oit != rOtherSet.pSet->end())) {
    if (*it < *oit) {
      it=pSet->lower_bound(*oit); // alt: ++it;
    }
    else if (*it == *oit) { 
      tmpit=it;
      ++it;
      ++oit;
      pSet->erase(tmpit);
    }
    else { // (*it > *oit)
      oit=rOtherSet.pSet->lower_bound(*it); // ++it2;
    }
  }
}


//Find(elem)
template<class T, class Cmp>
typename TBaseSet<T,Cmp>::Iterator TBaseSet<T,Cmp>::Find(const T& rElem) const {
  return ThisIterator(pSet->find(rElem));
}

//Exists(elem)
template<class T, class Cmp>
bool  TBaseSet<T,Cmp>::Exists(const T& rElem) const {
  return pSet->find(rElem) != pSet->end();
}


// SetUnion(set)
template<class T, class Cmp>
void TBaseSet<T,Cmp>::SetUnion(const TBaseSet& rOtherSet) {
  if(!mDetached) Detach();
  iterator it1 = pSet->begin();
  iterator it2 = rOtherSet.pSet->begin();
  while ((it1 != pSet->end()) && (it2 != rOtherSet.pSet->end())) {
    if (*it1 < *it2) {
      ++it1;
      continue;
    }
    if (*it1 == *it2) {
      ++it1;
      ++it2;
      continue;
    }
    // (*it1 > *it2)
    pSet->insert(it1,*it2);
    ++it2;
  }
  while (it2 != rOtherSet.pSet->end()) {
    pSet->insert(pSet->end(),*it2);
    ++it2;
  }
}

// SetIntersection(set)
template<class T, class Cmp>
void TBaseSet<T,Cmp>::SetIntersection(const TBaseSet& rOtherSet) {
  if(!mDetached) Detach();
  iterator tmpit;
  iterator it1 = pSet->begin();
  iterator it2 = rOtherSet.pSet->begin();
  while ((it1 != pSet->end()) && (it2 != rOtherSet.pSet->end())) {
    if (*it1 < *it2) {
      tmpit=it1;
      ++it1;
      pSet->erase(tmpit);
      continue;
    }
    if (*it1 == *it2) {
      ++it1;
      ++it2;
      continue;
    }
    // if (*it1 > *it2)
    ++it2;
  }
  while(it1 != pSet->end()) {
    tmpit=it1;
    ++it1;
    pSet->erase(tmpit);
  }
}


// operator +
template<class T, class Cmp>
TBaseSet<T,Cmp> TBaseSet<T,Cmp>::operator + (const TBaseSet& rOtherSet) const {
  FD_DC("TBaseSet(" << this << ")::operator + (" << &rOtherSet << ")");
  TBaseSet res;
  res.Detach();
  std::insert_iterator< std::set<Idx> > insit(*res.pSet, res.pSet->begin());
  std::set_union(pSet->begin(), pSet->end(), rOtherSet.pSet->begin(), rOtherSet.pSet->end(), insit);
  res.Name(CollapsString(Name() + "+" + rOtherSet.Name()));
  return res;
}

// operator -
template<class T, class Cmp>
TBaseSet<T,Cmp> TBaseSet<T,Cmp>::operator - (const TBaseSet& rOtherSet) const {
  FD_DC("TBaseSet(" << this << ")::operator - (" << &rOtherSet << ")");
  TBaseSet res;
  res.Detach();
  std::insert_iterator< std::set<Idx> > insit(*res.pSet, res.pSet->begin());
  std::set_difference(pSet->begin(), pSet->end(), rOtherSet.pSet->begin(), rOtherSet.pSet->end(), insit);
  res.Name(CollapsString(Name() + "-" + rOtherSet.Name()));
  return res;
}

     
// operator *
template<class T, class Cmp>
TBaseSet<T,Cmp> TBaseSet<T,Cmp>::operator * (const TBaseSet& rOtherSet) const {
  FD_DC("TBaseSet(" << this << ")::operator * (" << &rOtherSet << ")");
  TBaseSet res;
  res.Detach();
  std::insert_iterator< std::set<Idx> > insit(*res.pSet, res.pSet->begin());
  std::set_intersection(pSet->begin(), pSet->end(), rOtherSet.pSet->begin(), rOtherSet.pSet->end(), insit);
  res.Name(CollapsString(Name() + "*" + rOtherSet.Name()));
  return res;
}


// operator ==
template<class T, class Cmp>
bool TBaseSet<T,Cmp>::operator == (const TBaseSet& rOtherSet) const {
  return  ( *pSet == *rOtherSet.pSet );
}

// operator != 
template<class T, class Cmp>
bool TBaseSet<T,Cmp>::operator != (const TBaseSet& rOtherSet) const {
  return  ( *pSet != *rOtherSet.pSet );
}

// operator <=
template<class T, class Cmp>
bool TBaseSet<T,Cmp>::operator <= (const TBaseSet& rOtherSet) const {
  return ( std::includes(rOtherSet.pSet->begin(), rOtherSet.pSet->end(), pSet->begin(), pSet->end()) ) ;
}

// operator >=
template<class T, class Cmp>
bool TBaseSet<T,Cmp>::operator >= (const TBaseSet& rOtherSet) const {
  return ( std::includes(pSet->begin(), pSet->end(), rOtherSet.pSet->begin(), rOtherSet.pSet->end()) );
}

// operator <
template<class T, class Cmp>
bool TBaseSet<T,Cmp>::operator < (const TBaseSet& rOtherSet) const {
  return *pSet < *rOtherSet.pSet;
}


/** @} doxygen group */

} // namespace faudes

#endif 
