/** @file definitions.h 

Compiletime options, doxygen pages/sections. This file configures the runtime bahaviour of
libFAUDES, in particular informative output on stderr and stdout; see
also the Makefile.

*/


/* FAU Discrete Event Systems Library (libfaudes)

   Copyright (C) 2006  Bernd Opitz
   Copyright (C) 2008  Thomas Moor
   Exclusive copyright is granted to Klaus Schmidt

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */



#ifndef FAUDES_DEFINITIONS_H
#define FAUDES_DEFINITIONS_H

#include "configuration.h"

#include <sstream>
#include <iostream>

namespace faudes {

/** Type definition for index type */
typedef unsigned int Idx;

/** Length of strings for text fields in token output*/
#define FD_NAMELEN 13

/** Max length of automatic container names (set to -1 for unlimited) */
#define FD_MAXCONTAINERNAME 70

/** Max size of transrel for state name output */
#define FD_SMALLTRANSREL 100

/** Min number of states to use consecutive section in file output */
#define FD_CONSECUTIVE 6

/** Fallback version string (should define version in Makefile) */
#ifndef FAUDES_VERSION
#define FAUDES_VERSION "undefined version"
#endif


/** Debug: output macro for redirection to file */
#define FAUDES_WRITE_STDERR(message) { *faudes::DebugStream() << message << std::endl ; faudes::DebugStream()->flush() ;  };

/** Debug: always report warnings */
#define FD_WARN(message) FAUDES_WRITE_STDERR(message);

/** Debug: report more errors with file/line info */
#ifdef FAUDES_DEBUG_CODE
#define FD_ERR(message) FAUDES_WRITE_STDERR("FAUDES_CODE:"  << message << " in " \
		<< __FILE__ << ":" << __LINE__ );
#else
#define FD_ERR(message)
#endif

/** Debug: optional write progress report to stdout */
#ifdef FAUDES_WRITE_PROGRESS 
#define FD_WP(message) std::cout << message << std::endl;
#else
#define FD_WP(message)
#endif

/** Debug: optional report on user functions */
#ifdef FAUDES_DEBUG_FUNCTION
#define FD_DF(message) FAUDES_WRITE_STDERR("FAUDES_FUNCTION: " << message);
#else
#define FD_DF(message)
#endif


/** Debug: optional report on generator operations */
#ifdef FAUDES_DEBUG_GENERATOR
#define FD_DG(message) FAUDES_WRITE_STDERR("FAUDES_GENERATOR: " << message);
#else
#define FD_DG(message)
#endif

/** Debug: optional report on container operations */
#ifdef FAUDES_DEBUG_CONTAINER
#define FD_DC(message) FAUDES_WRITE_STDERR("FAUDES_CONTAINER: " << message);
#else
#define FD_DC(message)
#endif

/** Debug: optional low-level report on iterations and  token IO */
#ifdef FAUDES_DEBUG_VERBOSE
#define FD_DV(message) FAUDES_WRITE_STDERR("FAUDES_VERBOSE: " <<  message);
#else
#define FD_DV(message)
#endif

/** Doxygen: exclude this from doxygen */
#ifndef FAUDES_DOXYGEN
#define FAUDES_NODOC(a) a
#else
#define FAUDES_NODOC(a)
#endif

/** Doxygen: hide interface inheritynce from doxygen */
#ifndef FAUDES_DOXYGEN
#define FAUDES_TYPE public Type
#else
#define FAUDES_TYPE
#endif


} // namespace faudes

#endif 


/**
 
@mainpage libFAUDES - Friedrich Alexander University Discrete Event Systems Library

@section Overview

libFAUDES is a C++ library supporting research in the field of discrete event 
systems (DES) modeled by finite automata. It implements data structures and
algorithms for the manipulation of finite automata, and supplies algorithms 
for regular languages and for the supervisory control theory according to the 
Ramadge/Wonham-Framework. 

The main purpose of the library is to provide a basis for the implementation 
and evaluation of algorithms for the analysis and synthesis of DESs.
The standard Ramadge/Wonham supervisory control algorithms included in 
the library constitutes an example for such an implementation.

@section SecComponents Design

libFAUDES is organized in four components, namely container classes for
sets of events, states and transitions; generator classes to model finite
state machines; functions that implement general pupose algorithms on generators;
and specialised algorithms and derived generator classes provided by plugins.
The tutorial is seen as a pragmatic documentation which is 
supplemented by the doxygen generated html pages. 

- @ref ContainerClasses
- @ref GeneratorClasses
- @ref GeneratorFunctions
- @ref AllPlugins
- @ref Tutorials


@section Compile


libFAUDES is coded in posix C++ and uses the standard template library (STL). It
should compile flawlessly on most unix systems. Successful compilation on 
windows xp has been reported, see README.windows for details. We use linux as 
a development platfrom, where libFAUDES relies on gcc version 3.4 or above 
and GNU make.

To compile, run <tt>make</tt> in the libFAUDES directory. This will build the libFAUDES 
shared object file. Check the Makefile for compile-time options,
including verbose output, consistency checks, and plugings.
A <tt>make tutorial</tt> compiles examples from the 
tutorial section which are intended for testing/learning libFAUDES classes and 
functions. 



@section License

The core library sources are provided for free under conditions of the GNU Lesser 
General Public License.  Thus, while libFAUDES itself is an open project and
you are invited to contribute, you may also use libFAUDES for the
deveopment of non-open/commercial software. Note that different and perhaps more
restrictive license terms may apply to the plugins that reside in the plugin directory.

<br>
<br>
<br>

  See 
  <a href="http://www.rt.e-technik.uni-erlangen.de/FGdes/faudes" target="_top">
  http://www.rt.e-technik.uni-erlangen.de/FGdes/faudes</a>
  for further information.

  Copyright (C) 2006 Bernd Opitz <br>
  Copyright (C) 2008 Thomas Moor, Klaus Schmidt, Sebastian Perk 

*/


/** @defgroup GeneratorClasses  Generator Classes

This module collects all libFAUDES generator classes.

*/

/** @defgroup ContainerClasses State, Event and Transition Set

This module collects all basic container classes aka state- event- 
and transition-sets.

*/

/** @defgroup GeneratorFunctions  Operations on Generators

This module collects general purpose functions on Generators. 
It includes functions related to regular expressions,
projection, parallel composition and basic controller synthesis.

*/


/** @defgroup Tutorials  Example Code and Data

This module collects all tutorials. The tutorials are seen as a pragmatic
introduction to supplement the doxygen generated html documentaion of the
class definitions.


*/

/** @defgroup AllPlugins Plugins

Find below a list of all plugins that are installed and enabled
in this copy of libfaudes.

To install a plugin, copy its sources into the plug directory
of the libfaudes source tree. To enable a plugin, either edit the
main Makefile of add its name to the FAUDES_PLUGINS environment variable.
The example plugin further illustrates the plugin infrastructure.

*/

/** @namespace faudes

libFAUDES resides within the namespace faudes. Plugins may use the same namespace.

*/
