/** @file helper.h   Helper functions */

/* FAU Discrete Event Systems Library (libfaudes)

   Copyright (C) 2006  Bernd Opitz
   Exclusive copyright is granted to Klaus Schmidt

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */


#ifndef FAUDES_HELPER_H
#define FAUDES_HELPER_H

#include <cstdlib>
#include <cstring>
#include <string>
#include <iostream>
#include <sstream>
#include <fstream>
#include <limits>
#include <iomanip>


#include "definitions.h"
#include "exception.h"

namespace faudes {

/**
 * integer to string
 * 
 * @param number
 *   integer
 *
 * @return
 *   string
 */
std::string ToStringInteger(long int number);


/**
 * integer to string base 16
 * 
 * @param number
 *   integer
 *
 * @return
 *   string
 */
std::string ToStringInteger16(long int number);

/**
 * float to string 
 * 
 * @param number
 *   double
 *
 * @return
 *   string
 */
std::string ToStringFloat(double number);


/**
 * Fill string with spaces up to a given length if length of the string
 * is smaller than given length parameter.
 *
 * @param rString
 *   string
 * @param len
 *   Minimum number of charakters in string
 *
 * @return
 *   Expanded string
 */
std::string ExpandString(const std::string& rString, unsigned int len);

/**
 * Limit length of string, return head and tail of string
 *
 * @param rString
 *   string
 * @param len
 *   Maximum number of charakters in string (approx)
 *
 * @return
 *   Collapsed string
 */
 std::string CollapsString(const std::string& rString, unsigned int len= FD_MAXCONTAINERNAME);


/**
 * Convert a string to Idx
 *
 * @param rString
 *   string to convert
 *
 * @return 
 *   Idx
 *
 * @exception
 *   Idx overflow (id 600)
 */
Idx ToIdx(const std::string& rString);

/**
 * Return FAUDES_VERSION as std::string
 *
 * @return 
 *   std::string with FAUDES_VERSION
 */
std::string FDVersionString();

/**
 * Return FAUDES_PLUGINS as std::string
 *
 * @return 
 *   std::string with FAUDES_VERSION
 */
std::string FDPluginsString();

/**
 * Return contributors as std::string
 *
 * @return 
 *   std::string 
 */
std::string FDContributorsString();

/**
 * Convenience function: process dot file
 * to graphics output.  If no output format is given,
 * try to guess from filename extension. 
 *
 * @param rDotFile
 *   name of dot file
 * @param rOutFile
 *   name of graphics file 
 * @param rOutFormat
 *   graphics format eg "png", "jpg"
 * @param rDotExec
 *   path/name of executable
 *
 * @exception Exception
 *   - error in systemcall (id 3)
 *   - unkown format (id 3)
 *
 */
void ProcessDot(const std::string& rDotFile, const std::string& rOutFile, 
  const std::string& rOutFormat = "", const std::string& rDotExec = "dot");

/**
 * Create a temp file, length 0
 *
 * @return
 *   Name of temp file
 */

std::string CreateTempFile(void);


/**
 * Delete a file
 *
 * @param rFileName
 *   Name of file to delete
 */
bool RemoveFile(const std::string& rFileName);

/**
 * Extract directory from full filename.
 *
 * @param rFullName
 *   Full name of filr eg "/home/friend/data/generator.gen"
 * @return
 *   Directory eg "/home/friend/data"
 */
std::string ExtractPath(const std::string& rFullName);

/**
 * Construct full name from directory and filename.
 *
 * @param rPathName
 *   Directory eg "/home/friend/data"
 * @param rFileName
 *   File eg "generator.gen"
 * @return
 *   Path eg "/home/friend/data/generator.gen"
 */
std::string PrependPath(const std::string& rPathName, const std::string& rFileName);


/**
 * Set filename for debug log.
 * Debugging to file is meant for platforms that dont provide
 * a console for std::err (eg embedded systems); see also
 * FAUDES_WRITE_STDERR makro in definitions.
 *
 * @param rFileName
 *   File to log to 
 */
void DebugToFile(const std::string& rFileName="tmp_faudes_debug.log");

/**
 * File Stream to debug to
 */
extern std::ofstream* pDebugFileStream;

/**
 * Get debugging stream.
 *
 * @return
 *   Debugging Output Stream 
 */
 std::ostream* DebugStream(void);


} // namespace faudes

#endif

