/** @file interface.cpp Interface to types and operations */

/* FAU Discrete Event Systems Library (libfaudes)

   Copyright (C) 2008  Thomas Moor
   Exclusive copyright is granted to Klaus Schmidt

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNES FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */


#include "interface.h"


namespace faudes {

/*
********************************************************************
********************************************************************
********************************************************************

implemantation of faudes Type

********************************************************************
********************************************************************
********************************************************************
*/

// constructor
Type::Type(void)  {}

// copy constructor
Type::Type(const Type& rType)  {(void) rType;}

// destructor
Type::~Type(void) {}


// Clear()
void Type::Clear(void) {
  FD_DC("Type::Clear(): not re-implemented in " << typeid(*this).name());
}

// Write(context)
void Type::Write(const Type* pContext) const {
  TokenWriter tw(TokenWriter::Stdout);
  DoWrite(tw,"",pContext);
}

// Write(rFileName, label, context, openmode)
void Type::Write(const std::string& rFileName, const std::string& rLabel, 
  const Type* pContext, std::ios::openmode openmode) const {
  try {
    TokenWriter tw(rFileName, openmode);
    DoWrite(tw, rLabel, pContext);
  }
  catch (std::ios::failure&) {
    std::stringstream errstr;
    errstr << "Exception opening/writing file \"" << rFileName << "\"";
    throw Exception("Type::Write", errstr.str(), 2);
  }
}

// Write(rFileName, openmode)
void Type::Write(const std::string& rFileName, std::ios::openmode openmode) const {
  Write(rFileName,"",0,openmode);
}

// Write(tw,label,context)
void Type::Write(TokenWriter& rTw, const std::string& rLabel, const Type* pContext) const {
  DoWrite(rTw,rLabel,pContext);
}

// ToString(label, context)
std::string Type::ToString(const std::string& rLabel, const Type* pContext) const {
  TokenWriter tw(TokenWriter::String);
  DoWrite(tw,rLabel,pContext);
  return tw.Str();
}

// DWrite(context)
void Type::DWrite(const Type* pContext) const {
  TokenWriter tw(TokenWriter::Stdout);
  DoDWrite(tw,"",pContext);
}

// DWrite(rFileName, label, context, openmode)
void Type::DWrite(const std::string& rFileName, const std::string& rLabel, 
  const Type* pContext, std::ios::openmode openmode) const {
  try {
    TokenWriter tw(rFileName, openmode);
    DoDWrite(tw, rLabel, pContext);
  }
  catch (std::ios::failure&) {
    std::stringstream errstr;
    errstr << "Exception opening/writing file \"" << rFileName << "\"";
    throw Exception("Type::DWrite", errstr.str(), 2);
  }
}

// DWrite(tw,label,context)
void Type::DWrite(TokenWriter& rTw, const std::string& rLabel, const Type* pContext) const {
  DoDWrite(rTw,rLabel,pContext);
}

// Read(rFilename, rLabel,context)
void Type::Read(const std::string& rFilename, const std::string& rLabel, const Type* pContext) {
  Clear();
  TokenReader tr(rFilename);
  DoRead(tr,rLabel,pContext);
}

// Read(rTr, rLabel, context)
void Type::Read(TokenReader& rTr, const std::string& rLabel, const Type* pContext) {
  Clear();
  DoRead(rTr,rLabel,pContext);
}

//DoWrite(rTr,rLabel,pContext)
void Type::DoWrite(TokenWriter& rTw, const std::string& rLabel, const Type* pContext) const {
  (void) rTw; (void) rLabel; (void) pContext;
  FD_DC("Type::DoWrite(): not re-implemented in " << typeid(*this).name());
}

//DoDWrite(rTr,rLabel,pContext)
void Type::DoDWrite(TokenWriter& rTw, const std::string& rLabel, const Type* pContext) const {
  FD_DC("Type::DoDWrite(): not re-implemented in " << typeid(*this).name() << ", using DoDWrite instead");
  DoWrite(rTw,rLabel,pContext);
}

//DoRead(rTr,rLabel,pContext)
void Type::DoRead(TokenReader& rTr, const std::string& rLabel, const Type* pContext) {
  (void) rLabel; (void) pContext; (void) rTr;
  FD_DC("Type::DoRead(): not re-implemented in " << typeid(*this).name());
}


/*
********************************************************************
********************************************************************
********************************************************************

implemantation of faudes Operation

********************************************************************
********************************************************************
********************************************************************
*/

Operation::Operation(void) {}
Operation::~Operation(void) {}


} // namespace
