/** @file nameset.cpp @brief Classes NameSet */

/* FAU Discrete Event Systems Library (libfaudes)

   Copyright (C) 2006  Bernd Opitz
   Exclusive copyright is granted to Klaus Schmidt

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */


#include "nameset.h"

namespace faudes {

// empty constructor
NameSet::NameSet(void) {
  mpSymbolTable = SymbolTable::GlobalEventSymbolTablep();
  IndexSet::Name("NameSet");
  FD_DC("NameSet("<<this<<")::NameSet() with symtab "<< mpSymbolTable); 
}

// constructor from nameset
NameSet::NameSet(const NameSet& rOtherSet)  {
  FD_DC("NameSet(" << this << ")::NameSet(rOtherSet " << &rOtherSet << ")");
  Assign(rOtherSet);
  mpSymbolTable = rOtherSet.mpSymbolTable;
}


// read file constructor
NameSet::NameSet(const std::string& rFilename, const std::string& rLabel) {
  FD_DC("NameSet(" << this << ")::NameSet(" << rFilename << ")");
  mpSymbolTable = SymbolTable::GlobalEventSymbolTablep();
  Read(rFilename, rLabel);
}

// NewP()
NameSet* NameSet::NewP(void) const {
  NameSet* res = new NameSet();
  res->mpSymbolTable=mpSymbolTable;
  return res;
}

// NewN()
NameSet NameSet::NewN(void) const {
  NameSet res;
  res.mpSymbolTable=mpSymbolTable;
  return res;
}

// SymbolTablep()
SymbolTable* NameSet::SymbolTablep(void) const {
  return mpSymbolTable;
}

// SymbolTablep(pSymTab)
void NameSet::SymbolTablep(SymbolTable* pSymTab) {
  FD_DC("NameSet(" << this << ")::SymbolTablep(" << pSymTab << ")");
  if(!Empty()) {
    Clear();
  } 
  mpSymbolTable=pSymTab;
}

// DoWrite(tw, rLabel)
void NameSet::DoWrite(TokenWriter& tw, const std::string& rLabel, const Type* pContext) const {
  TaNameSet<AttributeVoid> vcopy= *this;
  vcopy.Write(tw,rLabel,pContext);
}

// DoDWrite()
void NameSet::DoDWrite(TokenWriter& tw, const std::string& rLabel, const Type* pContext) const {
  TaNameSet<AttributeVoid> vcopy= *this;
  vcopy.DWrite(tw,rLabel,pContext);
}

// DoRead(rTr, rLabel, pContext)
void NameSet::DoRead(TokenReader& rTr, const std::string& rLabel, const Type* pContext) {
  FD_DC("NameSet(" << this << ")::DoRead(tr," << rLabel << ", " << pContext << 
    ") with symtab " << mpSymbolTable);
  TaNameSet<AttributeVoid> vcopy;
  vcopy.SymbolTablep(mpSymbolTable);
  vcopy.DoRead(rTr,rLabel,pContext);
  *this=vcopy;
}


// Insert(index)
bool NameSet::Insert(Idx index) { 
#ifdef FAUDES_CHECKED
  if(!mpSymbolTable->Exists(index)) {
    std::stringstream errstr;
    errstr << "index " << index << " not known to symboltable" << std::endl;
    throw Exception("NameSet::Insert", errstr.str(), 65);
  }
#endif
  return IndexSet::Insert(index);
}

// Insert(rName)
Idx NameSet::Insert(const std::string& rName) {
  FD_DC("NameSet(" << this << ")::Insert(" << rName <<")");
  Idx index = mpSymbolTable->InsEntry(rName);
  IndexSet::Insert(index);
  return index;
}

// InsertSet(set)
void NameSet::InsertSet(const NameSet& rOtherSet) {
  FD_DC("NameSet(" << this << ")::InsertSet(" << rOtherSet.ToString() << ")");
#ifdef FAUDES_CHECKED
  if(rOtherSet.mpSymbolTable!=mpSymbolTable) {
    std::stringstream errstr;
    errstr << "symboltable mismatch aka not implemented" << std::endl;
    throw Exception("NameSet::InsertSet", errstr.str(), 67);
  }
#endif
  IndexSet::InsertSet(rOtherSet);
}
		
// Erase(index)
bool NameSet::Erase(Idx index) {
  FD_DC("NameSet(" << this << ")::Erase(" << index <<")");
  return IndexSet::Erase(index);
}

// Erase(rName)
bool NameSet::Erase(const std::string& rName) {
  FD_DC("NameSet(" << this << ")::Erase(" << rName <<")");
  Idx index = mpSymbolTable->Index(rName);
#ifdef FAUDES_CHECKED
  if (index == 0) {
    std::stringstream errstr;
    errstr << "name \"" << rName << "\" not found in NameSet" << std::endl;
    throw Exception("NameSet::Erase", errstr.str(), 66);
  }
#endif
  return IndexSet::Erase(index);
}

// Erase(pos)
NameSet::Iterator NameSet::Erase(const Iterator& pos) {
  return IndexSet::Erase(pos);
}

// EraseSet(set)
void NameSet::EraseSet(const NameSet& rOtherSet) {
#ifdef FAUDES_CHECKED
  if(rOtherSet.mpSymbolTable!=mpSymbolTable) {
    std::stringstream errstr;
    errstr << "symboltable mismatch aka not implemented" << std::endl;
    throw Exception("NameSet::EraseSet", errstr.str(), 67);
  }
#endif
  IndexSet::EraseSet(rOtherSet);
}

// SymbolicName(index)
std::string NameSet::SymbolicName(Idx index) const {
  return mpSymbolTable->Symbol(index);
}

// SymbolicName(index, name)
void NameSet::SymbolicName(Idx index, const std::string& rName) {
  FD_DC("NameSet(" << this << ")::SymbolicName(" << index << ", " << rName <<")");
#ifdef FAUDES_CHECKED
  if (! Exists(index)) {
    std::stringstream errstr;
    errstr << "index " << index << " not in this set" << std::endl;
    throw Exception("NameSet::SymbolicName", errstr.str(), 60);
  }
#endif
  mpSymbolTable->SetEntry(index, rName);
}

// SymbolicName(name, name)
void NameSet::SymbolicName(const std::string& rName, 
				  const std::string& rNewName) {
  FD_DC("NameSet(" << this << ")::SymbolicName(" << rName << ", " 
	<< rNewName <<")");
#ifdef FAUDES_CHECKED
  if (! Exists(rName)) {
    std::stringstream errstr;
    errstr << "name \"" << rName << "\" not found in NameSet" << std::endl;
    throw Exception("NameSet::Symbolic", errstr.str(), 66);
  }
#endif
  mpSymbolTable->SetEntry(Index(rName), rNewName);
}

// Index(rName)
Idx NameSet::Index(const std::string& rName) const {
  return mpSymbolTable->Index(rName);
}

// Exists(index)
bool NameSet::Exists(Idx index) const {
  return IndexSet::Exists(index);
}

// Exists(rName)
bool NameSet::Exists(const std::string& rName) const {
  return IndexSet::Exists(mpSymbolTable->Index(rName)) ;
}

// Find(index) const
NameSet::Iterator NameSet::Find(Idx index) const {
  return IndexSet::Find(index);
}

// Find(rName) const
NameSet::Iterator NameSet::Find(const std::string& rName) const {
  return IndexSet::Find(mpSymbolTable->Index(rName));
}


// operator +
NameSet NameSet::operator + (const NameSet& rOtherSet) const {
  FD_DC("NameSet(" << this << ")::operator + (" << &rOtherSet << ")");
#ifdef FAUDES_CHECKED
  if(rOtherSet.mpSymbolTable!=mpSymbolTable) {
    std::stringstream errstr;
    errstr << "symboltable mismazch aka not implemented" << std::endl;
    throw Exception("NameSet::Operator+", errstr.str(), 67);
  }
#endif
  NameSet res= NewN();
  res.Assign( IndexSet::operator + (rOtherSet) );
  return res;
}

// operator -
NameSet NameSet::operator - (const NameSet& rOtherSet) const {
  FD_DC("NameSet(" << this << ")::operator - (" << &rOtherSet << ")");
#ifdef FAUDES_CHECKED
  if(rOtherSet.mpSymbolTable!=mpSymbolTable) {
    std::stringstream errstr;
    errstr << "symboltable mismazch aka not implemented" << std::endl;
    throw Exception("NameSet::Operator-", errstr.str(), 67);
  }
#endif
  NameSet res= NewN();
  res.Assign( IndexSet::operator - (rOtherSet) );
  return res;
}

// operator *
NameSet NameSet::operator * (const NameSet& rOtherSet) const {
  FD_DC("NameSet(" << this << ")::operator * (" << &rOtherSet << ")");
#ifdef FAUDES_CHECKED
  if(rOtherSet.mpSymbolTable!=mpSymbolTable) {
    std::stringstream errstr;
    errstr << "symboltable mismazch aka not implemented" << std::endl;
    throw Exception("NameSet::Operator*", errstr.str(), 67);
  }
#endif
  NameSet res= NewN();
  res.Assign( IndexSet::operator * (rOtherSet) );
  return res;
}


// operator ==
bool NameSet::operator == (const NameSet& rOtherSet) const {
#ifdef FAUDES_CHECKED
  if(rOtherSet.mpSymbolTable!=mpSymbolTable) {
    std::stringstream errstr;
    errstr << "symboltable mismazch aka not implemented" << std::endl;
    throw Exception("NameSet::Operator==", errstr.str(), 67);
  }
#endif
  return IndexSet::operator == (rOtherSet);
}

// operator !=
bool NameSet::operator != (const NameSet& rOtherSet) const {
#ifdef FAUDES_CHECKED
  if(rOtherSet.mpSymbolTable!=mpSymbolTable) {
    std::stringstream errstr;
    errstr << "symboltable mismazch aka not implemented" << std::endl;
    throw Exception("NameSet::Operator!=", errstr.str(), 67);
  }
#endif
  return IndexSet::operator != (rOtherSet);
}

// operator <=
bool NameSet::operator <= (const NameSet& rOtherSet) const {
#ifdef FAUDES_CHECKED
  if(rOtherSet.mpSymbolTable!=mpSymbolTable) {
    std::stringstream errstr;
    errstr << "symboltable mismazch aka not implemented" << std::endl;
    throw Exception("NameSet::Operator<=", errstr.str(), 67);
  }
#endif
  return IndexSet::operator <= (rOtherSet);
}

// operator >=
bool NameSet::operator >= (const NameSet& rOtherSet) const {
#ifdef FAUDES_CHECKED
  if(rOtherSet.mpSymbolTable!=mpSymbolTable) {
    std::stringstream errstr;
    errstr << "symboltable mismazch aka not implemented" << std::endl;
    throw Exception("NameSet::Operator>=", errstr.str(), 67);
  }
#endif
  return IndexSet::operator >= (rOtherSet);
}

// Str(index)
std::string NameSet::Str(Idx index) const {
  return mpSymbolTable->Symbol(index)+"["+faudes::ToStringInteger(index)+"]";
}



} // namespace faudes


